#!/bin/env python

import base64
import gc
import json
import logging
import math
import random
import time
import traceback
from copy import deepcopy
from typing import Optional, Any

import numpy
import peft.peft_model
#import pandas as pd
import torch
#import torch.multiprocessing as mp
#import torch.nn
#import torch.nn.functional as F
#from tokenizers import AddedToken
from transformers import (AutoTokenizer, GPT2LMHeadModel)

from llm_attacks_bishopfox.jailbreak_detection.jailbreak_detection import get_default_negative_test_strings
from llm_attacks_bishopfox.jailbreak_detection.jailbreak_detection import get_default_positive_test_strings

logger = logging.getLogger(__name__)

# Dynamically import all of the transformers "ForCausalLM" classes
# To avoid either tedious maintenance of a manual list 
# or the very overly-broad former "from transformers import *" statement
# BEGIN: based loosely on https://julienharbulot.com/python-dynamical-import.html
from importlib import import_module
from inspect import isclass
from pathlib import Path
from pkgutil import iter_modules
import transformers

# workaround for the following issue in Transformers 4.46.2:
# Error while examining Transformers attribute 'DetrImageProcessorFast' during dynamic import of all ForCausalLM classes: Failed to import transformers.models.detr.image_processing_detr_fast because of the following error (look up to see its traceback):
# cannot import name 'pil_torch_interpolation_mapping' from 'transformers.image_utils' (/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/image_utils.py)
# Traceback (most recent call last):
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/utils/import_utils.py", line 1778, in _get_module
    # return importlib.import_module("." + module_name, self.__name__)
           # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  # File "/usr/lib/python3.11/importlib/__init__.py", line 126, in import_module
    # return _bootstrap._gcd_import(name[level:], package, level)
           # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  # File "<frozen importlib._bootstrap>", line 1206, in _gcd_import
  # File "<frozen importlib._bootstrap>", line 1178, in _find_and_load
  # File "<frozen importlib._bootstrap>", line 1149, in _find_and_load_unlocked
  # File "<frozen importlib._bootstrap>", line 690, in _load_unlocked
  # File "<frozen importlib._bootstrap_external>", line 940, in exec_module
  # File "<frozen importlib._bootstrap>", line 241, in _call_with_frames_removed
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/models/detr/image_processing_detr_fast.py", line 30, in <module>
    # from ...image_utils import (
# ImportError: cannot import name 'pil_torch_interpolation_mapping' from 'transformers.image_utils' (/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/image_utils.py)

# The above exception was the direct cause of the following exception:

# Traceback (most recent call last):
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/llm_attacks_bishopfox/base/attack_manager.py", line 41, in <module>
    # attribute = getattr(transformers, attribute_name)
                # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/utils/import_utils.py", line 1767, in __getattr__
    # value = getattr(module, name)
            # ^^^^^^^^^^^^^^^^^^^^^
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/utils/import_utils.py", line 1766, in __getattr__
    # module = self._get_module(self._class_to_module[name])
             # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  # File "/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/utils/import_utils.py", line 1780, in _get_module
    # raise RuntimeError(
# RuntimeError: Failed to import transformers.models.detr.image_processing_detr_fast because of the following error (look up to see its traceback):
# cannot import name 'pil_torch_interpolation_mapping' from 'transformers.image_utils' (/mnt/md0/Machine_Learning/test_build-04/lib/python3.11/site-packages/transformers/image_utils.py)

ignored_problem_attributes = [ "DetrImageProcessorFast" ]

for attribute_name in dir(transformers):
    if attribute_name not in ignored_problem_attributes:
        try:
            attribute = getattr(transformers, attribute_name)
            if isclass(attribute):
                #import pdb; pdb.Pdb(nosigint=True).set_trace()
                # Add the class to this package's variables
                if hasattr(attribute, "__name__"):
                    if "ForCausalLM" in attribute.__name__:
                        #logger.debug(f"importing {attribute.__name__} from transformers")
                        globals()[attribute_name] = attribute
        except Exception as e:
            logger.warning(f"Error while examining Transformers attribute '{attribute_name}' during dynamic import of all ForCausalLM classes: {e}\n{traceback.format_exc()}")

# END: based loosely on https://julienharbulot.com/python-dynamical-import.html

class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        if isinstance(obj, numpy.floating):
            return float(obj)
        if isinstance(obj, numpy.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class EmbeddingLayerNotFoundException(Exception):
    pass

def is_phi_1_to_3_model(model):
    if isinstance(model, Phi3ForCausalLM):
        return True
    if isinstance(model, PhiForCausalLM):
        return True
    # hack because of the path to the Phi3 class being weird
    if "Phi3ForCausalLM" in f"{type(model)}":
        return True
    if "PhiForCausalLM" in f"{type(model)}":
        return True
    return False

def get_embedding_layer(attack_state):
    use_default_logic = False
    result = None
    try:
        if isinstance(attack_state.model, BartForCausalLM):
            result = attack_state.model.model.decoder.get_input_embeddings()
        if result is None and (isinstance(attack_state.model, BigBirdPegasusForCausalLM) or isinstance(attack_state.model, PegasusForCausalLM)):
            result = attack_state.model.model.decoder.embed_tokens
        if result is None and isinstance(attack_state.model, BlenderbotForCausalLM):
            result = attack_state.model.model.decoder.embed_tokens
        if result is None and isinstance(attack_state.model, FalconForCausalLM):
            result = attack_state.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, FalconMambaForCausalLM):
            result = attack_state.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, GemmaForCausalLM):
            result = attack_state.model.base_model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, Gemma2ForCausalLM):
            result = attack_state.model.base_model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, GPT2LMHeadModel):
            result = attack_state.model.transformer.wte
        if result is None and isinstance(attack_state.model, GPTJForCausalLM):
            result = attack_state.model.transformer.wte
        if result is None and isinstance(attack_state.model, GPTNeoForCausalLM):
            result = attack_state.model.base_model.wte
        if result is None and isinstance(attack_state.model, GPTNeoXForCausalLM):
            result = attack_state.model.base_model.embed_in
        if result is None and isinstance(attack_state.model, LlamaForCausalLM):
            if hasattr(attack_state.model, "model"):
                if hasattr(attack_state.model.model, "get_input_embeddings"):
                    result = attack_state.model.model.get_input_embeddings()
                if result is None and hasattr(attack_state.model.model, "embed_tokens"):
                    result = attack_state.model.model.embed_tokens
            if result is None and hasattr(attack_state.model, "embed_tokens"):
                result = attack_state.model.embed_tokens
            if result is None and hasattr(attack_state.model, "get_input_embeddings"):
                result = attack_state.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, MambaForCausalLM):
            result = attack_state.model.get_input_embeddings()
        #if result is None and isinstance(attack_state.model, MosaicGPT):
        if result is None and attack_state.model_type_name == "MosaicGPT":
            result = attack_state.model.transformer.wte
        if result is None and isinstance(attack_state.model, MptForCausalLM):
            result = attack_state.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, OPTForCausalLM):
            result = attack_state.model.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, peft.peft_model.PeftModelForCausalLM):
            use_default_logic = True
        if result is None and is_phi_1_to_3_model(attack_state.model):
            result = attack_state.model.model.embed_tokens
        if result is None and isinstance(attack_state.model, RobertaForCausalLM):
            result = attack_state.model.get_input_embeddings()
        # for Qwen 1.8B, maybe other oddball Qwen variations
        if result is None and "QWenLMHeadModel" in attack_state.model_type_name:
            result = attack_state.model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, Qwen2ForCausalLM):
            result = attack_state.model.base_model.get_input_embeddings()
        if result is None and isinstance(attack_state.model, StableLmForCausalLM):
            result = attack_state.model.base_model.embed_tokens
    except Exception as e:
        logger.error(f"Exception thrown when attempting to use model-class-specific logic: {e}\n{traceback.format_exc()}")
        result = None
    if result is not None:
        return result
    
    result = None
    result_name = None
    if attack_state.log_manager.get_lowest_log_level() <= logging.DEBUG:
        logger.debug(f"Unrecognized model type '{type(attack_state.model).__name__}' - using default/fallback method for obtaining input embeddings.")
    
    if result is None and hasattr(attack_state.model, "get_input_embeddings"):
        result_name = "attack_state.model.get_input_embeddings()"
        try:
            result = attack_state.model.get_input_embeddings()                
        except Exception as e:
            logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
            result = None
            result_name = None
    if result is None and hasattr(attack_state.model, "model"):
        if hasattr(attack_state.model.model, "get_input_embeddings"):
            result_name = "attack_state.model.model.get_input_embeddings()"
            try:
                result = attack_state.model.model.get_input_embeddings()
            except Exception as e:
                logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
                result = None
                result_name = None
        if result is None and hasattr(attack_state.model.model, "embed_tokens"):
            result_name = "attack_state.model.model.embed_tokens"
            try:
                result = attack_state.model.model.embed_tokens
            except Exception as e:
                logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
                result = None
                result_name = None
    if result is None and hasattr(attack_state.model, "embed_tokens"):            
        result_name = "attack_state.model.embed_tokens"
        try:
            result = attack_state.model.embed_tokens
        except Exception as e:
            logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
            result = None
            result_name = None
    if result is None and hasattr(attack_state.model, "transformer"):
        if hasattr(attack_state.model.transformer, "wte"):
            result_name = "attack_state.model.transformer.wte"
            try:
                result = attack_state.model.transformer.wte
            except Exception as e:
                logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
                result = None
                result_name = None
    if result is None and hasattr(attack_state.model, "base_model"):
        if hasattr(attack_state.model.base_model, "wte"):
            result_name = "attack_state.model.base_model.wte"
            try:
                result = attack_state.model.base_model.wte
            except Exception as e:
                logger.error(f"Exception thrown when attempting to use {result_name}: {e}\n{traceback.format_exc()}")
                result = None
                result_name = None
    if result is not None:
        if not use_default_logic:
            logger.warning(f"unrecognized model type {type(attack_state.model)} - using {result_name} as embedding layer - this may cause unexpected behaviour.")
        return result
    
    result_message = f"Error: unrecognized model type {type(attack_state.model)} "
    if use_default_logic:
        result_message = f"Error: model type {type(attack_state.model)} "

    result_message += "did not have an input embedding property or method in any of the default locations Broken Hill is configured to search for. Processing cannot continue."
    
    raise EmbeddingLayerNotFoundException(result_message)

def get_embedding_matrix(attack_state):
    embedding_layer = get_embedding_layer(attack_state)
    result = embedding_layer.weight
 
    if attack_state.log_manager.get_lowest_log_level() <= logging.DEBUG:
        logger.debug(f"result = {result}")
    # Some models return a function that returns the weight values instead of the 
    # weight values themselves. I assume this is because of missing () in the model
    # code, but regardless, this works around that problem
    if callable(result):
        result = result()
        if attack_state.log_manager.get_lowest_log_level() <= logging.DEBUG:
            logger.debug(f"result after calling original result = {result}")
        if not isinstance(result, torch.nn.Parameter):
            try:
                result = torch.nn.Parameter(result)
            except Exception as e:
                result = torch.nn.Parameter(result, requires_grad = False)
                logger.warning(f"returning the embedding matrix with requires_grad = False")
            if attack_state.log_manager.get_lowest_log_level() <= logging.DEBUG:
                logger.debug(f"result after conversion to Parameter = {result}")
    return result

def get_embeddings(attack_state, input_ids):
    embedding_layer = get_embedding_layer(attack_state)
    # TKTK: validate that setting dtype = attack_state.model.dtype explicitly doesn't change/break anything.
    result = embedding_layer(input_ids)
    if result.dtype != attack_state.model.dtype:
        logger.warning(f"converting embedding layer from {result.dtype} to {attack_state.model.dtype}")
        result = result.to(dtype = attack_state.model.dtype)
    return result
    # return_half = False

    # if isinstance(attack_state.model, GPTJForCausalLM) or isinstance(model, GPT2LMHeadModel):
        # return_half = True
    # elif isinstance(attack_state.model, GPTNeoForCausalLM):
        # return_half = True
    # elif isinstance(attack_state.model, GPTNeoXForCausalLM):
        # return_half = True
    
    # #logger.debug(f"result properties: {result}") 
    
    # if return_half:
        # logger.warning(f"returning .half() variation of embedding layer")
        # result = result.half()
        # #logger.debug(f"result properties after .half(): {result}")
    
    # return result

# This list is hardcoded so that results can be deterministic
# 2024-09-26 - 16384 random seeds should be enough for anyone 
# moved to the end of the file because it's so enormous now
def get_random_seed_list_for_comparisons():
    return [ 0x56, 0xea, 0x7b, 0x6d, 0xc3, 0x71, 0x20, 0x31, 0x51, 0x79, 0xae, 0x7c, 0xf6, 0x92, 0xc2, 0x12, 0x93, 0x4c, 0x7e, 0x32, 0x63, 0x4a, 
        0xac, 0x73, 0x9a, 0xc7, 0x69, 0x98, 0x89, 0xe1, 0x2c, 0xe7, 0x5a, 0xbd, 0x45, 0x70, 0x0d, 0x34, 0x24, 0xe6, 0x65, 0xbc, 0x50, 0x03, 
        0x7f, 0x28, 0x7a, 0x48, 0x67, 0xfd, 0x42, 0x59, 0xc4, 0x97, 0x3a, 0x3d, 0x83, 0xf5, 0xf2, 0xdf, 0xd7, 0x3f, 0xa9, 0x86, 0x21, 0x68, 
        0x94, 0x1a, 0x02, 0x23, 0x38, 0x5c, 0x8a, 0x17, 0x6a, 0xf9, 0xd8, 0xd2, 0x88, 0xa7, 0x2e, 0x00, 0x37, 0x41, 0x8f, 0xcc, 0x90, 0xf7, 
        0x8e, 0x4d, 0xba, 0xd3, 0x36, 0xe3, 0xb4, 0x8d, 0x4f, 0x29, 0xf4, 0x87, 0x3c, 0x58, 0x57, 0x66, 0x10, 0x9f, 0xa6, 0x75, 0x9c, 0x81, 
        0x09, 0xb6, 0xa8, 0x76, 0xe4, 0xbe, 0x01, 0xef, 0x07, 0x85, 0xbf, 0x18, 0xa0, 0x3e, 0x2d, 0xa4, 0x6c, 0xc8, 0x74, 0x46, 0x77, 0xfc, 
        0x33, 0x30, 0xb5, 0x44, 0xb8, 0xd4, 0xb9, 0x14, 0xa5, 0x78, 0xd6, 0xed, 0x15, 0x6f, 0x08, 0x7d, 0x3b, 0xc6, 0x5e, 0x0a, 0xdd, 0xe5, 
        0xf3, 0x2a, 0x8c, 0xec, 0xce, 0x1d, 0xb7, 0x52, 0xc0, 0xfb, 0x27, 0x13, 0xcb, 0x43, 0xd5, 0x6e, 0xd1, 0x72, 0x62, 0xad, 0x26, 0x9b, 
        0x2b, 0x6b, 0x84, 0xcf, 0xdc, 0x1f, 0x0c, 0x61, 0x55, 0xb2, 0x35, 0x9e, 0x54, 0x5f, 0xde, 0xca, 0x64, 0x04, 0x91, 0xe0, 0xeb, 0xaa, 
        0x19, 0xf8, 0xbb, 0x40, 0xf1, 0xc5, 0x82, 0x05, 0x99, 0xc1, 0xab, 0xa1, 0xa3, 0xb1, 0x5b, 0x25, 0x0f, 0xd9, 0xcd, 0x0b, 0xb0, 0x2f, 
        0xaf, 0x39, 0xa2, 0xda, 0x22, 0x11, 0xc9, 0xdb, 0x4b, 0x53, 0x1b, 0x16, 0xd0, 0x06, 0xfa, 0x80, 0x60, 0x95, 0xb3, 0xe2, 0x5d, 0x1c, 
        0xfe, 0xf0, 0xff, 0x47, 0x0e, 0x49, 0x1e, 0xee, 0x8b, 0x9d, 0xe8, 0x4e, 0xe9, 0x96, 0x30EB, 0x2FD5, 0x245E, 0x30F4, 0x2B73, 0x37F9, 
        0x0636, 0x2C7E, 0x2965, 0x1842, 0x03D7, 0x0482, 0x2DA4, 0x0A06, 0x1ED6, 0x05BB, 0x038A, 0x3F76, 0x22EE, 0x08B1, 0x2F43, 0x3747, 
        0x3AF1, 0x2C1B, 0x1BD6, 0x0624, 0x37BB, 0x0296, 0x027B, 0x0E52, 0x2C9E, 0x1D82, 0x03C5, 0x2E6E, 0x02ED, 0x1A61, 0x2C38, 0x14A9, 
        0x10AB, 0x2DEC, 0x3246, 0x1F2E, 0x3FE7, 0x13B0, 0x1A1C, 0x3C33, 0x2C58, 0x0EE6, 0x112F, 0x354A, 0x0E2C, 0x0CAB, 0x015C, 0x1F15, 
        0x2A38, 0x31A4, 0x3779, 0x2E0E, 0x3CB3, 0x37EB, 0x1E96, 0x1A74, 0x0E58, 0x2E27, 0x31A7, 0x390F, 0x1583, 0x0184, 0x0817, 0x3852, 
        0x343F, 0x1131, 0x3CB8, 0x1070, 0x0DDA, 0x280B, 0x2F93, 0x3F78, 0x0E89, 0x2DB8, 0x35AE, 0x1D09, 0x0CD3, 0x03A0, 0x2714, 0x2120, 
        0x3FF7, 0x2DD8, 0x1E7F, 0x3722, 0x0F36, 0x35A8, 0x21A7, 0x2688, 0x2071, 0x1B12, 0x3B0A, 0x2CC6, 0x36EA, 0x2DBA, 0x2DD9, 0x15AF, 
        0x1877, 0x0D65, 0x3683, 0x2A2E, 0x3ADA, 0x12C0, 0x39B8, 0x21BB, 0x32B4, 0x3F99, 0x3963, 0x2672, 0x3D5D, 0x1BE3, 0x0384, 0x1D08, 
        0x17F4, 0x3E24, 0x385D, 0x0CE6, 0x219A, 0x15C5, 0x3518, 0x3154, 0x2AD7, 0x2DD5, 0x1826, 0x0394, 0x1344, 0x0211, 0x0257, 0x2EE9, 
        0x3570, 0x1FE4, 0x2D29, 0x21B8, 0x30E2, 0x0117, 0x1BE2, 0x3C76, 0x3FD3, 0x1210, 0x2266, 0x37FE, 0x030E, 0x143A, 0x3318, 0x0440, 
        0x3750, 0x15EC, 0x171C, 0x174A, 0x23E1, 0x14C5, 0x1092, 0x0996, 0x18F7, 0x02FA, 0x1006, 0x07E9, 0x039B, 0x326F, 0x19E4, 0x2EB5, 
        0x2CE3, 0x070E, 0x1BFA, 0x0A39, 0x331D, 0x2DB3, 0x3753, 0x2EC3, 0x3CA7, 0x247E, 0x1E8B, 0x2541, 0x3B66, 0x02B7, 0x164A, 0x2166, 
        0x10D7, 0x382F, 0x327B, 0x2021, 0x131D, 0x2FDD, 0x150A, 0x21DF, 0x025D, 0x251F, 0x1339, 0x132A, 0x2A4C, 0x38D7, 0x0790, 0x0BF5, 
        0x01CD, 0x3CAD, 0x020D, 0x0D2A, 0x32C4, 0x1FB9, 0x28B7, 0x244C, 0x1D63, 0x24D3, 0x07BD, 0x19B1, 0x3FD1, 0x07D7, 0x3D4C, 0x06FB, 
        0x31F6, 0x3CB0, 0x0188, 0x09EB, 0x0E22, 0x273D, 0x1367, 0x1EA4, 0x37F4, 0x0338, 0x1492, 0x3894, 0x1991, 0x2095, 0x0CE3, 0x343B, 
        0x03EA, 0x30B4, 0x0848, 0x20B8, 0x36DF, 0x3D83, 0x351E, 0x32E1, 0x1390, 0x1D0B, 0x1E48, 0x364B, 0x3ECF, 0x37B7, 0x3D72, 0x0A2F, 
        0x1A70, 0x0CB3, 0x1F5D, 0x100F, 0x2674, 0x31CE, 0x1DF1, 0x369C, 0x0368, 0x1488, 0x14EB, 0x3D71, 0x15E1, 0x2375, 0x15C4, 0x2403, 
        0x397A, 0x17FA, 0x0928, 0x0293, 0x2C75, 0x1F95, 0x0471, 0x1010, 0x1C40, 0x33EE, 0x274B, 0x3ED2, 0x0C23, 0x3C2E, 0x14BB, 0x0431, 
        0x2D7A, 0x0643, 0x1E1B, 0x3B01, 0x19A9, 0x16EF, 0x3EA4, 0x0691, 0x1741, 0x1D35, 0x03B9, 0x3A5A, 0x215F, 0x12AD, 0x2934, 0x04B8, 
        0x24E1, 0x2D13, 0x1560, 0x1E9E, 0x04C0, 0x3A11, 0x11A0, 0x082C, 0x32F4, 0x3F40, 0x2B49, 0x1D24, 0x3459, 0x01ED, 0x1662, 0x1DDC, 
        0x2127, 0x13E2, 0x064E, 0x2C85, 0x31A8, 0x02F5, 0x22B5, 0x1B39, 0x0CE2, 0x3EA9, 0x1B18, 0x37D8, 0x0DB8, 0x16FE, 0x13A8, 0x3C43, 
        0x2758, 0x11B2, 0x0B59, 0x1FF3, 0x355D, 0x3951, 0x3B68, 0x344A, 0x176A, 0x16BC, 0x0D12, 0x044E, 0x3538, 0x23BD, 0x14DD, 0x04D0, 
        0x3F08, 0x16FC, 0x1A7C, 0x0DEA, 0x3284, 0x0F50, 0x2B7C, 0x1508, 0x0295, 0x04ED, 0x0118, 0x2832, 0x1CF0, 0x0BD8, 0x2CBD, 0x144A, 
        0x2EBB, 0x360E, 0x0CDD, 0x08DD, 0x1DF4, 0x13E7, 0x1BA8, 0x3E8A, 0x07F5, 0x27F5, 0x0E74, 0x20B2, 0x1AB8, 0x3982, 0x0876, 0x3F69, 
        0x2590, 0x2B0A, 0x1BE6, 0x26F8, 0x2F3C, 0x2E91, 0x2083, 0x313C, 0x217A, 0x0389, 0x2D6B, 0x1FDA, 0x27BE, 0x212E, 0x183B, 0x1094, 
        0x2F71, 0x168D, 0x04E1, 0x0C31, 0x199B, 0x3A90, 0x01CC, 0x2B47, 0x20D3, 0x05AA, 0x317B, 0x0BD0, 0x1F43, 0x2D21, 0x17DC, 0x3971, 
        0x1289, 0x2D8A, 0x2D22, 0x157E, 0x03FA, 0x147B, 0x10E2, 0x3500, 0x2F75, 0x1928, 0x020B, 0x1395, 0x1DB0, 0x03EB, 0x28E7, 0x2ABF, 
        0x2A0D, 0x3696, 0x234A, 0x0BFB, 0x0973, 0x240A, 0x39E1, 0x0478, 0x3C97, 0x032B, 0x1A42, 0x292E, 0x36FA, 0x35C6, 0x06C6, 0x0EC9, 
        0x1F62, 0x3740, 0x3A62, 0x1CCA, 0x34D4, 0x2CAD, 0x38C0, 0x326B, 0x2AE8, 0x1ED0, 0x3209, 0x338C, 0x01F5, 0x25FE, 0x3F13, 0x2256, 
        0x3FC8, 0x36E9, 0x2585, 0x3EC9, 0x194B, 0x2939, 0x0DC0, 0x13A7, 0x27AC, 0x2022, 0x3B9B, 0x37BD, 0x187E, 0x24E0, 0x25EA, 0x3067, 
        0x1F67, 0x1AE6, 0x2B80, 0x33C6, 0x3F9F, 0x0ACE, 0x2A0C, 0x1FE5, 0x0B50, 0x0414, 0x2FE0, 0x3B87, 0x3C73, 0x1B44, 0x298E, 0x02FD, 
        0x2135, 0x1DA3, 0x1A0F, 0x13C3, 0x3DB7, 0x1C91, 0x1D7C, 0x0429, 0x3787, 0x2826, 0x2D2F, 0x1910, 0x1564, 0x0F6B, 0x097A, 0x39BA, 
        0x02DB, 0x36DC, 0x0456, 0x1273, 0x2059, 0x3E33, 0x2C87, 0x1DC4, 0x1A2E, 0x06A1, 0x2E9E, 0x206F, 0x242D, 0x159F, 0x37BF, 0x0623, 
        0x3BEE, 0x06E6, 0x09CD, 0x1B6F, 0x0BA7, 0x2296, 0x095B, 0x358A, 0x1F1E, 0x2D69, 0x07A4, 0x0103, 0x3595, 0x0FB2, 0x15ED, 0x3C65, 
        0x0ED9, 0x37F3, 0x1C75, 0x2DF9, 0x0E4E, 0x016A, 0x1B92, 0x34E7, 0x02E3, 0x18C9, 0x3ECC, 0x0FFB, 0x129B, 0x0FE2, 0x1EBC, 0x18C7, 
        0x0567, 0x239E, 0x06B3, 0x1427, 0x2B9B, 0x190B, 0x38EC, 0x0C94, 0x149D, 0x0CCB, 0x0133, 0x21D3, 0x3D55, 0x12CE, 0x381D, 0x2954, 
        0x04DA, 0x0CB6, 0x256B, 0x3205, 0x3692, 0x35B1, 0x2DEE, 0x2818, 0x0E0E, 0x0619, 0x22D7, 0x2198, 0x0ECE, 0x2169, 0x15AB, 0x1A45, 
        0x03F6, 0x3962, 0x0ACC, 0x1CD4, 0x01AE, 0x2D12, 0x30E9, 0x0E68, 0x0826, 0x1794, 0x0E97, 0x0317, 0x028B, 0x315B, 0x2F8F, 0x0D7C, 
        0x3793, 0x2C5E, 0x0A89, 0x2B48, 0x21C4, 0x3548, 0x01FA, 0x342E, 0x126D, 0x2E09, 0x1FD9, 0x0A58, 0x27FF, 0x3B70, 0x2A1C, 0x2F04, 
        0x2461, 0x3C25, 0x2B60, 0x1295, 0x1C53, 0x2FD9, 0x3206, 0x0E05, 0x29BE, 0x2DF0, 0x24E7, 0x3EC6, 0x25B4, 0x0824, 0x3BB2, 0x3610, 
        0x34FF, 0x3BE2, 0x028F, 0x25C1, 0x3CA2, 0x0B57, 0x35A7, 0x1BF5, 0x139E, 0x0B18, 0x21B2, 0x1128, 0x3410, 0x0AC8, 0x346B, 0x0C62, 
        0x2E24, 0x22C9, 0x1369, 0x25CC, 0x1AB1, 0x18EE, 0x05DA, 0x32B2, 0x24C2, 0x16C3, 0x10B7, 0x34E4, 0x13D2, 0x2A2F, 0x1647, 0x1270, 
        0x1F4D, 0x1155, 0x2EAA, 0x2D89, 0x0D47, 0x280E, 0x3858, 0x3DEA, 0x31DC, 0x3573, 0x1D8E, 0x0565, 0x226C, 0x0EE2, 0x309A, 0x393E, 
        0x30CF, 0x107F, 0x1540, 0x1D91, 0x0AE7, 0x0BAD, 0x1F7C, 0x352E, 0x2BC8, 0x2E2A, 0x2707, 0x190D, 0x20BC, 0x3444, 0x101E, 0x27C6, 
        0x3AAC, 0x24B6, 0x049B, 0x036D, 0x32A0, 0x0C3F, 0x2343, 0x3B78, 0x041B, 0x1356, 0x1F92, 0x10A4, 0x37DC, 0x2D36, 0x21F1, 0x33F9, 
        0x1915, 0x397C, 0x01AC, 0x15C1, 0x0C91, 0x28C1, 0x3CC2, 0x24BE, 0x2C6B, 0x3159, 0x1281, 0x0863, 0x2581, 0x18F4, 0x0B01, 0x1EB0, 
        0x10B8, 0x1381, 0x1A8B, 0x1D7B, 0x0388, 0x35E4, 0x2384, 0x2EFA, 0x20CB, 0x19ED, 0x3D76, 0x08BA, 0x05AD, 0x1665, 0x0CE7, 0x3426, 
        0x21C6, 0x07F8, 0x0AC6, 0x232A, 0x088F, 0x1758, 0x32D1, 0x0A13, 0x14A8, 0x3E56, 0x20C6, 0x1966, 0x12BF, 0x1895, 0x3870, 0x0FA5, 
        0x3C5F, 0x3AFB, 0x38BF, 0x1164, 0x3506, 0x24AB, 0x2F10, 0x1192, 0x0EAC, 0x24AD, 0x24B7, 0x0972, 0x09EC, 0x2F7A, 0x0168, 0x169B, 
        0x0E78, 0x1DF3, 0x0FD9, 0x3DE5, 0x255C, 0x3CC4, 0x19C6, 0x2B8C, 0x3922, 0x145D, 0x2A65, 0x0ED1, 0x042A, 0x1803, 0x09E3, 0x2CB6, 
        0x0B82, 0x06E1, 0x22BF, 0x3ABA, 0x3FBB, 0x263D, 0x3417, 0x3156, 0x02E4, 0x3839, 0x3A7B, 0x3BD2, 0x0E7F, 0x3CF7, 0x16A0, 0x317C, 
        0x375B, 0x2DF3, 0x1FA0, 0x3ED9, 0x1E14, 0x21D0, 0x29D2, 0x3498, 0x2B46, 0x1E2B, 0x21FA, 0x1CE6, 0x28A1, 0x3463, 0x2FA8, 0x3FB1, 
        0x0A8E, 0x0AB8, 0x14FC, 0x3533, 0x08A6, 0x08F1, 0x0A94, 0x013F, 0x20C3, 0x3413, 0x0ED4, 0x1676, 0x1EDF, 0x1B3E, 0x3EA6, 0x1EF6, 
        0x2589, 0x3FB8, 0x3088, 0x0BE5, 0x3F83, 0x12E7, 0x39F2, 0x328D, 0x1729, 0x0217, 0x0C18, 0x1E32, 0x03F2, 0x011A, 0x15DB, 0x0A5E, 
        0x213A, 0x1ABD, 0x23D8, 0x107C, 0x36D7, 0x3BCD, 0x37A2, 0x3932, 0x37D7, 0x12FD, 0x0CF9, 0x29E9, 0x0420, 0x208E, 0x21DD, 0x102E, 
        0x2D2B, 0x0E43, 0x0A99, 0x1047, 0x1D97, 0x1618, 0x2B23, 0x3EFA, 0x10CD, 0x2D5A, 0x0F34, 0x0251, 0x276A, 0x21BA, 0x1CC2, 0x32F0, 
        0x247F, 0x056B, 0x04DD, 0x247B, 0x0664, 0x1DB6, 0x0776, 0x2F08, 0x0265, 0x2F8E, 0x2C32, 0x225F, 0x13E9, 0x1FFA, 0x09DE, 0x3330, 
        0x36FF, 0x17FC, 0x26CC, 0x3161, 0x35A5, 0x1403, 0x39E8, 0x1A48, 0x0D34, 0x2897, 0x2696, 0x01CA, 0x215E, 0x3847, 0x3FD6, 0x237B, 
        0x15FB, 0x1F41, 0x0A81, 0x39C1, 0x04F3, 0x1E23, 0x032C, 0x098E, 0x1FB2, 0x08F0, 0x13BA, 0x3F56, 0x32EA, 0x3DD8, 0x23F3, 0x31CA, 
        0x3879, 0x0488, 0x2844, 0x311E, 0x0F40, 0x05E7, 0x1297, 0x2574, 0x1ECB, 0x1D78, 0x1FA7, 0x34AB, 0x1AE8, 0x31B2, 0x245D, 0x3D6D, 
        0x310D, 0x08EE, 0x07BE, 0x33E4, 0x160F, 0x01EA, 0x0B76, 0x26A2, 0x364E, 0x0581, 0x29EB, 0x30DA, 0x2BE6, 0x0CBA, 0x1B11, 0x0FD5, 
        0x3650, 0x0897, 0x0359, 0x166D, 0x3D00, 0x1AD8, 0x3919, 0x2511, 0x2A0B, 0x0243, 0x34EE, 0x1A4D, 0x0758, 0x3953, 0x2B12, 0x17D4, 
        0x2B45, 0x091E, 0x0846, 0x3848, 0x1A5D, 0x0ABA, 0x0A0B, 0x3E28, 0x0BA3, 0x0F4F, 0x0933, 0x2E8F, 0x165A, 0x32E2, 0x104E, 0x3B26, 
        0x035E, 0x0798, 0x1D56, 0x3A61, 0x1406, 0x3B18, 0x36D1, 0x198A, 0x0ECA, 0x3D6A, 0x0C1E, 0x021F, 0x08A0, 0x1609, 0x0A62, 0x121E, 
        0x3FD2, 0x0280, 0x141F, 0x0D49, 0x11A4, 0x37AB, 0x0901, 0x0936, 0x196A, 0x116A, 0x16EA, 0x0166, 0x24C6, 0x37F8, 0x3D53, 0x201E, 
        0x04FB, 0x3DA5, 0x3423, 0x2CB9, 0x24E8, 0x1BAF, 0x3C11, 0x2731, 0x3A66, 0x2E36, 0x0CFE, 0x1BE4, 0x3F47, 0x1AC8, 0x2940, 0x155B, 
        0x3913, 0x07CC, 0x0546, 0x28D2, 0x05B7, 0x11DE, 0x1447, 0x2CCB, 0x0602, 0x3DA6, 0x2032, 0x3DB8, 0x1BE1, 0x052A, 0x36C0, 0x287C, 
        0x065B, 0x3CE5, 0x28FE, 0x2CE1, 0x04BE, 0x0B07, 0x19A6, 0x0286, 0x3E3C, 0x1B63, 0x0711, 0x28BF, 0x04A8, 0x09C6, 0x23E3, 0x3BA4, 
        0x104D, 0x298B, 0x2999, 0x2382, 0x2000, 0x10B6, 0x383B, 0x0B96, 0x3990, 0x3251, 0x2C69, 0x1D4C, 0x0686, 0x1AB0, 0x3F3D, 0x2DDE, 
        0x30BD, 0x25F8, 0x0B91, 0x12E3, 0x15D7, 0x14E5, 0x0361, 0x04D5, 0x3028, 0x27A7, 0x19E5, 0x0753, 0x2B75, 0x30FA, 0x2DBE, 0x0696, 
        0x37BE, 0x3A56, 0x279B, 0x2A20, 0x3814, 0x2ECF, 0x2154, 0x2F46, 0x1CBF, 0x34A2, 0x3357, 0x32C2, 0x3674, 0x3530, 0x32F5, 0x3DA8, 
        0x17B7, 0x0200, 0x2DC6, 0x38EA, 0x160D, 0x2665, 0x3F22, 0x078F, 0x1A01, 0x1DBD, 0x2EB7, 0x33F2, 0x17E1, 0x25C2, 0x232C, 0x1F2B, 
        0x081D, 0x0B4C, 0x349C, 0x04F9, 0x299A, 0x1072, 0x305D, 0x1967, 0x140D, 0x327F, 0x1366, 0x2F48, 0x1981, 0x07B7, 0x3241, 0x3E27, 
        0x1558, 0x1426, 0x023D, 0x1C77, 0x30FE, 0x2D27, 0x1FC1, 0x060F, 0x0834, 0x257C, 0x2153, 0x0AF3, 0x32AF, 0x2285, 0x08F9, 0x01E4, 
        0x07AB, 0x09FC, 0x210D, 0x0C9D, 0x311A, 0x2F35, 0x1C55, 0x205E, 0x1011, 0x1BCF, 0x3920, 0x3204, 0x0177, 0x28C2, 0x1B59, 0x36AD, 
        0x01A2, 0x2C72, 0x0E5E, 0x3E30, 0x073B, 0x10CC, 0x3C8F, 0x316D, 0x209B, 0x184F, 0x2A95, 0x369A, 0x1341, 0x01AB, 0x1152, 0x2FBF, 
        0x0240, 0x1DAE, 0x13D8, 0x22D5, 0x3914, 0x1768, 0x07B9, 0x18C0, 0x0739, 0x2A13, 0x147A, 0x03D9, 0x047F, 0x04CB, 0x111D, 0x2C37, 
        0x018A, 0x2D77, 0x1ABC, 0x2D24, 0x2BEF, 0x13D9, 0x2E0B, 0x0E4C, 0x2930, 0x1B0D, 0x1EA5, 0x2C93, 0x21CF, 0x2371, 0x2C86, 0x3185, 
        0x1515, 0x05C0, 0x1394, 0x3081, 0x31A5, 0x049F, 0x2A74, 0x16C1, 0x03FC, 0x15C9, 0x0865, 0x275C, 0x2182, 0x1579, 0x0469, 0x0B99, 
        0x1633, 0x30F9, 0x34E5, 0x2635, 0x2970, 0x2ACA, 0x0D24, 0x3E59, 0x380A, 0x2055, 0x3A4D, 0x096E, 0x1C57, 0x3BC2, 0x0909, 0x3F88, 
        0x1F3E, 0x0B66, 0x02B8, 0x3DE3, 0x1CE4, 0x16D5, 0x0A72, 0x0722, 0x3761, 0x16D8, 0x1506, 0x1CD9, 0x37C2, 0x3725, 0x0A14, 0x2A64, 
        0x09C5, 0x22C2, 0x145F, 0x1AE3, 0x082E, 0x1A2B, 0x33EB, 0x2056, 0x1A46, 0x28C5, 0x0528, 0x0D3F, 0x3191, 0x3E84, 0x2C3D, 0x0627, 
        0x1F49, 0x151E, 0x109B, 0x10E0, 0x0BAB, 0x2DE2, 0x339D, 0x17B5, 0x3448, 0x060C, 0x063B, 0x3BDA, 0x20AB, 0x1AEC, 0x1078, 0x2188, 
        0x0A8B, 0x145E, 0x2206, 0x3DE4, 0x0E72, 0x174B, 0x1AE9, 0x0962, 0x18D3, 0x2435, 0x2E0C, 0x2520, 0x385A, 0x224B, 0x2BD4, 0x3363, 
        0x2229, 0x3F67, 0x0D8C, 0x2CD9, 0x01DA, 0x0C9C, 0x37EF, 0x2FA9, 0x12C3, 0x0873, 0x229F, 0x37F0, 0x1E04, 0x1D4B, 0x2363, 0x2369, 
        0x1AA4, 0x0DD8, 0x090E, 0x33D6, 0x25D6, 0x02A6, 0x0486, 0x2E5E, 0x21A1, 0x3E97, 0x1144, 0x0A47, 0x3465, 0x010A, 0x1EC0, 0x0301, 
        0x2CDA, 0x0CDE, 0x3856, 0x28A9, 0x0708, 0x1651, 0x1A9D, 0x1AB3, 0x087A, 0x2378, 0x2F54, 0x38B1, 0x2871, 0x3391, 0x0B83, 0x01E7, 
        0x2649, 0x1DC5, 0x29F3, 0x2CEE, 0x25D7, 0x30D0, 0x38B0, 0x28E0, 0x162A, 0x2F78, 0x022B, 0x03B4, 0x2156, 0x398A, 0x243A, 0x3E51, 
        0x29C9, 0x1C66, 0x3D4B, 0x3C50, 0x13D4, 0x0201, 0x28F2, 0x2F62, 0x0533, 0x30BB, 0x2E84, 0x1C90, 0x299B, 0x2AD1, 0x2A7F, 0x0BAA, 
        0x2CC5, 0x3411, 0x3300, 0x0DA9, 0x0443, 0x1BC0, 0x01E0, 0x0405, 0x2E6D, 0x05F3, 0x2658, 0x304B, 0x2601, 0x12B0, 0x0882, 0x3776, 
        0x2903, 0x17B6, 0x2838, 0x24D5, 0x352B, 0x1747, 0x1229, 0x1649, 0x11B7, 0x14D0, 0x3D40, 0x39FD, 0x1F3B, 0x301B, 0x2F4A, 0x0F76, 
        0x2842, 0x1389, 0x12EF, 0x3345, 0x0E5C, 0x1FB5, 0x138D, 0x34C8, 0x3B64, 0x1EF5, 0x1D49, 0x01F1, 0x0B5F, 0x101A, 0x2345, 0x3A24, 
        0x0F4A, 0x1642, 0x23D7, 0x3DBB, 0x2DB0, 0x1A82, 0x32E9, 0x11FD, 0x2639, 0x0D28, 0x1CB7, 0x0B46, 0x203F, 0x13DB, 0x3F0A, 0x0E4D, 
        0x0831, 0x2925, 0x3D12, 0x32FC, 0x2F0E, 0x2B93, 0x3E2B, 0x3048, 0x3C41, 0x28AA, 0x0BCC, 0x26A4, 0x036E, 0x0CDC, 0x3FFF, 0x3DC1, 
        0x1FA1, 0x0C65, 0x3FA3, 0x344B, 0x2290, 0x283A, 0x0EAE, 0x3806, 0x0946, 0x2C7C, 0x31A9, 0x1D79, 0x0F42, 0x36E0, 0x20B6, 0x3A16, 
        0x0EC8, 0x3AA1, 0x3D1C, 0x084B, 0x3668, 0x0313, 0x26E0, 0x1578, 0x0406, 0x2C16, 0x0F56, 0x25CF, 0x2512, 0x0FFD, 0x2D06, 0x37D2, 
        0x28F6, 0x2894, 0x050A, 0x11BE, 0x3CC9, 0x3CF5, 0x106E, 0x2EA7, 0x02FC, 0x396E, 0x3049, 0x2168, 0x04E9, 0x2E1A, 0x06FD, 0x3841, 
        0x3435, 0x047A, 0x193D, 0x34C4, 0x3F1F, 0x0E29, 0x31AC, 0x1033, 0x23A8, 0x06BF, 0x1B25, 0x2F2D, 0x3B3F, 0x3DCD, 0x2A42, 0x365A, 
        0x3E40, 0x2257, 0x0849, 0x04B4, 0x2207, 0x08DC, 0x229B, 0x0100, 0x309D, 0x0BA4, 0x04B6, 0x0EB4, 0x016F, 0x038D, 0x110A, 0x24B4, 
        0x0613, 0x2910, 0x2B5E, 0x092B, 0x3794, 0x3A72, 0x085E, 0x0A10, 0x1EB5, 0x13CD, 0x0A6F, 0x3667, 0x0563, 0x1E28, 0x02D5, 0x31BA, 
        0x2891, 0x0A77, 0x2B7E, 0x1B03, 0x17D2, 0x1752, 0x084E, 0x2BBB, 0x3933, 0x2593, 0x20A2, 0x126A, 0x2D1F, 0x0EFA, 0x3126, 0x35E6, 
        0x294A, 0x1883, 0x14E3, 0x0472, 0x3FFD, 0x217B, 0x1DF6, 0x05C2, 0x263B, 0x3016, 0x3DF2, 0x3272, 0x0F52, 0x0E35, 0x0409, 0x2D80, 
        0x1E2D, 0x05FE, 0x38C1, 0x0438, 0x1CCC, 0x258C, 0x3179, 0x2815, 0x1775, 0x0EB2, 0x018B, 0x20CF, 0x2EEE, 0x323F, 0x2471, 0x05A4, 
        0x0379, 0x2D73, 0x0CDF, 0x2E8A, 0x33BE, 0x3CF2, 0x0DF9, 0x1A19, 0x1CD7, 0x199D, 0x0AE5, 0x2FBC, 0x1B4C, 0x1A0C, 0x08E4, 0x2209, 
        0x0D5B, 0x1FC3, 0x290F, 0x10B9, 0x01E6, 0x029C, 0x1EDD, 0x3916, 0x3C4A, 0x0C4F, 0x3273, 0x1136, 0x2722, 0x2949, 0x0ACA, 0x054F, 
        0x123F, 0x2F64, 0x3C0C, 0x375F, 0x115A, 0x2969, 0x113A, 0x3649, 0x3620, 0x30DD, 0x2347, 0x2E3B, 0x253A, 0x273E, 0x16F6, 0x12C7, 
        0x13FB, 0x3C4D, 0x2B4B, 0x2F3B, 0x391B, 0x3AB9, 0x1F93, 0x2E2C, 0x2D32, 0x1F88, 0x2EF6, 0x0995, 0x05A7, 0x3F2F, 0x1621, 0x2A04, 
        0x1998, 0x083A, 0x2DB4, 0x02A2, 0x1F59, 0x26D7, 0x1FF8, 0x2E68, 0x1567, 0x32C3, 0x19D1, 0x2EF3, 0x2AFE, 0x1F06, 0x1850, 0x14BE, 
        0x10D1, 0x0C6A, 0x2FA6, 0x1236, 0x22AB, 0x377D, 0x0D15, 0x3287, 0x18BC, 0x1622, 0x1556, 0x0E81, 0x31FD, 0x0C60, 0x2D48, 0x18A3, 
        0x0854, 0x25CE, 0x0F86, 0x26BE, 0x3803, 0x264B, 0x3991, 0x1EA6, 0x3731, 0x33A4, 0x266B, 0x3C09, 0x2AED, 0x36AF, 0x3555, 0x2CD8, 
        0x27E7, 0x3CFE, 0x24BF, 0x2A39, 0x1704, 0x0BFA, 0x302C, 0x01B3, 0x3AAA, 0x2648, 0x148A, 0x11DC, 0x1A1D, 0x0CE5, 0x15A7, 0x37E6, 
        0x0965, 0x294F, 0x12FB, 0x05B2, 0x0C9F, 0x21C2, 0x3DFB, 0x2334, 0x3A4A, 0x10AF, 0x3364, 0x1E0A, 0x1B29, 0x183A, 0x30AE, 0x15FA, 
        0x0763, 0x3ADF, 0x3E25, 0x1C4E, 0x1E7B, 0x2874, 0x316B, 0x338B, 0x3AC4, 0x2A47, 0x2BCD, 0x2C52, 0x0D67, 0x05D8, 0x29F1, 0x1584, 
        0x1511, 0x14D1, 0x387D, 0x1969, 0x3A3B, 0x072A, 0x3651, 0x286B, 0x35FA, 0x1B1D, 0x2F86, 0x0AF6, 0x14A7, 0x2D4C, 0x1E03, 0x27D6, 
        0x09D6, 0x3383, 0x3552, 0x339B, 0x0589, 0x37ED, 0x2E44, 0x0E88, 0x351F, 0x24C8, 0x319C, 0x0F04, 0x38D2, 0x3A9A, 0x2D34, 0x16DF, 
        0x21E4, 0x2547, 0x3E04, 0x047B, 0x04EE, 0x3B1C, 0x054D, 0x3027, 0x2973, 0x0391, 0x0C95, 0x2862, 0x342C, 0x2A6A, 0x137C, 0x3D70, 
        0x0C59, 0x36D6, 0x1A2A, 0x061D, 0x2A45, 0x2863, 0x1222, 0x2E3D, 0x3085, 0x1BDF, 0x289E, 0x19CF, 0x0AD6, 0x22A6, 0x10FF, 0x0141, 
        0x1DA5, 0x2526, 0x3B60, 0x0E00, 0x035A, 0x1674, 0x1EFC, 0x23CC, 0x0C46, 0x202F, 0x1487, 0x0B34, 0x27E3, 0x3D77, 0x2328, 0x2F80, 
        0x3821, 0x2ACC, 0x0E1A, 0x1C48, 0x0868, 0x3662, 0x3138, 0x242A, 0x04B0, 0x3149, 0x32A5, 0x0E4A, 0x3DB6, 0x128B, 0x26DF, 0x29C3, 
        0x1BAC, 0x246E, 0x1CF2, 0x1576, 0x1F8C, 0x335F, 0x313B, 0x3063, 0x2F73, 0x22F8, 0x010D, 0x171F, 0x149B, 0x369F, 0x1030, 0x2238, 
        0x0402, 0x1F96, 0x1B40, 0x02B6, 0x3505, 0x3737, 0x0F79, 0x2536, 0x270A, 0x07E8, 0x076B, 0x0B95, 0x36EC, 0x200C, 0x0DEF, 0x3B12, 
        0x0751, 0x2149, 0x2E73, 0x0803, 0x36C5, 0x10E3, 0x1AA2, 0x1F45, 0x36D5, 0x119C, 0x0997, 0x0754, 0x30AC, 0x3711, 0x16CE, 0x11C2, 
        0x145C, 0x1C83, 0x3745, 0x2B5A, 0x2EC1, 0x36E1, 0x2896, 0x132C, 0x30C2, 0x1FA4, 0x2B0F, 0x3FCA, 0x0F72, 0x23D1, 0x3824, 0x1EE6, 
        0x0C53, 0x2B3F, 0x0FD2, 0x3DB1, 0x3C8D, 0x0666, 0x121A, 0x1824, 0x3698, 0x2AA1, 0x2890, 0x2C7F, 0x312B, 0x1A20, 0x36B9, 0x2E08, 
        0x33A3, 0x1FF6, 0x20EE, 0x2DAD, 0x36A3, 0x2937, 0x051A, 0x3424, 0x2F34, 0x15DE, 0x3590, 0x189B, 0x35DA, 0x282C, 0x1B4E, 0x0249, 
        0x07E5, 0x36E4, 0x1D3F, 0x3334, 0x3A8D, 0x30F2, 0x1880, 0x2C18, 0x1B20, 0x2A52, 0x39CB, 0x2389, 0x3846, 0x3DE6, 0x1302, 0x0D40, 
        0x1E40, 0x38B8, 0x3317, 0x0C74, 0x0127, 0x23F6, 0x2E38, 0x24FE, 0x3972, 0x18B5, 0x397D, 0x30F0, 0x1FB8, 0x3789, 0x21EB, 0x2992, 
        0x0F30, 0x3994, 0x1C46, 0x1109, 0x17C5, 0x2634, 0x39CA, 0x28D1, 0x2929, 0x2507, 0x027A, 0x2E30, 0x2508, 0x16E4, 0x1293, 0x3BE8, 
        0x26AA, 0x38DA, 0x3ED5, 0x088B, 0x2E50, 0x2F06, 0x3EE3, 0x1480, 0x3336, 0x175C, 0x38F6, 0x2AC3, 0x0464, 0x3C45, 0x1C10, 0x1F78, 
        0x258F, 0x1EAF, 0x19AB, 0x17EC, 0x2433, 0x1C8E, 0x054B, 0x02C1, 0x0DC9, 0x091C, 0x17DA, 0x35CB, 0x0496, 0x073C, 0x1E29, 0x2DF2, 
        0x1975, 0x3813, 0x0E86, 0x316C, 0x1C08, 0x3F32, 0x0BDB, 0x196F, 0x02C8, 0x1175, 0x366F, 0x1D96, 0x3688, 0x1C94, 0x1B99, 0x290D, 
        0x112C, 0x32BB, 0x07A8, 0x28ED, 0x2B2A, 0x396A, 0x1AFE, 0x13BF, 0x22E9, 0x2294, 0x3CFD, 0x1268, 0x0397, 0x329A, 0x1626, 0x28AB, 
        0x0822, 0x0539, 0x3145, 0x3EA5, 0x1D14, 0x1BB1, 0x1AAE, 0x20E1, 0x0B2E, 0x09BB, 0x1F9C, 0x3E82, 0x3B5A, 0x0AE1, 0x261C, 0x0F6F, 
        0x2F36, 0x1CBE, 0x1151, 0x0237, 0x048E, 0x13CA, 0x0C7B, 0x3E71, 0x0B0B, 0x29E8, 0x2ADD, 0x1724, 0x3B45, 0x22EC, 0x0706, 0x06FA, 
        0x3038, 0x019D, 0x0931, 0x0BC7, 0x26A0, 0x25CD, 0x1D89, 0x0326, 0x1B1E, 0x3BE4, 0x36C9, 0x05ED, 0x0BB4, 0x0944, 0x270F, 0x2F45, 
        0x3DC0, 0x3F48, 0x293B, 0x01DF, 0x0CAC, 0x0BE4, 0x0866, 0x05D4, 0x05BD, 0x25D0, 0x0FD4, 0x352A, 0x0FC7, 0x1982, 0x19C7, 0x18F9, 
        0x2975, 0x31E1, 0x06F4, 0x2E93, 0x1CEA, 0x14C1, 0x18E9, 0x1C47, 0x1EE0, 0x07A2, 0x09A1, 0x3FC2, 0x1B3D, 0x2F42, 0x1116, 0x341E, 
        0x1D12, 0x2FFA, 0x1138, 0x0229, 0x2A00, 0x3657, 0x1509, 0x06EA, 0x0C45, 0x1749, 0x3FE1, 0x162D, 0x1E9D, 0x36CB, 0x2DE3, 0x1496, 
        0x0345, 0x0CC8, 0x1C95, 0x0AEC, 0x204E, 0x06EF, 0x3E6B, 0x33E0, 0x1ABB, 0x3D8D, 0x19BF, 0x2CFC, 0x306D, 0x151F, 0x3221, 0x3133, 
        0x09EA, 0x066B, 0x0FAE, 0x1361, 0x21AC, 0x1EF7, 0x2572, 0x321F, 0x12AA, 0x10E5, 0x3F20, 0x2D94, 0x177B, 0x03D0, 0x2253, 0x1EF8, 
        0x0C54, 0x2B29, 0x018E, 0x0484, 0x38D8, 0x3EE4, 0x3348, 0x146F, 0x0D6E, 0x17E0, 0x3BC6, 0x3531, 0x30BF, 0x035D, 0x1732, 0x04A2, 
        0x31E0, 0x07D8, 0x0C51, 0x3D7A, 0x06BB, 0x188F, 0x295F, 0x1C25, 0x2BD1, 0x19DA, 0x184E, 0x0BC5, 0x3CBA, 0x3324, 0x02D6, 0x0AB2, 
        0x2CA4, 0x2974, 0x33F6, 0x37CD, 0x3659, 0x31CF, 0x26A3, 0x1694, 0x3D97, 0x1470, 0x041E, 0x244F, 0x2848, 0x3FA1, 0x1D1D, 0x3CEC, 
        0x107A, 0x2993, 0x227F, 0x3C05, 0x18CD, 0x0DBF, 0x20EF, 0x37E9, 0x1234, 0x3805, 0x3942, 0x3490, 0x3C9F, 0x3907, 0x2C99, 0x32D2, 
        0x39F5, 0x04B3, 0x170F, 0x3FDF, 0x0954, 0x265B, 0x0F57, 0x2C2C, 0x066E, 0x2B51, 0x3682, 0x2DAF, 0x2C14, 0x0AA9, 0x15B5, 0x3E1C, 
        0x0961, 0x1ED7, 0x3210, 0x1E8E, 0x028A, 0x1500, 0x38F0, 0x1636, 0x12F8, 0x3851, 0x2D31, 0x2462, 0x0970, 0x39E7, 0x22EB, 0x0CD1, 
        0x2E47, 0x2292, 0x17E6, 0x0E61, 0x1148, 0x1E49, 0x2FE2, 0x1424, 0x0BE1, 0x1331, 0x2843, 0x2D42, 0x3071, 0x16AB, 0x16B3, 0x20DC, 
        0x02DE, 0x3C5B, 0x0332, 0x1F2A, 0x291A, 0x08F4, 0x09F4, 0x17C9, 0x0F10, 0x27E9, 0x0E8A, 0x238C, 0x0E6F, 0x15DF, 0x3568, 0x3D4A, 
        0x34D1, 0x1E71, 0x1A7F, 0x07FF, 0x0E10, 0x1D17, 0x2AD5, 0x2578, 0x11ED, 0x3D9F, 0x3045, 0x1531, 0x3C7C, 0x3DED, 0x3002, 0x035F, 
        0x027D, 0x0B32, 0x0197, 0x20FB, 0x065D, 0x31AB, 0x26C0, 0x2DD7, 0x0641, 0x0334, 0x3E62, 0x0151, 0x0254, 0x3976, 0x3D44, 0x2C0B, 
        0x102C, 0x1E56, 0x13F8, 0x27BA, 0x071D, 0x0D2D, 0x3E01, 0x1697, 0x23EC, 0x20CE, 0x2035, 0x028E, 0x1922, 0x1A8E, 0x2195, 0x0D9E, 
        0x26F2, 0x3F43, 0x1B91, 0x3B2D, 0x10F9, 0x1B13, 0x01AF, 0x1446, 0x0715, 0x1183, 0x25D5, 0x1893, 0x034B, 0x1B67, 0x2630, 0x33DD, 
        0x1207, 0x15BF, 0x1314, 0x01F0, 0x348E, 0x11A3, 0x086D, 0x2482, 0x0F59, 0x315E, 0x2747, 0x3FF9, 0x2FC8, 0x1CAE, 0x37E3, 0x0969, 
        0x260D, 0x0998, 0x2AB7, 0x347B, 0x3804, 0x2183, 0x31D5, 0x289D, 0x25EB, 0x2FAE, 0x2AC1, 0x040F, 0x3FE0, 0x386D, 0x0F21, 0x1319, 
        0x0D39, 0x1536, 0x2094, 0x2F9B, 0x1988, 0x2D35, 0x2EAB, 0x2037, 0x1839, 0x18B2, 0x230A, 0x0B7A, 0x1717, 0x2719, 0x0D74, 0x1D83, 
        0x3173, 0x1859, 0x08B2, 0x2D1A, 0x0D16, 0x3D2F, 0x0BDF, 0x3315, 0x1958, 0x2616, 0x1C2C, 0x1661, 0x06CD, 0x16A9, 0x33B6, 0x0CC9, 
        0x23D2, 0x2B41, 0x0366, 0x2A49, 0x14E6, 0x0F7F, 0x0E24, 0x0E50, 0x3F75, 0x194F, 0x323A, 0x3E00, 0x2215, 0x20DA, 0x3C98, 0x0418, 
        0x371B, 0x0979, 0x0620, 0x3635, 0x24D4, 0x01F4, 0x28F0, 0x3DDF, 0x1B89, 0x137A, 0x0273, 0x1068, 0x06FF, 0x2E0D, 0x29C1, 0x1BE5, 
        0x2F1B, 0x022C, 0x021B, 0x046C, 0x01BD, 0x301C, 0x1BE9, 0x0EE4, 0x206C, 0x030F, 0x0ABC, 0x029D, 0x1F42, 0x115F, 0x0BB1, 0x3A6D, 
        0x33F0, 0x2223, 0x156D, 0x1436, 0x1AD6, 0x1872, 0x1DB8, 0x06F6, 0x0F54, 0x38A1, 0x20DD, 0x0AA4, 0x3443, 0x2E56, 0x069E, 0x121C, 
        0x2FA5, 0x3581, 0x0316, 0x0B6A, 0x3D9E, 0x0F19, 0x014D, 0x34FD, 0x2FCE, 0x217E, 0x367A, 0x17B2, 0x3B54, 0x2A3C, 0x1E07, 0x166C, 
        0x2E99, 0x03B3, 0x27DB, 0x1FA8, 0x1186, 0x1D39, 0x0BF4, 0x07E4, 0x0B74, 0x3387, 0x0DF7, 0x2DFC, 0x2B8A, 0x18EB, 0x36E5, 0x3525, 
        0x3A42, 0x0330, 0x1A36, 0x1DCA, 0x127D, 0x0796, 0x1DE2, 0x2C21, 0x2A46, 0x3163, 0x35DB, 0x0B15, 0x0CD7, 0x30B7, 0x2EC6, 0x300B, 
        0x12C2, 0x2AC2, 0x2321, 0x12E5, 0x132F, 0x320C, 0x0B04, 0x2141, 0x276B, 0x31AD, 0x1757, 0x10D6, 0x3A51, 0x1405, 0x3ACC, 0x10A3, 
        0x35A6, 0x38D0, 0x0685, 0x09EF, 0x25BC, 0x3CD5, 0x343C, 0x141B, 0x106F, 0x0617, 0x037E, 0x3D2A, 0x242B, 0x1CD0, 0x32CD, 0x03C9, 
        0x2236, 0x09C1, 0x3CBB, 0x306E, 0x2B53, 0x23BB, 0x0CDA, 0x1466, 0x3BC7, 0x114F, 0x04F7, 0x067F, 0x10D9, 0x3CE6, 0x1245, 0x107B, 
        0x1F1F, 0x2A6C, 0x2A92, 0x2DF1, 0x0762, 0x1C9D, 0x2DC9, 0x0735, 0x3836, 0x2125, 0x29DC, 0x0501, 0x3212, 0x31C1, 0x1E8D, 0x12EB, 
        0x2793, 0x1EB6, 0x2867, 0x256A, 0x1157, 0x019B, 0x2D87, 0x2BB4, 0x1D57, 0x3285, 0x223E, 0x0C9B, 0x276D, 0x3D26, 0x19D0, 0x30F1, 
        0x0522, 0x0F00, 0x3405, 0x1315, 0x2B92, 0x124D, 0x349B, 0x25C7, 0x1AEB, 0x3ECB, 0x0E39, 0x2EDF, 0x169D, 0x0D7A, 0x1CB2, 0x3385, 
        0x1937, 0x2B22, 0x3255, 0x1DB1, 0x1251, 0x3F19, 0x2A44, 0x23D0, 0x2963, 0x1E06, 0x12E2, 0x12CD, 0x0F48, 0x0761, 0x228B, 0x2BAD, 
        0x27B2, 0x0FB3, 0x2B33, 0x109C, 0x378C, 0x2BF7, 0x026B, 0x1123, 0x27B8, 0x2274, 0x33E5, 0x1214, 0x2F1E, 0x17BD, 0x1C29, 0x0AE9, 
        0x0C09, 0x1CAB, 0x0607, 0x347A, 0x3766, 0x2EAF, 0x08DF, 0x28AD, 0x2A8E, 0x3226, 0x073F, 0x3D2E, 0x1C34, 0x05BE, 0x1AD3, 0x09A5, 
        0x150F, 0x0EF2, 0x2CB3, 0x0449, 0x3E5B, 0x2F84, 0x10EC, 0x34DA, 0x21FD, 0x0AFB, 0x0A92, 0x02EB, 0x12F0, 0x3943, 0x2CAB, 0x29FF, 
        0x1780, 0x2C5B, 0x2C9F, 0x1411, 0x3365, 0x12C6, 0x0BAC, 0x1452, 0x2757, 0x3DF9, 0x1274, 0x03BC, 0x22F9, 0x0499, 0x0C71, 0x0B2F, 
        0x12E6, 0x24D1, 0x0D1C, 0x3340, 0x230F, 0x034D, 0x27F4, 0x1BEE, 0x331A, 0x31D2, 0x2F0F, 0x19C8, 0x0E7C, 0x1324, 0x1E86, 0x2368, 
        0x2D54, 0x27C5, 0x07F6, 0x0289, 0x38C3, 0x1A85, 0x3628, 0x1AEF, 0x332B, 0x2496, 0x26A8, 0x045E, 0x0F13, 0x051C, 0x0DE1, 0x2BA8, 
        0x24C4, 0x0606, 0x1F0F, 0x05EB, 0x2DD1, 0x3447, 0x309E, 0x0885, 0x39D6, 0x2AB4, 0x2467, 0x32A4, 0x04BC, 0x35AD, 0x0683, 0x27B6, 
        0x17C7, 0x30E1, 0x1E98, 0x25ED, 0x2614, 0x0C2D, 0x228E, 0x3C18, 0x19AD, 0x01B9, 0x3A96, 0x36EB, 0x0292, 0x126B, 0x3FB7, 0x3231, 
        0x29A1, 0x1C98, 0x1185, 0x1F04, 0x2AB1, 0x101D, 0x0B9E, 0x1D7D, 0x2BE4, 0x0E32, 0x3E92, 0x1EAC, 0x29EE, 0x22BE, 0x14B1, 0x2A43, 
        0x3E1F, 0x3D3F, 0x2CF2, 0x069D, 0x037A, 0x133E, 0x3B84, 0x31C2, 0x2ED2, 0x3EED, 0x0D80, 0x29A4, 0x1E9A, 0x28E3, 0x13A0, 0x0700, 
        0x2085, 0x05BC, 0x357D, 0x1BBF, 0x25C3, 0x1B85, 0x0B3A, 0x27D4, 0x3D11, 0x2FC3, 0x39AF, 0x0900, 0x2F7D, 0x0E04, 0x24FD, 0x15B2, 
        0x13E5, 0x1F7D, 0x29B7, 0x133F, 0x3AAF, 0x2B8D, 0x04E3, 0x23EF, 0x06E0, 0x1AC2, 0x2E4F, 0x30A2, 0x21E0, 0x1B3C, 0x3E38, 0x2342, 
        0x163A, 0x1063, 0x22D2, 0x2E29, 0x1663, 0x1CA2, 0x32E0, 0x2FC1, 0x21E6, 0x353A, 0x3712, 0x0535, 0x1264, 0x3F2C, 0x201C, 0x2548, 
        0x11D5, 0x0910, 0x04CD, 0x13D0, 0x2BF3, 0x1524, 0x3478, 0x3633, 0x3492, 0x105E, 0x1205, 0x24A4, 0x06F0, 0x35D2, 0x1BB2, 0x2740, 
        0x3B65, 0x1936, 0x32AA, 0x0107, 0x31E7, 0x0A21, 0x267A, 0x2303, 0x31F0, 0x1296, 0x3BE9, 0x3F30, 0x176E, 0x27A5, 0x1459, 0x1B1B, 
        0x1255, 0x0663, 0x2199, 0x31DE, 0x385F, 0x2008, 0x3469, 0x1490, 0x0CC2, 0x3398, 0x391A, 0x3C1F, 0x12DA, 0x2AFB, 0x2E96, 0x3E7C, 
        0x2729, 0x388C, 0x3AC2, 0x3710, 0x3366, 0x0A96, 0x2458, 0x2D4B, 0x0F69, 0x02F9, 0x076D, 0x2425, 0x16C8, 0x24BB, 0x3830, 0x399A, 
        0x0C49, 0x3E75, 0x10A6, 0x347C, 0x1837, 0x07AA, 0x24B2, 0x15F2, 0x1721, 0x207B, 0x2DFA, 0x2CCD, 0x3E64, 0x0220, 0x2F9F, 0x1E46, 
        0x0759, 0x0858, 0x2BBC, 0x38A9, 0x072D, 0x2792, 0x39E3, 0x0430, 0x0C4D, 0x3677, 0x1BC1, 0x3AFE, 0x3A1E, 0x1BF9, 0x2061, 0x095C, 
        0x3999, 0x0F8A, 0x0393, 0x2F6D, 0x1CEF, 0x27D9, 0x35C2, 0x2AAB, 0x387B, 0x19B0, 0x3B4C, 0x2310, 0x3729, 0x02FF, 0x0670, 0x208A, 
        0x3C83, 0x0A84, 0x2472, 0x1AF9, 0x3F16, 0x2ECC, 0x2298, 0x0263, 0x356A, 0x3C34, 0x32A7, 0x3860, 0x38CB, 0x1763, 0x0A9E, 0x22A8, 
        0x38B3, 0x1CAD, 0x35BB, 0x0DB5, 0x0EF9, 0x1FE3, 0x1960, 0x35E5, 0x3337, 0x1103, 0x360C, 0x2ABD, 0x1AC9, 0x0CF8, 0x3225, 0x0D79, 
        0x1F7F, 0x3DC6, 0x1764, 0x3523, 0x2D20, 0x22B1, 0x1CC6, 0x1143, 0x0EC1, 0x1189, 0x0A41, 0x330A, 0x0FC8, 0x2453, 0x1053, 0x3E61, 
        0x3286, 0x1A35, 0x07A9, 0x3936, 0x2B6F, 0x217D, 0x357A, 0x010E, 0x0958, 0x0671, 0x2186, 0x2770, 0x198C, 0x2962, 0x3B3C, 0x2B06, 
        0x018D, 0x3290, 0x3845, 0x1C17, 0x16D2, 0x1EF2, 0x2B59, 0x261E, 0x3C72, 0x2045, 0x1F17, 0x3E63, 0x29A3, 0x1E72, 0x13ED, 0x0CEE, 
        0x0EA5, 0x09BF, 0x1E2E, 0x2742, 0x26E1, 0x130C, 0x0B1B, 0x3672, 0x363F, 0x3ED7, 0x3EAA, 0x0E51, 0x30B6, 0x1653, 0x1885, 0x13D3, 
        0x1D90, 0x3ABC, 0x3C8E, 0x235B, 0x22A5, 0x30C6, 0x3C60, 0x2BF1, 0x0E33, 0x0899, 0x32F7, 0x395C, 0x2B4D, 0x2A28, 0x1D3C, 0x06F5, 
        0x2335, 0x3242, 0x3D01, 0x3C1B, 0x2CB8, 0x35E8, 0x0D8A, 0x3516, 0x0E69, 0x118E, 0x0322, 0x103A, 0x0F32, 0x304E, 0x06AC, 0x1A00, 
        0x081F, 0x0E40, 0x1927, 0x3E0E, 0x09E9, 0x3089, 0x0EAD, 0x0A1A, 0x0B0D, 0x2068, 0x1C64, 0x3E91, 0x0126, 0x3959, 0x0AB0, 0x3253, 
        0x0989, 0x3949, 0x279E, 0x2157, 0x0A6A, 0x0EBE, 0x3A89, 0x23CF, 0x1B1A, 0x119B, 0x05F6, 0x1ED3, 0x13AD, 0x350E, 0x208F, 0x1D1C, 
        0x3993, 0x3B43, 0x2B63, 0x3840, 0x03DF, 0x29E3, 0x1DA7, 0x08B3, 0x275B, 0x0A70, 0x0C7D, 0x1333, 0x3511, 0x2E31, 0x146E, 0x260F, 
        0x3D10, 0x2CE8, 0x2EF5, 0x04E4, 0x3C3F, 0x2C49, 0x3DB5, 0x2A05, 0x0CAE, 0x0485, 0x3686, 0x2F49, 0x12EC, 0x11D8, 0x21D4, 0x0AE0, 
        0x01D2, 0x0529, 0x2F63, 0x2316, 0x1B8C, 0x3E74, 0x0F33, 0x38F2, 0x2F5D, 0x0D0E, 0x2DBF, 0x3532, 0x2FB2, 0x3B85, 0x100A, 0x241A, 
        0x2308, 0x17A7, 0x06C4, 0x089D, 0x0497, 0x3560, 0x2641, 0x3EBE, 0x3F3B, 0x31EB, 0x12CA, 0x27F1, 0x139F, 0x16F1, 0x1A62, 0x1463, 
        0x2976, 0x0FC0, 0x2AC7, 0x1959, 0x0E95, 0x3E36, 0x1CA5, 0x3B9C, 0x0B8F, 0x13E6, 0x304F, 0x0E1C, 0x29C4, 0x345A, 0x0FB6, 0x39B6, 
        0x170E, 0x2628, 0x039A, 0x0CB8, 0x1280, 0x25A5, 0x1E74, 0x207C, 0x3E81, 0x04C6, 0x1783, 0x18DB, 0x1B48, 0x1A17, 0x3AFC, 0x3451, 
        0x3CC7, 0x1F32, 0x1559, 0x0888, 0x21F3, 0x232E, 0x0C02, 0x234F, 0x0FF5, 0x249C, 0x2680, 0x180E, 0x29F4, 0x0128, 0x1931, 0x3E8D, 
        0x1641, 0x11E7, 0x3288, 0x0585, 0x3235, 0x0ABB, 0x0481, 0x0AF1, 0x092F, 0x0A3B, 0x14BF, 0x28FF, 0x37A7, 0x3A0A, 0x305F, 0x2535, 
        0x15D2, 0x36A6, 0x31D3, 0x023F, 0x0934, 0x0435, 0x1E26, 0x3798, 0x0AF0, 0x254F, 0x1DD8, 0x2074, 0x0D69, 0x181B, 0x2EE3, 0x2B77, 
        0x0B61, 0x2070, 0x2F16, 0x34B9, 0x14A4, 0x3BAD, 0x2EDC, 0x2CF4, 0x1349, 0x1A77, 0x2F69, 0x0459, 0x0E77, 0x3BF2, 0x2C9A, 0x3A38, 
        0x29A0, 0x25C6, 0x0AD0, 0x28F9, 0x3332, 0x0BBB, 0x2926, 0x18E4, 0x2855, 0x0D8F, 0x032F, 0x053A, 0x1A05, 0x2F8B, 0x1AD9, 0x38DE, 
        0x0FAA, 0x0F93, 0x0C00, 0x1BFD, 0x0283, 0x0B5D, 0x0C05, 0x3D38, 0x1ECD, 0x3C32, 0x124E, 0x164B, 0x313A, 0x0F4B, 0x3B1A, 0x2341, 
        0x268F, 0x3537, 0x3927, 0x2D2D, 0x064C, 0x01E2, 0x1342, 0x3910, 0x33C3, 0x2550, 0x3BC5, 0x0DB6, 0x3EF2, 0x3F60, 0x20E7, 0x3BF3, 
        0x24DE, 0x2406, 0x0F3F, 0x0B19, 0x181A, 0x123D, 0x3854, 0x3E4F, 0x143B, 0x2F3E, 0x0176, 0x0911, 0x2CEF, 0x0E3B, 0x067B, 0x0651, 
        0x2408, 0x2297, 0x33DE, 0x19A1, 0x3CE0, 0x1869, 0x3956, 0x0734, 0x2E85, 0x3E9E, 0x2869, 0x0B25, 0x3D6E, 0x01F6, 0x258B, 0x2E14, 
        0x1804, 0x1B10, 0x2EE6, 0x37CC, 0x284B, 0x3177, 0x310A, 0x0A80, 0x21FF, 0x2232, 0x177C, 0x1EF9, 0x2C5D, 0x1ECF, 0x3FF1, 0x2599, 
        0x2625, 0x09D8, 0x02DD, 0x2DE0, 0x0112, 0x1C9E, 0x0B7D, 0x122B, 0x25FD, 0x1A9C, 0x2B27, 0x130E, 0x36BA, 0x2D7E, 0x11A2, 0x1379, 
        0x3BCF, 0x2B54, 0x2C41, 0x2AEE, 0x08E2, 0x32FB, 0x391D, 0x0A51, 0x0A6E, 0x2A78, 0x2C96, 0x3BCB, 0x1D31, 0x34AC, 0x0A09, 0x1631, 
        0x06BE, 0x1989, 0x0290, 0x39C4, 0x0804, 0x1EB8, 0x18E6, 0x1EFF, 0x1712, 0x3066, 0x1963, 0x0EFF, 0x1A63, 0x3480, 0x0ADE, 0x1E33, 
        0x1299, 0x2E3C, 0x3D8E, 0x262A, 0x080A, 0x3123, 0x20A9, 0x20EC, 0x1F3C, 0x26F4, 0x118F, 0x2DFD, 0x0DFA, 0x0D87, 0x3749, 0x29F8, 
        0x3230, 0x0C3A, 0x2E34, 0x3102, 0x15BD, 0x2443, 0x3F91, 0x3A06, 0x12FC, 0x3C8C, 0x3A5F, 0x0A00, 0x2AE1, 0x10EA, 0x32B6, 0x034C, 
        0x2900, 0x0875, 0x20FD, 0x3D98, 0x32E8, 0x28FB, 0x1899, 0x271A, 0x2DB2, 0x3A0D, 0x3C85, 0x1F9D, 0x024A, 0x333E, 0x0A0A, 0x2B31, 
        0x3EFC, 0x37AD, 0x058E, 0x1728, 0x0E8E, 0x15C2, 0x0C5B, 0x2BA1, 0x1BC6, 0x112A, 0x235C, 0x0C57, 0x184C, 0x1A64, 0x1C04, 0x0F65, 
        0x119A, 0x2090, 0x3AC6, 0x176F, 0x1ABE, 0x0580, 0x042C, 0x2E8D, 0x1BA6, 0x3A60, 0x20BE, 0x0AC5, 0x2426, 0x0A90, 0x10E8, 0x2A01, 
        0x25D2, 0x0724, 0x2BF6, 0x3973, 0x0F7B, 0x286C, 0x1ADD, 0x05A8, 0x2A48, 0x1D4E, 0x0150, 0x0D1A, 0x3B7A, 0x34D0, 0x14CD, 0x3E3D, 
        0x3632, 0x3842, 0x1D32, 0x3774, 0x2299, 0x182D, 0x363D, 0x3113, 0x15BA, 0x33BB, 0x2BDA, 0x1E11, 0x2C13, 0x0E70, 0x0C63, 0x3A9F, 
        0x077D, 0x3678, 0x13EA, 0x2233, 0x0F0C, 0x2768, 0x114B, 0x349E, 0x0E6D, 0x2E1D, 0x3661, 0x1906, 0x1656, 0x2E15, 0x33A5, 0x3A81, 
        0x04FD, 0x226A, 0x0E23, 0x0441, 0x2D0F, 0x1E08, 0x10FE, 0x24BD, 0x2D5B, 0x2501, 0x3F79, 0x3748, 0x3607, 0x21FE, 0x0D61, 0x2CD1, 
        0x2B2E, 0x0C7C, 0x03DB, 0x2A2D, 0x0D8B, 0x376E, 0x0902, 0x23AA, 0x0D0F, 0x0C5F, 0x1929, 0x0A4A, 0x1718, 0x2DB5, 0x14AD, 0x2012, 
        0x2B05, 0x0344, 0x1D5F, 0x199C, 0x2EED, 0x3CC3, 0x2B1C, 0x3DF7, 0x077C, 0x1770, 0x1DC2, 0x18CA, 0x1BD0, 0x2C11, 0x1F54, 0x1B47, 
        0x2580, 0x1400, 0x2B6E, 0x393D, 0x2DC8, 0x25FC, 0x2ADC, 0x3118, 0x1C26, 0x0A68, 0x288D, 0x2703, 0x227A, 0x29CA, 0x1527, 0x1038, 
        0x3B16, 0x3E9F, 0x0B8A, 0x2A35, 0x0DE4, 0x336D, 0x0537, 0x0C15, 0x233D, 0x2B2F, 0x1C27, 0x3B8F, 0x321E, 0x200E, 0x3DF3, 0x2D0C, 
        0x1657, 0x1FB3, 0x31C8, 0x0E44, 0x397F, 0x38FC, 0x1AF7, 0x08B7, 0x2D64, 0x10B0, 0x2ED6, 0x2C1A, 0x34E3, 0x1D19, 0x1FD1, 0x3908, 
        0x2CE9, 0x1617, 0x25A8, 0x0ACF, 0x3436, 0x28B9, 0x118A, 0x26EC, 0x0B38, 0x1C2A, 0x2790, 0x0B97, 0x1C35, 0x1901, 0x0BC3, 0x1FC5, 
        0x3C1C, 0x2216, 0x2642, 0x2F98, 0x0592, 0x20C4, 0x3758, 0x30FB, 0x3093, 0x012C, 0x0675, 0x323C, 0x0308, 0x06CC, 0x078C, 0x18FB, 
        0x0E4B, 0x18AC, 0x3EB0, 0x2EBE, 0x1B74, 0x2886, 0x2529, 0x0AB5, 0x2948, 0x3815, 0x26B5, 0x14D2, 0x0C93, 0x27A4, 0x0712, 0x2753, 
        0x3B74, 0x2CFE, 0x3400, 0x1E73, 0x1322, 0x29D7, 0x2FDC, 0x28CF, 0x025F, 0x160E, 0x3022, 0x2FA0, 0x30D6, 0x376D, 0x3D08, 0x39C7, 
        0x3C7F, 0x0E03, 0x0E8B, 0x0D70, 0x3025, 0x1D98, 0x2BB7, 0x175F, 0x2646, 0x02CF, 0x177F, 0x1D47, 0x03C1, 0x1028, 0x05BA, 0x3A44, 
        0x1B61, 0x2178, 0x2554, 0x3BFE, 0x342B, 0x3DA0, 0x3EC1, 0x10BF, 0x3DEE, 0x3B53, 0x3392, 0x1D6B, 0x05AC, 0x0595, 0x0777, 0x049A, 
        0x24B8, 0x2830, 0x3980, 0x3A97, 0x3BBF, 0x3141, 0x30BE, 0x2A54, 0x20A4, 0x0E7D, 0x046E, 0x09B9, 0x3616, 0x1CC9, 0x28EE, 0x15F0, 
        0x0457, 0x23E0, 0x0253, 0x3AE0, 0x06DF, 0x23E8, 0x39E9, 0x214E, 0x19E6, 0x1B04, 0x2493, 0x04E8, 0x0DE6, 0x39FC, 0x2A8B, 0x0B14, 
        0x0297, 0x05EA, 0x0CC0, 0x3B46, 0x2A85, 0x1DDB, 0x20AC, 0x3C90, 0x1B93, 0x0167, 0x3415, 0x3C61, 0x3E32, 0x3104, 0x0EDC, 0x20F5, 
        0x2906, 0x1D8A, 0x214A, 0x09F8, 0x0F95, 0x2525, 0x14CE, 0x356D, 0x2F82, 0x077A, 0x22D3, 0x3582, 0x38CD, 0x25AD, 0x0845, 0x10C6, 
        0x2E12, 0x02FE, 0x27E8, 0x2FCC, 0x31D6, 0x2BDC, 0x1D3D, 0x03CB, 0x2F6C, 0x2D26, 0x0E2E, 0x0EF7, 0x1D7E, 0x250D, 0x2947, 0x39D5, 
        0x2EC9, 0x2325, 0x351A, 0x1A22, 0x172B, 0x09B3, 0x281F, 0x3EE0, 0x1B5B, 0x17D6, 0x272C, 0x3F3C, 0x1AAB, 0x314F, 0x1032, 0x2812, 
        0x0C6C, 0x27A3, 0x0351, 0x3F7C, 0x28B6, 0x3DE8, 0x356B, 0x239C, 0x033E, 0x0AF5, 0x0271, 0x0BE3, 0x3D8F, 0x2533, 0x32DE, 0x1F48, 
        0x2185, 0x08D8, 0x01E5, 0x1F33, 0x1C6E, 0x0B90, 0x2049, 0x1A76, 0x1347, 0x3454, 0x273C, 0x2379, 0x1CC0, 0x3844, 0x24F7, 0x254C, 
        0x1CB0, 0x1BA5, 0x0D1F, 0x30C0, 0x38F5, 0x1E95, 0x1F81, 0x1595, 0x1B1C, 0x0EA2, 0x15C8, 0x2B6B, 0x048A, 0x0629, 0x09F5, 0x0AF4, 
        0x2F5F, 0x2699, 0x330B, 0x3520, 0x2AAE, 0x3E68, 0x0BF1, 0x3DA2, 0x0AE2, 0x19F8, 0x2878, 0x33E1, 0x11EF, 0x249D, 0x2A36, 0x17EF, 
        0x2204, 0x378A, 0x225A, 0x2018, 0x3F51, 0x377F, 0x3F39, 0x0FB5, 0x14A6, 0x3CE4, 0x0E75, 0x0F74, 0x1888, 0x38E7, 0x3723, 0x3497, 
        0x2D5F, 0x226E, 0x09E8, 0x080E, 0x2EA2, 0x29F0, 0x3B4A, 0x1D4D, 0x227E, 0x0FDE, 0x2AE3, 0x3642, 0x3DA9, 0x0C2E, 0x3F49, 0x14C7, 
        0x10F4, 0x1B6D, 0x1B01, 0x0D44, 0x2E70, 0x1158, 0x0CC7, 0x2CB5, 0x1A2C, 0x0851, 0x358D, 0x322B, 0x2710, 0x3589, 0x33A0, 0x3D6C, 
        0x251C, 0x1C97, 0x2FAB, 0x0E63, 0x1B7C, 0x38A7, 0x3420, 0x2122, 0x02D9, 0x3C4B, 0x051B, 0x242F, 0x3808, 0x2763, 0x31E5, 0x391C, 
        0x35D7, 0x1E6F, 0x0DF6, 0x3169, 0x3A98, 0x3178, 0x3EEC, 0x0755, 0x2994, 0x296A, 0x18D8, 0x2DA1, 0x28DF, 0x1A3F, 0x3068, 0x1FD5, 
        0x3AEC, 0x27F6, 0x3D87, 0x0D4B, 0x09DB, 0x17BB, 0x11C8, 0x39DF, 0x2411, 0x200B, 0x21ED, 0x1EE8, 0x0F6C, 0x0395, 0x0D8E, 0x2B8E, 
        0x32C9, 0x2F6E, 0x2A1E, 0x3248, 0x3060, 0x272B, 0x3018, 0x1FFB, 0x1021, 0x1BB6, 0x2319, 0x2D8B, 0x0370, 0x111F, 0x128D, 0x342D, 
        0x2038, 0x1898, 0x17FD, 0x31B6, 0x0267, 0x1FCC, 0x181F, 0x09E2, 0x233E, 0x025C, 0x2B6A, 0x3BE3, 0x0811, 0x088C, 0x0B92, 0x020F, 
        0x3CFC, 0x35D6, 0x15CF, 0x0D96, 0x27BC, 0x335D, 0x0D4A, 0x3FAE, 0x0407, 0x37E4, 0x1AB7, 0x0119, 0x1592, 0x18A8, 0x1C3F, 0x0D13, 
        0x3120, 0x0DF2, 0x0B4D, 0x16DB, 0x2F2A, 0x11F7, 0x11AD, 0x36B3, 0x02E6, 0x3214, 0x1C0A, 0x0227, 0x0608, 0x330D, 0x125B, 0x1137, 
        0x2C76, 0x27F7, 0x2551, 0x06A5, 0x17A3, 0x22D1, 0x2E6A, 0x1E35, 0x3FB5, 0x1E68, 0x1265, 0x26B7, 0x3A2A, 0x25A1, 0x26E7, 0x12DD, 
        0x2C8A, 0x14DB, 0x272D, 0x0E66, 0x3DFE, 0x18E2, 0x20A3, 0x2FE9, 0x1444, 0x1D93, 0x0A91, 0x154F, 0x0C24, 0x1154, 0x371F, 0x33B0, 
        0x38A5, 0x161E, 0x1471, 0x1CB8, 0x31E4, 0x0B9A, 0x0FE7, 0x3456, 0x2697, 0x1B0A, 0x25BA, 0x29EC, 0x31A1, 0x0CEC, 0x322F, 0x1396, 
        0x068B, 0x3C81, 0x3131, 0x1EF4, 0x285B, 0x055D, 0x3BE6, 0x26F1, 0x2381, 0x16F2, 0x37B8, 0x390B, 0x13E3, 0x1904, 0x0530, 0x2C2F, 
        0x2E37, 0x34EA, 0x2A5C, 0x3483, 0x27E4, 0x1D28, 0x235D, 0x3A20, 0x3FCB, 0x1166, 0x1561, 0x0C5E, 0x31D8, 0x0F9C, 0x3AB1, 0x0444, 
        0x0C40, 0x3262, 0x240E, 0x1FAE, 0x176C, 0x0EA4, 0x1E90, 0x2192, 0x1664, 0x2F03, 0x1924, 0x2AAF, 0x0DA7, 0x3B07, 0x055F, 0x3072, 
        0x30AB, 0x2766, 0x0B9C, 0x2598, 0x0E9F, 0x3143, 0x269D, 0x02E9, 0x37C8, 0x1572, 0x12DC, 0x2B85, 0x2364, 0x393F, 0x2143, 0x2F3D, 
        0x070B, 0x3012, 0x0D11, 0x0655, 0x06BC, 0x3B73, 0x1F16, 0x2B30, 0x2A57, 0x0244, 0x02D8, 0x33C7, 0x2592, 0x01B7, 0x06C2, 0x0D45, 
        0x0B77, 0x12F3, 0x311F, 0x1786, 0x0415, 0x2463, 0x0AA8, 0x18AE, 0x017F, 0x0FA4, 0x1CDA, 0x39B4, 0x190E, 0x035C, 0x07E3, 0x3106, 
        0x1176, 0x16BB, 0x1373, 0x1866, 0x08A3, 0x2B28, 0x3861, 0x1534, 0x1F0C, 0x0963, 0x3D9D, 0x0874, 0x144B, 0x3D88, 0x2BA2, 0x31A6, 
        0x142D, 0x2EFB, 0x2EF9, 0x1249, 0x135C, 0x19B8, 0x28B0, 0x0281, 0x0C41, 0x16E2, 0x25FA, 0x2BB0, 0x1DDE, 0x283E, 0x0F47, 0x1897, 
        0x2FAF, 0x389F, 0x1D06, 0x38CE, 0x28DC, 0x3867, 0x39CD, 0x37E7, 0x0A65, 0x24EC, 0x1CE1, 0x2D05, 0x2583, 0x3356, 0x2C66, 0x1585, 
        0x08FC, 0x163E, 0x2096, 0x05D2, 0x0E2F, 0x04BA, 0x08E1, 0x1CCB, 0x0FC2, 0x3E5F, 0x3D3D, 0x07DA, 0x1259, 0x3D30, 0x0952, 0x1059, 
        0x263F, 0x0448, 0x1A1E, 0x1167, 0x2FDE, 0x0513, 0x23C3, 0x0B5B, 0x05E8, 0x03B1, 0x3946, 0x3E7B, 0x3335, 0x1EEF, 0x2163, 0x08BF, 
        0x045C, 0x2631, 0x3EA8, 0x190C, 0x1C84, 0x1B4F, 0x1939, 0x05FC, 0x0D0C, 0x3C63, 0x3BB8, 0x1591, 0x0F12, 0x17CF, 0x06D5, 0x06E9, 
        0x3238, 0x0EC0, 0x3207, 0x1AAF, 0x3658, 0x192D, 0x111C, 0x3249, 0x191A, 0x3F2D, 0x1E39, 0x31BE, 0x362D, 0x1614, 0x1304, 0x130F, 
        0x24DF, 0x2B09, 0x20C2, 0x192E, 0x3CEF, 0x361C, 0x280F, 0x266E, 0x3F9E, 0x21EF, 0x17A5, 0x3788, 0x0AB3, 0x3019, 0x1EBA, 0x0232, 
        0x2063, 0x03FB, 0x0786, 0x3902, 0x287E, 0x1B68, 0x3765, 0x2165, 0x0E98, 0x1BEC, 0x20A5, 0x1A13, 0x156B, 0x089B, 0x1392, 0x381C, 
        0x3129, 0x074E, 0x0123, 0x1B22, 0x2E80, 0x1E13, 0x35C0, 0x28C7, 0x05EE, 0x16CA, 0x0312, 0x312E, 0x22D8, 0x3198, 0x23AC, 0x2F41, 
        0x3B4F, 0x2C25, 0x0392, 0x1460, 0x2BD2, 0x2C44, 0x14AC, 0x1F1B, 0x353B, 0x2491, 0x1F61, 0x3108, 0x115D, 0x1E54, 0x11EA, 0x2952, 
        0x2E92, 0x06C0, 0x3247, 0x07EA, 0x05A5, 0x16BA, 0x09F1, 0x2C68, 0x2935, 0x0FAD, 0x2311, 0x14E0, 0x06E2, 0x0221, 0x3F31, 0x3618, 
        0x3C7E, 0x08C5, 0x205C, 0x3C54, 0x164D, 0x35D9, 0x2C19, 0x0552, 0x398C, 0x2FEC, 0x2338, 0x24CD, 0x18EF, 0x3829, 0x0AA5, 0x19E9, 
        0x14B8, 0x11F3, 0x24B3, 0x388F, 0x10DA, 0x295C, 0x1920, 0x1615, 0x0915, 0x1CB6, 0x1597, 0x0B02, 0x18DF, 0x1A4F, 0x0587, 0x34C5, 
        0x0DC2, 0x3A7F, 0x1FBB, 0x3C64, 0x248B, 0x250B, 0x2255, 0x3A33, 0x0D2F, 0x05C3, 0x16F9, 0x0C34, 0x2661, 0x1345, 0x0640, 0x16F8, 
        0x276C, 0x2AC4, 0x1949, 0x0D89, 0x16E3, 0x3C24, 0x067C, 0x34C7, 0x2706, 0x0ED2, 0x3D51, 0x1E12, 0x043C, 0x0750, 0x39B7, 0x211D, 
        0x01F3, 0x02F7, 0x10EF, 0x27EE, 0x34F1, 0x223D, 0x2E66, 0x0B03, 0x0398, 0x22B2, 0x19F2, 0x1E05, 0x3550, 0x06A4, 0x0A31, 0x1785, 
        0x0F9F, 0x324E, 0x1799, 0x372A, 0x1833, 0x2544, 0x1D9B, 0x3C0B, 0x371C, 0x0EFC, 0x3D2B, 0x3AE1, 0x06EC, 0x331F, 0x273A, 0x06D0, 
        0x0A5D, 0x0AAF, 0x0C92, 0x0A3A, 0x0BFD, 0x38BE, 0x18F2, 0x0C99, 0x3E8B, 0x196C, 0x21C0, 0x0EB5, 0x2A99, 0x1246, 0x214C, 0x1AB6, 
        0x102F, 0x0F98, 0x1E7E, 0x135B, 0x364A, 0x3FD9, 0x333D, 0x0DF4, 0x3771, 0x1547, 0x389C, 0x2283, 0x0412, 0x22E8, 0x263E, 0x2466, 
        0x0272, 0x305C, 0x3080, 0x2C83, 0x2F0B, 0x0E34, 0x2476, 0x38F4, 0x271D, 0x3C74, 0x123A, 0x0A36, 0x2E42, 0x17A9, 0x15E8, 0x0789, 
        0x24C7, 0x0FF8, 0x21E2, 0x329B, 0x0EA7, 0x149A, 0x0F15, 0x0BEC, 0x29AE, 0x1F7A, 0x3057, 0x059D, 0x13C1, 0x25AA, 0x3A15, 0x33D4, 
        0x3AF7, 0x34D8, 0x2DCB, 0x0D5F, 0x2317, 0x204F, 0x1DBB, 0x324C, 0x3E0A, 0x1B80, 0x3E18, 0x0E17, 0x21B0, 0x1ACA, 0x3440, 0x09AA, 
        0x1337, 0x0DFD, 0x39A7, 0x2C77, 0x2E75, 0x0F45, 0x1EC5, 0x1983, 0x134F, 0x3D0C, 0x1863, 0x1FD3, 0x09C4, 0x150B, 0x22AE, 0x0561, 
        0x02C0, 0x2F00, 0x0639, 0x20B1, 0x386C, 0x0369, 0x2732, 0x1FE2, 0x07B4, 0x158F, 0x3360, 0x06DE, 0x14CB, 0x22B4, 0x022E, 0x1D16, 
        0x3A0F, 0x23C4, 0x2302, 0x041F, 0x1E89, 0x2EB4, 0x02CC, 0x3437, 0x1E1F, 0x2A83, 0x1573, 0x313E, 0x0ED8, 0x3676, 0x0451, 0x3608, 
        0x26FE, 0x0B13, 0x18D7, 0x1ED1, 0x14BA, 0x0DF1, 0x0588, 0x1E0F, 0x3FC6, 0x2AE2, 0x27CA, 0x1802, 0x062A, 0x2A59, 0x3DC3, 0x30D1, 
        0x3362, 0x1701, 0x0222, 0x2027, 0x1034, 0x0315, 0x3166, 0x1484, 0x0CC3, 0x3A8C, 0x1D30, 0x042D, 0x0572, 0x278D, 0x103C, 0x15BC, 
        0x25E4, 0x2828, 0x38BA, 0x259E, 0x0FE3, 0x23F5, 0x2A14, 0x3EA0, 0x3904, 0x1E88, 0x25A3, 0x2785, 0x0B55, 0x2C6C, 0x0259, 0x111E, 
        0x232D, 0x053C, 0x0935, 0x3579, 0x1119, 0x26D8, 0x1513, 0x0A1F, 0x3F26, 0x2E8B, 0x2EAD, 0x38C2, 0x08AD, 0x19E1, 0x28BE, 0x1784, 
        0x2276, 0x0FE5, 0x15AA, 0x10FA, 0x1461, 0x12A1, 0x32BC, 0x0495, 0x26CE, 0x0836, 0x0F20, 0x063F, 0x33A8, 0x2079, 0x1507, 0x2982, 
        0x0309, 0x203B, 0x37BA, 0x0AB1, 0x05D0, 0x14B6, 0x1918, 0x148C, 0x38CA, 0x2C4B, 0x10A0, 0x078E, 0x1495, 0x250A, 0x0349, 0x327A, 
        0x3DD6, 0x3FE6, 0x0633, 0x3180, 0x2067, 0x2D7D, 0x1C1F, 0x2036, 0x0714, 0x19C2, 0x3202, 0x1968, 0x2196, 0x16BE, 0x2D0A, 0x0E30, 
        0x23AE, 0x285D, 0x091A, 0x08FF, 0x3295, 0x20F7, 0x1F6A, 0x1E10, 0x032E, 0x2E9B, 0x1602, 0x12DE, 0x2708, 0x2BE0, 0x3EE1, 0x2CBC, 
        0x2CFA, 0x3FAC, 0x2558, 0x0677, 0x1AE1, 0x14FE, 0x2D6A, 0x2184, 0x2C36, 0x0357, 0x0323, 0x1FC9, 0x07FA, 0x1E4D, 0x2588, 0x0659, 
        0x352F, 0x0BED, 0x3349, 0x2254, 0x3896, 0x1334, 0x2E82, 0x165E, 0x0D9C, 0x2EBF, 0x211F, 0x2C4C, 0x1372, 0x06E8, 0x2E49, 0x0E3F, 
        0x2AAD, 0x33F1, 0x0C1B, 0x0631, 0x23B8, 0x0FCA, 0x1DB3, 0x0E79, 0x149F, 0x3DB9, 0x08D3, 0x0BA8, 0x36F3, 0x2E7A, 0x2C3E, 0x2BC3, 
        0x2721, 0x14C0, 0x1AA3, 0x2470, 0x0FD7, 0x2126, 0x186A, 0x3B79, 0x2AA3, 0x2783, 0x1AD2, 0x151C, 0x0CE0, 0x160B, 0x1557, 0x3F77, 
        0x20B5, 0x0630, 0x06D3, 0x215A, 0x21F2, 0x2CFD, 0x02CE, 0x2228, 0x17B0, 0x1B17, 0x2F85, 0x27AD, 0x369E, 0x2F68, 0x19EF, 0x3359, 
        0x0884, 0x0C85, 0x1EB4, 0x1535, 0x0EC6, 0x09D4, 0x1788, 0x354B, 0x3466, 0x39E6, 0x33D2, 0x2DC2, 0x3689, 0x1EC1, 0x3DAA, 0x1B9D, 
        0x0DC4, 0x30B0, 0x111B, 0x393B, 0x3E1E, 0x0F05, 0x0653, 0x1BB5, 0x19FB, 0x0BCD, 0x1267, 0x157D, 0x3B31, 0x34B6, 0x39B1, 0x360D, 
        0x39C5, 0x0C90, 0x048F, 0x1EB9, 0x3229, 0x268A, 0x03E6, 0x1C71, 0x1B5A, 0x0E25, 0x223A, 0x24D9, 0x3C9A, 0x319B, 0x0E06, 0x3680, 
        0x0CF1, 0x29F5, 0x231C, 0x2243, 0x1051, 0x1F66, 0x2879, 0x09A2, 0x2221, 0x3222, 0x0476, 0x0C73, 0x0BD9, 0x2553, 0x2557, 0x3E0D, 
        0x213F, 0x1516, 0x2314, 0x24A0, 0x3CB4, 0x0B73, 0x0B30, 0x1455, 0x1244, 0x1391, 0x3B1D, 0x1FF4, 0x2420, 0x25EC, 0x0538, 0x0681, 
        0x0BC2, 0x1A9B, 0x0335, 0x3F8D, 0x2F51, 0x0A3C, 0x24AA, 0x25CB, 0x2260, 0x177D, 0x2D6D, 0x3BCA, 0x1C70, 0x3E85, 0x35C8, 0x1D9D, 
        0x1170, 0x2BA7, 0x0AD8, 0x1AB5, 0x2A06, 0x08A4, 0x366A, 0x142B, 0x25C0, 0x1073, 0x3EC7, 0x02A3, 0x1846, 0x3DEF, 0x3E23, 0x0250, 
        0x281D, 0x2C3C, 0x26FD, 0x138C, 0x2F4E, 0x2B4F, 0x1C5F, 0x0C5C, 0x34FE, 0x1773, 0x03C7, 0x3F57, 0x1E92, 0x178F, 0x2BBD, 0x227D, 
        0x3CE3, 0x0CBD, 0x1D51, 0x3C57, 0x30A7, 0x184D, 0x3055, 0x2919, 0x249F, 0x1E00, 0x0248, 0x22E2, 0x2D17, 0x1012, 0x25F3, 0x2C7B, 
        0x276E, 0x18AA, 0x374D, 0x09AB, 0x2180, 0x3053, 0x2799, 0x2C35, 0x3D3A, 0x0BC4, 0x21BC, 0x0AC3, 0x1B7B, 0x2C40, 0x0EC7, 0x3342, 
        0x1AAD, 0x0BBF, 0x2173, 0x0E42, 0x02A9, 0x17D5, 0x2133, 0x1853, 0x3AD8, 0x2FF8, 0x1C6A, 0x2A3F, 0x287A, 0x0B65, 0x1C60, 0x3276, 
        0x09FD, 0x19CD, 0x2293, 0x1CEC, 0x35E3, 0x0A3D, 0x1541, 0x292B, 0x11EC, 0x286E, 0x2282, 0x1867, 0x1ED9, 0x1825, 0x068F, 0x1599, 
        0x1A65, 0x2D25, 0x158B, 0x1B71, 0x22CD, 0x2754, 0x3449, 0x166E, 0x19C5, 0x1722, 0x3FEB, 0x1EFA, 0x05C5, 0x36C6, 0x15F7, 0x108D, 
        0x37E2, 0x10DE, 0x39A6, 0x2EE5, 0x3F3A, 0x3F93, 0x01D5, 0x197E, 0x36ED, 0x1926, 0x2D60, 0x06D2, 0x28A3, 0x031E, 0x2044, 0x015E, 
        0x13DC, 0x0EFE, 0x320B, 0x087B, 0x2E7D, 0x3A80, 0x08B4, 0x3FA0, 0x2FDF, 0x1D41, 0x2A56, 0x1BED, 0x01C7, 0x0833, 0x167E, 0x0F66, 
        0x29E2, 0x3CC5, 0x2CBA, 0x11B3, 0x2C9D, 0x3B3D, 0x098A, 0x1023, 0x1B7E, 0x1E5E, 0x39DD, 0x2081, 0x2CCE, 0x04BD, 0x3111, 0x1D7F, 
        0x37DF, 0x2DFB, 0x24A9, 0x2505, 0x282F, 0x2524, 0x2594, 0x294E, 0x19AF, 0x01CE, 0x3AD0, 0x201B, 0x3EFE, 0x2B04, 0x315C, 0x06B0, 
        0x061A, 0x30C9, 0x300A, 0x3AA3, 0x0621, 0x3602, 0x22C3, 0x03E2, 0x3496, 0x12BD, 0x35FB, 0x23F9, 0x0D33, 0x3152, 0x3513, 0x32F3, 
        0x0820, 0x34FA, 0x3086, 0x2479, 0x0B35, 0x06DA, 0x15E5, 0x2923, 0x2691, 0x1CA4, 0x2B3E, 0x2016, 0x2F60, 0x051F, 0x37F7, 0x0CF3, 
        0x1BCE, 0x0E16, 0x22BA, 0x040E, 0x2AA9, 0x1206, 0x12A6, 0x03F4, 0x156E, 0x01B1, 0x2D9B, 0x35FE, 0x32D4, 0x2158, 0x0E71, 0x1230, 
        0x26F6, 0x0DED, 0x349D, 0x39C8, 0x0E85, 0x3950, 0x36B5, 0x1714, 0x2FCF, 0x2EF1, 0x1B52, 0x06A2, 0x2FB9, 0x0B85, 0x163B, 0x2EF4, 
        0x23AD, 0x3597, 0x017B, 0x3F38, 0x1AC5, 0x17C1, 0x34B8, 0x0733, 0x2A0E, 0x05B0, 0x2734, 0x36BE, 0x1256, 0x2C15, 0x2611, 0x1CDD, 
        0x2278, 0x379A, 0x2E19, 0x193A, 0x1D0A, 0x3A10, 0x29A5, 0x23EB, 0x1014, 0x0656, 0x0E5B, 0x17BF, 0x285E, 0x19D2, 0x0FEA, 0x0ACD, 
        0x186C, 0x21B1, 0x20E4, 0x2733, 0x277E, 0x2C0E, 0x0135, 0x3898, 0x137F, 0x2C9C, 0x31B8, 0x144E, 0x0291, 0x0F7A, 0x2E87, 0x1F56, 
        0x04CF, 0x154C, 0x36C3, 0x3ABF, 0x0F2D, 0x2932, 0x1A39, 0x10C4, 0x0C12, 0x1B49, 0x0531, 0x294D, 0x034E, 0x3773, 0x082F, 0x35B5, 
        0x108C, 0x34C1, 0x3E4C, 0x1805, 0x2ABC, 0x1CA1, 0x3304, 0x1DB2, 0x136F, 0x0CAD, 0x25B9, 0x27CE, 0x13D7, 0x3190, 0x1277, 0x06CE, 
        0x2CDE, 0x1ECA, 0x20DB, 0x3DFF, 0x13AF, 0x2B72, 0x38EB, 0x06F2, 0x0C47, 0x0894, 0x3CFB, 0x187F, 0x05CB, 0x2EC0, 0x2603, 0x0CD8, 
        0x1A56, 0x3263, 0x3BA3, 0x0860, 0x14B0, 0x117D, 0x3457, 0x2DAA, 0x13F3, 0x2E1E, 0x0B64, 0x1B33, 0x13BE, 0x302F, 0x252F, 0x03CC, 
        0x3ED0, 0x04FF, 0x3D74, 0x2C62, 0x3FEC, 0x22E1, 0x0583, 0x20F4, 0x09EE, 0x1E4B, 0x0782, 0x21E5, 0x3183, 0x0E12, 0x01B4, 0x0922, 
        0x1693, 0x1EAB, 0x1D1E, 0x030A, 0x253C, 0x0E38, 0x2484, 0x0C88, 0x173A, 0x357C, 0x22D9, 0x1A07, 0x3EE7, 0x2A03, 0x050F, 0x2E25, 
        0x2B6C, 0x0AC2, 0x1952, 0x209C, 0x102D, 0x065E, 0x04C3, 0x128F, 0x1548, 0x1B75, 0x3BD1, 0x32AD, 0x2D7F, 0x043F, 0x3C58, 0x3D80, 
        0x1976, 0x3035, 0x0DAE, 0x3864, 0x25BE, 0x095D, 0x1B8D, 0x0F35, 0x18AF, 0x0162, 0x1464, 0x23FC, 0x3F1C, 0x2BAF, 0x1C3D, 0x2995, 
        0x3C10, 0x2821, 0x077E, 0x0716, 0x1D52, 0x0378, 0x3E65, 0x2D63, 0x2880, 0x0768, 0x055E, 0x12A9, 0x1C05, 0x3446, 0x1F47, 0x0B6B, 
        0x2689, 0x1687, 0x1434, 0x2517, 0x135D, 0x10F1, 0x3D16, 0x0D4F, 0x020C, 0x2D96, 0x2C4A, 0x3E67, 0x0DE7, 0x3655, 0x2492, 0x1B77, 
        0x215B, 0x06F1, 0x296F, 0x3921, 0x35F6, 0x1E60, 0x1FDC, 0x2831, 0x3D23, 0x1772, 0x3A48, 0x0DA4, 0x321D, 0x3E03, 0x22FF, 0x147C, 
        0x3713, 0x3693, 0x1B19, 0x1F57, 0x02DA, 0x309F, 0x0B62, 0x1840, 0x2606, 0x3488, 0x2756, 0x1196, 0x12A7, 0x0AAE, 0x10F2, 0x0A32, 
        0x197F, 0x073D, 0x3313, 0x091F, 0x1A31, 0x2CCC, 0x283F, 0x3817, 0x2FC4, 0x25B3, 0x1CE7, 0x1789, 0x1A4A, 0x2C7D, 0x3BAF, 0x32B3, 
        0x2EDB, 0x1D92, 0x2788, 0x32D9, 0x0BC9, 0x1024, 0x0805, 0x307B, 0x1870, 0x2D08, 0x332F, 0x1934, 0x34A3, 0x0637, 0x185C, 0x1D40, 
        0x0C38, 0x3075, 0x2BA6, 0x2ED1, 0x2EFE, 0x3BF7, 0x1DFE, 0x0879, 0x06C7, 0x21E3, 0x0311, 0x31EC, 0x05F0, 0x3E73, 0x3A0B, 0x074F, 
        0x12D3, 0x3E55, 0x3F2A, 0x1743, 0x2F4F, 0x23A1, 0x35F7, 0x020A, 0x23CE, 0x2F56, 0x3021, 0x17A4, 0x1E82, 0x1594, 0x34EF, 0x34F6, 
        0x15D0, 0x1F8B, 0x114D, 0x103F, 0x3876, 0x3C0D, 0x06FC, 0x0692, 0x1F51, 0x3216, 0x130D, 0x25DA, 0x295E, 0x3705, 0x2F81, 0x04CE, 
        0x349F, 0x1DE7, 0x2CA7, 0x016B, 0x11DA, 0x1A1B, 0x19B5, 0x3795, 0x1858, 0x13B2, 0x3B28, 0x37B9, 0x334B, 0x22EF, 0x0FB8, 0x19F7, 
        0x1B84, 0x3389, 0x26EF, 0x3889, 0x35BD, 0x3507, 0x2FB3, 0x0198, 0x0CAF, 0x3477, 0x3442, 0x2FBA, 0x03AA, 0x3DE2, 0x0929, 0x104A, 
        0x2964, 0x0208, 0x2C30, 0x1EEB, 0x3FDC, 0x2A09, 0x2D72, 0x0603, 0x045F, 0x26B2, 0x2C01, 0x19F5, 0x2B52, 0x28EF, 0x37D0, 0x2395, 
        0x1C1C, 0x35DE, 0x252A, 0x348A, 0x3C06, 0x158C, 0x0A9A, 0x1C28, 0x09FB, 0x0E96, 0x192B, 0x2E76, 0x308C, 0x0207, 0x0C20, 0x2BC7, 
        0x01DB, 0x1343, 0x3160, 0x31A0, 0x0590, 0x22D0, 0x315A, 0x31BF, 0x0277, 0x0A1C, 0x07E1, 0x09DD, 0x1007, 0x1984, 0x1E02, 0x234D, 
        0x1385, 0x16DE, 0x16B7, 0x2C47, 0x090F, 0x0DB4, 0x1325, 0x1130, 0x23F7, 0x0CB2, 0x1CC1, 0x17AE, 0x0E9C, 0x3AF4, 0x0D82, 0x312A, 
        0x2E7F, 0x3EF0, 0x3BD4, 0x2BEA, 0x120C, 0x3681, 0x2B88, 0x1A90, 0x2E7C, 0x3F6B, 0x1E7C, 0x3D7F, 0x38F8, 0x2DF6, 0x3515, 0x2244, 
        0x3299, 0x2136, 0x23E6, 0x2C71, 0x3B30, 0x0346, 0x18BF, 0x0109, 0x1CFB, 0x3E44, 0x037D, 0x3B44, 0x0D3B, 0x1C4A, 0x3139, 0x13D5, 
        0x2C29, 0x068C, 0x0C25, 0x0BE7, 0x115B, 0x0C8F, 0x1114, 0x31E8, 0x26D9, 0x3CD6, 0x3F6F, 0x1FFC, 0x05DB, 0x27F8, 0x2FE5, 0x11A5, 
        0x3482, 0x27E5, 0x2DBB, 0x3339, 0x38C8, 0x3384, 0x3558, 0x09D1, 0x3DF5, 0x1AB4, 0x2C89, 0x0D43, 0x19EA, 0x0F1C, 0x1042, 0x08D7, 
        0x3FB4, 0x3A25, 0x3C26, 0x07C8, 0x0818, 0x11E9, 0x19EC, 0x178E, 0x2AEA, 0x0614, 0x0D09, 0x2A73, 0x268D, 0x1519, 0x394B, 0x047C, 
        0x3399, 0x2A6B, 0x165B, 0x2225, 0x3062, 0x1287, 0x1B0B, 0x2AAA, 0x02E8, 0x1C69, 0x0325, 0x1744, 0x1DE9, 0x1D2A, 0x2DD3, 0x24D7, 
        0x018C, 0x2741, 0x16E8, 0x3812, 0x3395, 0x2BFE, 0x0122, 0x0704, 0x13EF, 0x1FC0, 0x01D8, 0x34F7, 0x323B, 0x2C51, 0x2080, 0x1C06, 
        0x1DE4, 0x3EFD, 0x377A, 0x161F, 0x17DF, 0x1971, 0x16B8, 0x11CF, 0x2B76, 0x1420, 0x0129, 0x1F85, 0x17C6, 0x1ED4, 0x1BF4, 0x09D9, 
        0x2AA8, 0x3333, 0x3ED6, 0x0426, 0x0FDC, 0x3638, 0x1C16, 0x1353, 0x3C7A, 0x3A82, 0x1835, 0x3404, 0x0142, 0x13F2, 0x39B0, 0x0BDC, 
        0x0C1D, 0x230C, 0x36A2, 0x3096, 0x0BF0, 0x3329, 0x34FC, 0x2652, 0x1F5A, 0x32B8, 0x1CAC, 0x22CB, 0x238B, 0x0FAC, 0x02C4, 0x277F, 
        0x0A29, 0x3965, 0x34B0, 0x0B2C, 0x0658, 0x211C, 0x3171, 0x1C65, 0x1323, 0x123E, 0x2CEB, 0x0433, 0x341F, 0x3905, 0x0A8A, 0x337F, 
        0x13B7, 0x1C31, 0x181D, 0x1777, 0x0302, 0x3F41, 0x1605, 0x0EE3, 0x0224, 0x26C5, 0x3041, 0x0A5F, 0x0679, 0x30CD, 0x2C1F, 0x198D, 
        0x0EA0, 0x307C, 0x08A8, 0x2CF6, 0x3656, 0x103E, 0x099E, 0x38DC, 0x3964, 0x3B7B, 0x0988, 0x2CF3, 0x0424, 0x22B0, 0x3F1A, 0x1D87, 
        0x1A8F, 0x0DE5, 0x2E65, 0x03B8, 0x0106, 0x1D5A, 0x29ED, 0x2D6C, 0x2189, 0x381B, 0x27E0, 0x1E6B, 0x185A, 0x2187, 0x3E05, 0x08DB, 
        0x03AC, 0x100E, 0x0F29, 0x20B4, 0x3FBE, 0x0E3E, 0x339C, 0x2DEF, 0x2AC6, 0x057D, 0x0646, 0x3C15, 0x2E88, 0x207F, 0x1BF2, 0x1220, 
        0x11E1, 0x373B, 0x3305, 0x03F5, 0x255A, 0x1954, 0x0C04, 0x0F4C, 0x21BF, 0x0B11, 0x3A09, 0x2B57, 0x3DF4, 0x1E0C, 0x3AD9, 0x13AB, 
        0x2617, 0x3FD4, 0x2362, 0x24DD, 0x082A, 0x3203, 0x2DE6, 0x0D59, 0x0465, 0x0CB9, 0x1F38, 0x3E5D, 0x3FEE, 0x1DD3, 0x3DCC, 0x3A85, 
        0x1F87, 0x25DB, 0x3B3A, 0x2D50, 0x38BB, 0x13BB, 0x3201, 0x2C6A, 0x0898, 0x11C3, 0x2960, 0x25AB, 0x2390, 0x1E2F, 0x0CFC, 0x28AF, 
        0x2490, 0x2EA9, 0x2118, 0x32C7, 0x0BCA, 0x3D96, 0x10EE, 0x1F7E, 0x2AF7, 0x38FD, 0x21D2, 0x3D14, 0x21C7, 0x0895, 0x3A63, 0x0B37, 
        0x1E70, 0x0D6F, 0x3575, 0x1C0C, 0x245B, 0x0A50, 0x1C8B, 0x24AE, 0x1292, 0x3AE3, 0x13C7, 0x282E, 0x29F6, 0x3115, 0x1F4C, 0x0C69, 
        0x01BA, 0x1B0C, 0x3F5E, 0x0C64, 0x2427, 0x3283, 0x0FCD, 0x373D, 0x2DCA, 0x3D50, 0x1415, 0x3153, 0x0B5C, 0x1B35, 0x219C, 0x17BA, 
        0x019F, 0x3BD7, 0x1DD9, 0x0878, 0x1439, 0x2C1C, 0x246A, 0x0E08, 0x0990, 0x30EC, 0x1134, 0x0D7B, 0x2FA1, 0x39A3, 0x1EA9, 0x246B, 
        0x2E32, 0x0442, 0x0545, 0x2989, 0x087F, 0x286A, 0x3137, 0x2BE9, 0x12C5, 0x0752, 0x3215, 0x0CCF, 0x2B16, 0x3095, 0x350F, 0x12C9, 
        0x38F9, 0x3254, 0x0276, 0x1DC1, 0x345C, 0x362A, 0x3BAB, 0x0364, 0x22CE, 0x117E, 0x1CA9, 0x196E, 0x3958, 0x0E46, 0x1EC2, 0x3256, 
        0x1330, 0x01D0, 0x27F2, 0x2B70, 0x06A9, 0x3409, 0x2B5C, 0x23F8, 0x2979, 0x1D25, 0x02CB, 0x264F, 0x1E75, 0x3869, 0x1501, 0x18B1, 
        0x1F22, 0x1CF7, 0x3270, 0x1BA3, 0x3EB9, 0x0FF1, 0x1820, 0x296D, 0x113E, 0x0BA6, 0x1B7D, 0x0FE9, 0x26E5, 0x1B57, 0x3C2F, 0x3D63, 
        0x1814, 0x335C, 0x234C, 0x1248, 0x382C, 0x2712, 0x23DA, 0x3D4D, 0x0EA6, 0x2675, 0x30A1, 0x2421, 0x34D2, 0x258A, 0x08BE, 0x28EA, 
        0x35BA, 0x1815, 0x2A76, 0x0EAB, 0x33D8, 0x0BBE, 0x2DB6, 0x3A01, 0x33ED, 0x1639, 0x044C, 0x0131, 0x1454, 0x1318, 0x31A3, 0x29E4, 
        0x1489, 0x1F26, 0x0DD5, 0x0999, 0x1C6D, 0x3274, 0x3A0C, 0x173D, 0x03E3, 0x39DE, 0x0212, 0x391F, 0x254E, 0x389A, 0x2FCA, 0x045A, 
        0x13C9, 0x105F, 0x0A7B, 0x0511, 0x3CD1, 0x0CE9, 0x219F, 0x35DD, 0x0AAD, 0x3D82, 0x348D, 0x186B, 0x2E17, 0x2532, 0x3B82, 0x1A95, 
        0x08AC, 0x3AE5, 0x274D, 0x155F, 0x389B, 0x288E, 0x0B00, 0x2EF8, 0x14AA, 0x22A2, 0x0B89, 0x2EF7, 0x0840, 0x16FA, 0x2327, 0x30CB, 
        0x2564, 0x27DF, 0x22E5, 0x13D1, 0x1A16, 0x0F06, 0x2918, 0x237C, 0x3699, 0x1795, 0x2936, 0x3E4B, 0x36B8, 0x1A5F, 0x3F1B, 0x0BE6, 
        0x3F4F, 0x0A57, 0x30D7, 0x2B55, 0x2330, 0x0502, 0x05F2, 0x25C8, 0x2B38, 0x2344, 0x1782, 0x384A, 0x37B2, 0x2D30, 0x3A04, 0x2268, 
        0x1B09, 0x371A, 0x12EA, 0x1D36, 0x3A4F, 0x1563, 0x1081, 0x0764, 0x2C74, 0x3407, 0x24AF, 0x03FE, 0x3752, 0x25B5, 0x2ADF, 0x1FC2, 
        0x298C, 0x2889, 0x3EA1, 0x2EB2, 0x2269, 0x2383, 0x12B4, 0x10F6, 0x1A92, 0x23D9, 0x3BF6, 0x204B, 0x0226, 0x0C6F, 0x221B, 0x395F, 
        0x1C49, 0x370B, 0x02AC, 0x337C, 0x1F6E, 0x2112, 0x27C8, 0x0CFA, 0x170C, 0x1DCD, 0x0EB8, 0x3239, 0x3412, 0x26A7, 0x2F38, 0x2725, 
        0x334F, 0x1162, 0x3986, 0x346C, 0x0FC6, 0x0E9B, 0x28C3, 0x29D0, 0x021C, 0x198E, 0x3A9C, 0x1EC3, 0x232B, 0x0F88, 0x345E, 0x2F30, 
        0x28CA, 0x1CDC, 0x2835, 0x0356, 0x372B, 0x3124, 0x362C, 0x30EF, 0x0960, 0x3D9B, 0x38C7, 0x3843, 0x217F, 0x18A7, 0x1C80, 0x1015, 
        0x157B, 0x0A61, 0x2CB1, 0x3291, 0x30B3, 0x2B66, 0x167B, 0x117A, 0x0E5F, 0x1670, 0x2AEB, 0x1332, 0x101C, 0x1A80, 0x15FC, 0x3D39, 
        0x295D, 0x3CF6, 0x3E50, 0x01C6, 0x2612, 0x24F4, 0x20BA, 0x01FB, 0x36F7, 0x3637, 0x1F4B, 0x36EE, 0x02D3, 0x1FA5, 0x1C5B, 0x36A7, 
        0x01DC, 0x3890, 0x05FA, 0x2497, 0x104F, 0x0920, 0x3130, 0x3BBA, 0x140A, 0x305B, 0x058B, 0x10C1, 0x3718, 0x021E, 0x0F2C, 0x3743, 
        0x1EA0, 0x0926, 0x300C, 0x1A99, 0x3192, 0x32E5, 0x2015, 0x3277, 0x0737, 0x1970, 0x0F5F, 0x0955, 0x14E2, 0x3ADB, 0x1A71, 0x21EA, 
        0x3F14, 0x2798, 0x3783, 0x1101, 0x360A, 0x3427, 0x30E3, 0x2AB8, 0x2E02, 0x3F96, 0x2007, 0x38CF, 0x0F97, 0x1807, 0x0D3A, 0x37C4, 
        0x07D2, 0x3A5E, 0x2218, 0x055A, 0x3AE2, 0x2119, 0x2480, 0x14EE, 0x0EF5, 0x3326, 0x1E4E, 0x2F9D, 0x176B, 0x080F, 0x0823, 0x1165, 
        0x290C, 0x3940, 0x3C9C, 0x0A9B, 0x246D, 0x29A7, 0x2EC8, 0x022A, 0x3F80, 0x1A11, 0x2B3D, 0x33D5, 0x0F38, 0x1378, 0x2F83, 0x0287, 
        0x26C4, 0x066C, 0x0A87, 0x01B0, 0x3557, 0x2D92, 0x27FE, 0x0DB2, 0x34E9, 0x195D, 0x336E, 0x3868, 0x1BAE, 0x0A11, 0x122F, 0x3EB3, 
        0x0C14, 0x3031, 0x3107, 0x1C32, 0x0C2C, 0x2A02, 0x384F, 0x3408, 0x1828, 0x2E71, 0x0D23, 0x3B4B, 0x267B, 0x180C, 0x0CCA, 0x2C8B, 
        0x27F0, 0x31DB, 0x0B10, 0x1276, 0x34BA, 0x0E15, 0x14D5, 0x04B9, 0x37B5, 0x2802, 0x12D8, 0x34D9, 0x3B81, 0x394D, 0x3C2B, 0x1C3C, 
        0x3FE4, 0x1416, 0x067E, 0x1A55, 0x2BE8, 0x09AD, 0x2663, 0x04AE, 0x0385, 0x316A, 0x2D82, 0x1E85, 0x2331, 0x351C, 0x193F, 0x29DD, 
        0x1EE3, 0x09A0, 0x39EB, 0x3BD0, 0x06C3, 0x0EC4, 0x3396, 0x036F, 0x241B, 0x2047, 0x0245, 0x39D2, 0x074D, 0x11C5, 0x3331, 0x25BF, 
        0x2CB0, 0x0B0A, 0x1C1E, 0x3BEC, 0x1E36, 0x374C, 0x16C2, 0x1CEE, 0x3FCE, 0x39A4, 0x1121, 0x21EE, 0x1A6D, 0x0216, 0x3B09, 0x2CFB, 
        0x1B24, 0x08E6, 0x302A, 0x3E90, 0x2950, 0x1A4C, 0x12BB, 0x1EAD, 0x1517, 0x3AC0, 0x0BA9, 0x1197, 0x0A15, 0x3FDE, 0x0D1E, 0x137E, 
        0x308D, 0x35E2, 0x228A, 0x2EEC, 0x12D6, 0x14E1, 0x350B, 0x154B, 0x1DD7, 0x059B, 0x1E5B, 0x0CD9, 0x1997, 0x365C, 0x179A, 0x04AD, 
        0x2BD0, 0x1F3D, 0x1142, 0x1625, 0x28B3, 0x0E76, 0x295A, 0x062C, 0x2280, 0x15A0, 0x2F79, 0x21C1, 0x3B63, 0x0A5A, 0x09F3, 0x3895, 
        0x0A33, 0x261B, 0x2FE7, 0x0355, 0x15CE, 0x0436, 0x3244, 0x0FA3, 0x1F09, 0x0E8D, 0x0856, 0x367B, 0x1E4C, 0x1D75, 0x07C9, 0x3EAD, 
        0x0C0D, 0x1C62, 0x074B, 0x3186, 0x3C67, 0x30FC, 0x0A76, 0x1250, 0x350D, 0x09A8, 0x1CA0, 0x0C3B, 0x20EB, 0x0EDA, 0x192C, 0x0905, 
        0x116F, 0x23BA, 0x1DAD, 0x32DF, 0x13A5, 0x2F8D, 0x3EB7, 0x23B6, 0x228C, 0x120D, 0x024B, 0x2988, 0x2EAE, 0x020E, 0x3540, 0x2A58, 
        0x1B4D, 0x2A67, 0x0E45, 0x0401, 0x0DCA, 0x3D54, 0x0F2A, 0x1F76, 0x3583, 0x0BB5, 0x344D, 0x3978, 0x3797, 0x097F, 0x3F52, 0x1AFC, 
        0x0F9A, 0x2F61, 0x1A78, 0x2730, 0x2AD6, 0x25AF, 0x0A5B, 0x39F9, 0x15A1, 0x2277, 0x1E22, 0x1F10, 0x2087, 0x2816, 0x2577, 0x0549, 
        0x37C7, 0x0A6B, 0x0D01, 0x3406, 0x3BB5, 0x1B76, 0x33D9, 0x2265, 0x0D5D, 0x2041, 0x3D02, 0x3640, 0x2534, 0x3A91, 0x04DF, 0x118C, 
        0x3B14, 0x0CFB, 0x15FF, 0x340E, 0x0CB4, 0x3D7E, 0x3508, 0x2C88, 0x1F74, 0x32C0, 0x1C00, 0x19FA, 0x1EE9, 0x1923, 0x0E73, 0x2F18, 
        0x2693, 0x0729, 0x1090, 0x06E3, 0x186E, 0x0857, 0x32D6, 0x1FEC, 0x3DE1, 0x0DFE, 0x323E, 0x2C39, 0x2224, 0x1E5F, 0x2291, 0x355A, 
        0x057F, 0x0CD0, 0x1AC3, 0x13FF, 0x1AFB, 0x241D, 0x1FCE, 0x0D52, 0x1761, 0x1B60, 0x316E, 0x3E1B, 0x2AA7, 0x29AA, 0x35B7, 0x11F0, 
        0x166F, 0x16E7, 0x1F1D, 0x1CFC, 0x2576, 0x2DCE, 0x3E08, 0x02C9, 0x2BDE, 0x23FB, 0x3258, 0x3D37, 0x120B, 0x2FDA, 0x2955, 0x06E7, 
        0x1E9C, 0x051E, 0x3E52, 0x3314, 0x16C5, 0x30CE, 0x2B44, 0x2105, 0x290E, 0x310B, 0x1284, 0x1834, 0x19F0, 0x2BC0, 0x2DB7, 0x19C0, 
        0x3B57, 0x3259, 0x357E, 0x1C30, 0x15C0, 0x390C, 0x1765, 0x300E, 0x0AF9, 0x3BF1, 0x3168, 0x2164, 0x22A0, 0x1B34, 0x24FA, 0x1149, 
        0x0348, 0x0400, 0x278B, 0x2C00, 0x08C0, 0x33CE, 0x3112, 0x1ED2, 0x23D4, 0x248E, 0x392D, 0x0FD6, 0x0FDF, 0x1C0F, 0x287B, 0x1F6B, 
        0x2DF8, 0x17F6, 0x2F8C, 0x2BDD, 0x2FA4, 0x1AB9, 0x0780, 0x157C, 0x0354, 0x20CC, 0x28D0, 0x336C, 0x0F9D, 0x1905, 0x3EB8, 0x1767, 
        0x1F70, 0x2967, 0x301D, 0x101B, 0x1A9F, 0x27EC, 0x1CC5, 0x37B0, 0x2A41, 0x35F8, 0x3029, 0x36C4, 0x0F2F, 0x36DD, 0x3A77, 0x3871, 
        0x299C, 0x3DF8, 0x135E, 0x082B, 0x1C3E, 0x057C, 0x3D86, 0x2246, 0x2D2C, 0x0B33, 0x0D05, 0x3F94, 0x1554, 0x39E0, 0x38D6, 0x3FF8, 
        0x2968, 0x2BB2, 0x21E1, 0x13A6, 0x3614, 0x1F91, 0x0557, 0x2F15, 0x2915, 0x23DE, 0x0BD4, 0x152C, 0x39F0, 0x2E10, 0x303E, 0x066D, 
        0x2876, 0x24DC, 0x31DA, 0x209F, 0x32DA, 0x1ADA, 0x0A49, 0x078B, 0x160A, 0x2D97, 0x2B1E, 0x18D4, 0x0A74, 0x278C, 0x08CB, 0x3441, 
        0x1A87, 0x2020, 0x1453, 0x111A, 0x0157, 0x3C38, 0x2B90, 0x2A84, 0x18E0, 0x3524, 0x11D2, 0x2051, 0x353E, 0x133D, 0x1681, 0x3C3C, 
        0x0950, 0x14E4, 0x27CC, 0x39E4, 0x30E4, 0x10C7, 0x1C19, 0x0532, 0x110F, 0x0353, 0x01CB, 0x1864, 0x3D24, 0x2B7A, 0x359D, 0x09BD, 
        0x1A29, 0x11F9, 0x1A12, 0x1EAA, 0x243E, 0x17B1, 0x2C09, 0x1476, 0x28CC, 0x277D, 0x39D0, 0x20F9, 0x2C03, 0x0D4D, 0x1979, 0x15FD, 
        0x2BC6, 0x1095, 0x03A4, 0x2D67, 0x168F, 0x3E54, 0x0A2A, 0x317E, 0x27AE, 0x0ECF, 0x3E9B, 0x21AE, 0x010F, 0x24C5, 0x34F0, 0x3577, 
        0x3BB1, 0x092D, 0x2D7C, 0x275A, 0x03B2, 0x1C86, 0x2333, 0x3781, 0x2401, 0x3BF5, 0x2AF2, 0x310C, 0x05B4, 0x04FC, 0x2100, 0x2E22, 
        0x326A, 0x32A9, 0x15AC, 0x31F4, 0x33CB, 0x1F2D, 0x2E77, 0x2171, 0x3E6C, 0x2370, 0x34F9, 0x11DF, 0x30C8, 0x1CFD, 0x3250, 0x1930, 
        0x1EEA, 0x2B21, 0x3D03, 0x153B, 0x07BA, 0x091B, 0x3B27, 0x246C, 0x2924, 0x0B71, 0x04B5, 0x3194, 0x0DCC, 0x3F0C, 0x3E5A, 0x1BB9, 
        0x2E4D, 0x075E, 0x27ED, 0x0652, 0x1EB2, 0x0258, 0x1A58, 0x3F28, 0x239A, 0x1A4E, 0x2C6F, 0x205A, 0x35A9, 0x2B02, 0x21F4, 0x02B1, 
        0x2ABB, 0x3DFC, 0x2E8E, 0x22F5, 0x0FFE, 0x1575, 0x0649, 0x1ABA, 0x1C0D, 0x376F, 0x3ABD, 0x27EB, 0x0CBC, 0x38EF, 0x0E6E, 0x2B0D, 
        0x2901, 0x24F5, 0x0F96, 0x1868, 0x19E2, 0x2943, 0x0ECC, 0x3F24, 0x3C22, 0x0FF9, 0x3903, 0x0B51, 0x2EA3, 0x171E, 0x2CDD, 0x041C, 
        0x1041, 0x07AD, 0x03DE, 0x2099, 0x31B5, 0x0519, 0x15BE, 0x1F73, 0x3ECE, 0x129D, 0x0C68, 0x3CBD, 0x34DF, 0x1C5E, 0x0727, 0x3666, 
        0x2687, 0x1422, 0x1E09, 0x15CC, 0x2FB0, 0x0B79, 0x38B6, 0x2C95, 0x26D6, 0x3FC1, 0x18E5, 0x26AC, 0x3769, 0x041A, 0x282D, 0x0725, 
        0x3DD9, 0x3122, 0x3790, 0x1243, 0x3B3E, 0x2997, 0x28EB, 0x0143, 0x3528, 0x0B6D, 0x1371, 0x3A8B, 0x0DE3, 0x2EFC, 0x1380, 0x3D5A, 
        0x0CBF, 0x055B, 0x0FB9, 0x0F49, 0x3C87, 0x399F, 0x2573, 0x2EEA, 0x17BC, 0x1A6B, 0x38E4, 0x1DDA, 0x02B3, 0x195F, 0x20F2, 0x392A, 
        0x338A, 0x2605, 0x2D83, 0x12BE, 0x2F88, 0x3294, 0x3F3E, 0x265F, 0x24CA, 0x0520, 0x3909, 0x188E, 0x131E, 0x205B, 0x1678, 0x038F, 
        0x367D, 0x2F91, 0x303D, 0x0705, 0x0787, 0x09BE, 0x05FD, 0x396C, 0x0C32, 0x04B7, 0x3CA1, 0x3F55, 0x2145, 0x3EEE, 0x2313, 0x0FCB, 
        0x107D, 0x2E43, 0x309C, 0x1F9B, 0x0B5A, 0x1646, 0x3E7F, 0x2352, 0x2391, 0x2D02, 0x063A, 0x2908, 0x1E31, 0x0C22, 0x1412, 0x3C47, 
        0x0C4C, 0x1CD1, 0x04AF, 0x2530, 0x153C, 0x39F1, 0x3481, 0x17F2, 0x1C4D, 0x3289, 0x0521, 0x2685, 0x2E5F, 0x21D1, 0x130B, 0x1A3E, 
        0x20EA, 0x3BB6, 0x26A6, 0x14B2, 0x243F, 0x0FB0, 0x14C4, 0x3734, 0x33EF, 0x3E1A, 0x2009, 0x260A, 0x1F5F, 0x04DC, 0x3E7A, 0x044F, 
        0x0993, 0x256D, 0x2388, 0x2245, 0x0423, 0x02DC, 0x26FF, 0x2359, 0x220E, 0x23F4, 0x3716, 0x0F1B, 0x1FF7, 0x3541, 0x2B98, 0x267E, 
        0x1913, 0x3837, 0x0A64, 0x1002, 0x09AF, 0x03D8, 0x05B6, 0x21E8, 0x2856, 0x1E94, 0x3C28, 0x2C0A, 0x1BD3, 0x3F00, 0x0DFF, 0x2865, 
        0x2CD5, 0x3938, 0x03C3, 0x3DE9, 0x2645, 0x3A8E, 0x3162, 0x0A7E, 0x097E, 0x1351, 0x0DE2, 0x084F, 0x3312, 0x1C21, 0x1BF7, 0x152E, 
        0x1F1A, 0x3820, 0x159E, 0x2DE5, 0x14E8, 0x0CDB, 0x3B0C, 0x14A1, 0x257D, 0x33A7, 0x257F, 0x2EB6, 0x314A, 0x31D0, 0x1311, 0x1046, 
        0x0D3C, 0x059F, 0x3351, 0x38CC, 0x2AA4, 0x39D1, 0x02F0, 0x28F7, 0x0DAA, 0x31C7, 0x34C2, 0x1755, 0x09A6, 0x30CC, 0x048C, 0x106C, 
        0x2660, 0x38E5, 0x2F2E, 0x01FF, 0x3E49, 0x36E3, 0x0548, 0x3F4E, 0x20BB, 0x38F3, 0x337A, 0x141E, 0x1582, 0x1627, 0x32D7, 0x07F2, 
        0x22C5, 0x2F2F, 0x1BBE, 0x29A2, 0x138A, 0x1DC6, 0x0130, 0x079B, 0x2DCF, 0x249A, 0x2E61, 0x0AA3, 0x1C39, 0x0144, 0x28DE, 0x3A39, 
        0x3A84, 0x3380, 0x2175, 0x2DA9, 0x1ED5, 0x1793, 0x1005, 0x114C, 0x2A1B, 0x0180, 0x244B, 0x11E2, 0x1C9C, 0x325C, 0x3D8A, 0x1C1B, 
        0x12E8, 0x0172, 0x0428, 0x312F, 0x3561, 0x19D4, 0x319F, 0x3544, 0x146C, 0x175E, 0x3BE0, 0x20D8, 0x187B, 0x197B, 0x33A9, 0x3278, 
        0x1B6E, 0x0F22, 0x3C92, 0x16D6, 0x060E, 0x393C, 0x3164, 0x0A7D, 0x114E, 0x1DD0, 0x1C50, 0x229D, 0x26C9, 0x3DE0, 0x01A5, 0x0DBA, 
        0x04D8, 0x259F, 0x3782, 0x23F1, 0x3EC5, 0x3AAD, 0x26DA, 0x1A6F, 0x0B20, 0x1A41, 0x204D, 0x2BDB, 0x0205, 0x1D0C, 0x1B2C, 0x3BA8, 
        0x0183, 0x09B6, 0x2735, 0x2AF4, 0x23C7, 0x3FAF, 0x177E, 0x1428, 0x3157, 0x3CF4, 0x3E46, 0x1172, 0x04AB, 0x0D29, 0x389E, 0x2FF3, 
        0x36F8, 0x28F1, 0x097B, 0x3D3E, 0x1EA7, 0x331E, 0x134B, 0x392E, 0x2977, 0x05A9, 0x2AC0, 0x1269, 0x0155, 0x022D, 0x30E6, 0x24C1, 
        0x143C, 0x2813, 0x2F50, 0x225E, 0x0B4E, 0x2BED, 0x1593, 0x06D7, 0x3D65, 0x2B7F, 0x28D7, 0x2C34, 0x0FFA, 0x3853, 0x0EC2, 0x0CD6, 
        0x0A6C, 0x2CA8, 0x38AF, 0x0509, 0x39EC, 0x0C13, 0x0319, 0x3DDA, 0x0B8E, 0x2E5D, 0x368C, 0x1262, 0x3D36, 0x2613, 0x1D43, 0x1105, 
        0x1FAA, 0x09B8, 0x30CA, 0x2FC9, 0x03D1, 0x0F2B, 0x3DC7, 0x1F23, 0x1B16, 0x0510, 0x1ADB, 0x1009, 0x1A52, 0x081C, 0x3E76, 0x0916, 
        0x02D4, 0x024E, 0x0853, 0x1B79, 0x3321, 0x1EE4, 0x0225, 0x2895, 0x05E6, 0x0F81, 0x0416, 0x31B1, 0x24B1, 0x266F, 0x0CEF, 0x2716, 
        0x19D7, 0x03F8, 0x0795, 0x2423, 0x25A9, 0x15EA, 0x2666, 0x047D, 0x1E57, 0x247A, 0x04EF, 0x3DD1, 0x39EA, 0x2BA9, 0x235F, 0x34B1, 
        0x3E8C, 0x3D3C, 0x208D, 0x0B84, 0x3F35, 0x38BD, 0x058A, 0x1E3A, 0x3316, 0x0FEE, 0x118D, 0x1D42, 0x2300, 0x3799, 0x1B2E, 0x01D1, 
        0x2E2D, 0x1397, 0x31F5, 0x3BA6, 0x1EBB, 0x0328, 0x2817, 0x0F58, 0x25CA, 0x3BB0, 0x3EF5, 0x3F5C, 0x3280, 0x1A1A, 0x189F, 0x1469, 
        0x0102, 0x0F82, 0x14B5, 0x3F50, 0x204C, 0x357F, 0x1569, 0x363B, 0x3924, 0x070C, 0x18CB, 0x3FA4, 0x2340, 0x08E0, 0x1266, 0x23DC, 
        0x24B0, 0x19A0, 0x2E60, 0x0E54, 0x1616, 0x2D1B, 0x29B1, 0x31B0, 0x0E64, 0x3634, 0x0410, 0x1865, 0x1013, 0x07DB, 0x12D0, 0x2B99, 
        0x36C2, 0x23E5, 0x0108, 0x0CA7, 0x3807, 0x20D6, 0x1F9A, 0x0EF8, 0x38E9, 0x1D64, 0x34D5, 0x2A93, 0x3509, 0x31E3, 0x2739, 0x0C17, 
        0x3B33, 0x0718, 0x0A69, 0x06F3, 0x1D0E, 0x1D5D, 0x0618, 0x3489, 0x1C15, 0x0CA9, 0x3A03, 0x211B, 0x122C, 0x0CC1, 0x1589, 0x1CB1, 
        0x0CB0, 0x1EE1, 0x3592, 0x3B6E, 0x3468, 0x108F, 0x3F54, 0x3DDD, 0x189D, 0x3266, 0x03AE, 0x12E9, 0x394C, 0x0D5E, 0x0A2E, 0x2262, 
        0x09E0, 0x059E, 0x356E, 0x14DE, 0x3695, 0x20B7, 0x1D88, 0x2DAB, 0x1B65, 0x3A6C, 0x2159, 0x30C1, 0x039F, 0x0ED5, 0x27CD, 0x3E2F, 
        0x34F5, 0x2D2E, 0x2208, 0x0BA0, 0x22A3, 0x2203, 0x08BB, 0x1933, 0x3CE2, 0x3005, 0x1CF4, 0x271B, 0x25E1, 0x11BD, 0x02EC, 0x3B50, 
        0x19BA, 0x1ADF, 0x15B0, 0x179F, 0x2B83, 0x1AFD, 0x0BEB, 0x2883, 0x14D8, 0x3987, 0x0612, 0x2B11, 0x1DF7, 0x23FE, 0x1037, 0x06A7, 
        0x1A5C, 0x26D1, 0x3E89, 0x0F24, 0x1737, 0x02FB, 0x2C84, 0x3A5B, 0x124B, 0x1BC9, 0x1E8F, 0x35D4, 0x0B6F, 0x1DAB, 0x0E41, 0x0DD0, 
        0x3B36, 0x2194, 0x0D6C, 0x3E60, 0x058F, 0x33C1, 0x324D, 0x38AB, 0x1546, 0x1328, 0x257B, 0x2704, 0x23F0, 0x07D0, 0x095E, 0x2567, 
        0x1B2F, 0x2E41, 0x10E9, 0x14ED, 0x1F44, 0x3499, 0x34AA, 0x1BCD, 0x0767, 0x375C, 0x1CE9, 0x245A, 0x26D5, 0x015A, 0x0310, 0x3796, 
        0x2F58, 0x0E21, 0x19BD, 0x2DDB, 0x11E5, 0x2AD9, 0x34D3, 0x3FF6, 0x33DF, 0x0F7E, 0x02AF, 0x0F87, 0x13F1, 0x2893, 0x28B5, 0x3997, 
        0x1482, 0x23B5, 0x3FA9, 0x11B0, 0x34F4, 0x1BBD, 0x306F, 0x2D3B, 0x14F2, 0x20D1, 0x3882, 0x3639, 0x0C2A, 0x1EDA, 0x14D3, 0x38DD, 
        0x0421, 0x1F7B, 0x1819, 0x1DC9, 0x2676, 0x3DBF, 0x13A4, 0x0DC7, 0x31F2, 0x11B5, 0x0B58, 0x17FB, 0x1945, 0x3208, 0x3EF4, 0x3E69, 
        0x3DEC, 0x0AE8, 0x0D21, 0x0223, 0x3A83, 0x27BB, 0x013C, 0x0E2D, 0x03A8, 0x2C2B, 0x1E80, 0x3AE7, 0x3759, 0x3926, 0x031A, 0x3268, 
        0x1555, 0x06EE, 0x1187, 0x042B, 0x3549, 0x3AA6, 0x3B08, 0x21E7, 0x2F8A, 0x0A95, 0x0336, 0x0BDD, 0x2E67, 0x12A2, 0x117B, 0x245F, 
        0x162C, 0x19E8, 0x131A, 0x3A07, 0x2B42, 0x1B3A, 0x2304, 0x1BDC, 0x0B8D, 0x2CC7, 0x1449, 0x1B9F, 0x2899, 0x3188, 0x2DB9, 0x10BB, 
        0x2884, 0x3458, 0x3C6F, 0x1F18, 0x1228, 0x3F97, 0x1241, 0x3886, 0x2FDB, 0x2D41, 0x27D3, 0x2920, 0x1574, 0x08A1, 0x2519, 0x39ED, 
        0x2781, 0x3AFF, 0x1080, 0x325F, 0x0AE3, 0x0570, 0x27B0, 0x3341, 0x20E9, 0x2E35, 0x195A, 0x14C3, 0x0132, 0x2A9F, 0x2E33, 0x22F1, 
        0x2AE9, 0x327C, 0x0AF2, 0x3C6C, 0x0AB4, 0x3A2C, 0x0981, 0x131C, 0x3234, 0x31D9, 0x2415, 0x20DF, 0x1F60, 0x2ACE, 0x2B34, 0x3CFF, 
        0x29C0, 0x2464, 0x1635, 0x2392, 0x319D, 0x1122, 0x0719, 0x3D04, 0x1921, 0x1A67, 0x1B05, 0x25E3, 0x083E, 0x2944, 0x07CA, 0x21B9, 
        0x08BD, 0x0F16, 0x07AF, 0x0A30, 0x05CE, 0x3098, 0x0591, 0x19BB, 0x2AFF, 0x2AE4, 0x18CF, 0x221A, 0x182B, 0x2D37, 0x0453, 0x3A43, 
        0x3B6A, 0x2CA2, 0x3FCC, 0x14F9, 0x12AB, 0x2DFF, 0x0B2D, 0x2089, 0x0C6D, 0x250F, 0x3C07, 0x0CE1, 0x19DF, 0x22DB, 0x3F8A, 0x037B, 
        0x1E4F, 0x02CA, 0x0AC7, 0x23B7, 0x1E25, 0x0594, 0x2131, 0x265E, 0x1239, 0x2F14, 0x159C, 0x2794, 0x0AB9, 0x11F8, 0x1B70, 0x1370, 
        0x1171, 0x27D8, 0x2CDB, 0x2BB3, 0x2859, 0x06F9, 0x29A6, 0x32FD, 0x29E6, 0x2500, 0x2CDC, 0x09F6, 0x2F7F, 0x314E, 0x1705, 0x0121, 
        0x1AC4, 0x2622, 0x150C, 0x3382, 0x32DC, 0x3E2E, 0x3CD9, 0x0452, 0x2F66, 0x259A, 0x1B66, 0x05E5, 0x0559, 0x20FC, 0x040D, 0x163D, 
        0x1232, 0x0F61, 0x2745, 0x04FE, 0x13E1, 0x0F6A, 0x2BA5, 0x2EEB, 0x367E, 0x3218, 0x3121, 0x064F, 0x3D7B, 0x2BFB, 0x2AD8, 0x2A82, 
        0x25D1, 0x15EE, 0x0EB9, 0x1DF9, 0x0C78, 0x26B0, 0x3835, 0x1308, 0x11AB, 0x1FAD, 0x3601, 0x2644, 0x24FB, 0x2191, 0x0807, 0x1624, 
        0x3827, 0x330F, 0x2A17, 0x17AC, 0x1F50, 0x313F, 0x3184, 0x34BE, 0x19D8, 0x11BF, 0x03DA, 0x077F, 0x3B38, 0x08BC, 0x3CBF, 0x2F24, 
        0x3D06, 0x03EE, 0x21F9, 0x03E4, 0x2D8C, 0x33F3, 0x1CB5, 0x160C, 0x35EB, 0x3C53, 0x14FA, 0x28F4, 0x3C2C, 0x30C3, 0x1855, 0x1A0A, 
        0x1082, 0x188C, 0x048B, 0x3397, 0x2EC4, 0x166B, 0x1E1C, 0x14C6, 0x3E15, 0x2EDD, 0x3880, 0x1FF5, 0x02F4, 0x274C, 0x3100, 0x3D1D, 
        0x1EF0, 0x23BC, 0x3A73, 0x2432, 0x1C2B, 0x1D60, 0x11D3, 0x01C5, 0x279C, 0x0FFF, 0x2F4D, 0x3F02, 0x10C0, 0x30E5, 0x0B49, 0x2BA4, 
        0x2106, 0x0564, 0x151D, 0x0474, 0x1ADC, 0x1FD8, 0x2FBD, 0x3989, 0x1FF2, 0x249E, 0x26C1, 0x0B31, 0x3AA9, 0x1962, 0x076E, 0x0DBB, 
        0x05B8, 0x3C86, 0x2309, 0x381F, 0x25DE, 0x172C, 0x0417, 0x25B0, 0x2A0A, 0x0A8F, 0x2441, 0x3F23, 0x0279, 0x0D30, 0x1313, 0x339E, 
        0x23FF, 0x2F99, 0x1258, 0x0AD3, 0x065F, 0x289B, 0x270D, 0x1A5E, 0x2EF0, 0x27DD, 0x02AE, 0x0DA5, 0x090D, 0x3EA3, 0x3BF0, 0x0F46, 
        0x20F3, 0x3622, 0x3AFA, 0x2226, 0x246F, 0x2E5A, 0x36F5, 0x383F, 0x22C7, 0x130A, 0x1A5A, 0x03C4, 0x322D, 0x241E, 0x058D, 0x1DBF, 
        0x3CCA, 0x2091, 0x261A, 0x3B49, 0x095A, 0x3D13, 0x3FE5, 0x101F, 0x1F14, 0x099C, 0x3684, 0x3495, 0x236F, 0x0816, 0x1875, 0x272A, 
        0x22FD, 0x0A04, 0x2980, 0x08A5, 0x151A, 0x2713, 0x0E27, 0x28F3, 0x062E, 0x129A, 0x3D0A, 0x229E, 0x2DBC, 0x3623, 0x061F, 0x1BF1, 
        0x3BA0, 0x39DB, 0x36B7, 0x2549, 0x11A1, 0x22DE, 0x25F9, 0x0158, 0x34B4, 0x1432, 0x2D3E, 0x21DA, 0x1891, 0x1EAE, 0x380D, 0x01BC, 
        0x0BDA, 0x2069, 0x2D07, 0x3D46, 0x06BD, 0x1911, 0x09C7, 0x2D57, 0x0C89, 0x15A5, 0x1603, 0x033F, 0x2417, 0x23BE, 0x3BBD, 0x0FCE, 
        0x12A3, 0x3CAA, 0x1402, 0x103D, 0x2387, 0x32B1, 0x3320, 0x0B08, 0x3B22, 0x0906, 0x224C, 0x37AE, 0x1A7E, 0x121B, 0x3267, 0x14F0, 
        0x3B83, 0x3B4D, 0x04C9, 0x1CB4, 0x343A, 0x2EE8, 0x1955, 0x1BBC, 0x2538, 0x0951, 0x140E, 0x2BAE, 0x23CD, 0x39F7, 0x092A, 0x0582, 
        0x0EE1, 0x02E1, 0x0164, 0x212C, 0x280A, 0x2CA5, 0x372E, 0x23C1, 0x2CD6, 0x07FE, 0x31E9, 0x3C70, 0x11FA, 0x35FD, 0x1D44, 0x1CAA, 
        0x1457, 0x32A3, 0x0FED, 0x36CE, 0x3E43, 0x1456, 0x1C54, 0x33F8, 0x02B2, 0x3C68, 0x1D6F, 0x39C9, 0x3574, 0x2D33, 0x2D14, 0x179E, 
        0x241C, 0x24D0, 0x0717, 0x1EF3, 0x09D0, 0x252C, 0x2F09, 0x3AA7, 0x0AE4, 0x1D33, 0x2241, 0x1CD5, 0x3EBD, 0x0461, 0x1126, 0x3ED4, 
        0x172D, 0x2437, 0x35CE, 0x310E, 0x057B, 0x31B9, 0x1312, 0x08C1, 0x3F9D, 0x3494, 0x108E, 0x1F1C, 0x345B, 0x32C6, 0x32EB, 0x1F31, 
        0x1D61, 0x3BD3, 0x0E6B, 0x0F64, 0x35C9, 0x2442, 0x1247, 0x3B40, 0x1E77, 0x1184, 0x39E5, 0x1760, 0x0AA7, 0x04BF, 0x0E7A, 0x076A, 
        0x20BF, 0x29C5, 0x013E, 0x2EDE, 0x1EE7, 0x2AF3, 0x16C4, 0x233F, 0x01A0, 0x3F07, 0x2F21, 0x0ABF, 0x3DD5, 0x13F0, 0x18EC, 0x3A8A, 
        0x2E5C, 0x3A6F, 0x26C2, 0x2BF8, 0x33A1, 0x0B69, 0x1C7B, 0x3148, 0x33EC, 0x16ED, 0x1D00, 0x3B19, 0x3E35, 0x1832, 0x01A9, 0x3931, 
        0x3872, 0x33CC, 0x2F40, 0x3F03, 0x3A9B, 0x3EBF, 0x306C, 0x31B3, 0x1E93, 0x20C0, 0x121F, 0x3C0A, 0x2C31, 0x20ED, 0x0975, 0x25EE, 
        0x3838, 0x0694, 0x1D15, 0x3AD7, 0x25B7, 0x20E2, 0x1AD0, 0x1B8A, 0x3D1B, 0x0E80, 0x3B7E, 0x1200, 0x221F, 0x2107, 0x0B24, 0x31B7, 
        0x07C0, 0x11AF, 0x0A4B, 0x2F59, 0x0BEA, 0x011E, 0x2A6F, 0x38AD, 0x15C3, 0x1FD6, 0x3F17, 0x3AD2, 0x2454, 0x3A23, 0x2624, 0x16AF, 
        0x2367, 0x1914, 0x0305, 0x1EEE, 0x3099, 0x1C68, 0x39F3, 0x0573, 0x21A8, 0x1417, 0x09C8, 0x3E06, 0x2E6F, 0x3E1D, 0x32A8, 0x15D9, 
        0x230D, 0x35D1, 0x23F2, 0x031B, 0x2E18, 0x0479, 0x0B9B, 0x031C, 0x3B8B, 0x2694, 0x01E3, 0x2ED7, 0x13A9, 0x1257, 0x1DFB, 0x34A9, 
        0x0A27, 0x073A, 0x390E, 0x362B, 0x38C4, 0x079E, 0x3AB8, 0x1BC4, 0x3665, 0x16FB, 0x321C, 0x33A6, 0x294B, 0x317A, 0x10D3, 0x1074, 
        0x2772, 0x0342, 0x1E55, 0x2193, 0x3155, 0x21E9, 0x0D31, 0x1533, 0x0492, 0x10BD, 0x31D4, 0x1473, 0x1B82, 0x33DB, 0x1543, 0x1099, 
        0x0DDD, 0x0E1E, 0x1940, 0x38C9, 0x34F8, 0x1B83, 0x1498, 0x3C5E, 0x1D50, 0x0BCB, 0x2DE1, 0x1F0B, 0x072B, 0x178B, 0x29B5, 0x202C, 
        0x3B00, 0x194D, 0x3A2F, 0x3AB2, 0x2A9B, 0x1AC7, 0x0FCC, 0x1E19, 0x3DCE, 0x0FEF, 0x3CAC, 0x0515, 0x2C43, 0x3DC8, 0x0300, 0x2CA9, 
        0x2651, 0x0F4D, 0x1AA1, 0x3B1B, 0x2B95, 0x3534, 0x081B, 0x1071, 0x0517, 0x302E, 0x279A, 0x0F90, 0x141C, 0x340F, 0x3887, 0x1DEF, 
        0x2481, 0x033C, 0x104C, 0x31B4, 0x096C, 0x13B9, 0x1B51, 0x3151, 0x1D54, 0x0B05, 0x07B2, 0x38D3, 0x37A0, 0x2EB0, 0x3859, 0x18DE, 
        0x16A5, 0x301E, 0x04A3, 0x1C7C, 0x02F2, 0x2FE6, 0x16DD, 0x2419, 0x11A8, 0x2795, 0x2506, 0x2146, 0x0EEA, 0x31EE, 0x2088, 0x11C4, 
        0x3E39, 0x0BF6, 0x1D86, 0x31DD, 0x28E5, 0x0A0E, 0x1260, 0x3B95, 0x08C3, 0x3E0B, 0x349A, 0x362F, 0x1F6C, 0x2787, 0x3DBE, 0x1DEB, 
        0x2D86, 0x10FD, 0x318E, 0x0C37, 0x1F2F, 0x2C0D, 0x1F39, 0x3DC5, 0x286D, 0x211A, 0x0365, 0x04BB, 0x348F, 0x3E86, 0x12F9, 0x3C75, 
        0x1813, 0x15A9, 0x3CE1, 0x25E7, 0x1748, 0x2B6D, 0x11E3, 0x155A, 0x10E1, 0x2C65, 0x2809, 0x1FC4, 0x29F9, 0x3A3F, 0x2E3F, 0x12F2, 
        0x19CE, 0x0A55, 0x1E42, 0x2626, 0x0660, 0x18F1, 0x3FF5, 0x379F, 0x0A4D, 0x24A5, 0x10A9, 0x2B74, 0x3D35, 0x2ECD, 0x35DF, 0x0A17, 
        0x1956, 0x3243, 0x3B69, 0x1BE7, 0x1001, 0x0E2B, 0x0701, 0x221C, 0x0525, 0x103B, 0x148D, 0x3491, 0x03E5, 0x308B, 0x27B5, 0x27B3, 
        0x3A7C, 0x06AA, 0x19B2, 0x262B, 0x0A7C, 0x2CD4, 0x221D, 0x08F2, 0x1048, 0x209E, 0x18FD, 0x3DA3, 0x19FE, 0x3C35, 0x25E8, 0x2C78, 
        0x1F36, 0x2633, 0x2C48, 0x0F8D, 0x0AEE, 0x0169, 0x0B75, 0x2FBE, 0x2B36, 0x3529, 0x3593, 0x107E, 0x191F, 0x3C8B, 0x293F, 0x292F, 
        0x1F5B, 0x3AD1, 0x372D, 0x2858, 0x20AE, 0x3A79, 0x19EB, 0x1BA1, 0x3E14, 0x3F01, 0x188D, 0x0577, 0x1338, 0x3944, 0x3D89, 0x079A, 
        0x0B2B, 0x3AC8, 0x3CA9, 0x1499, 0x04D2, 0x1B1F, 0x3FFA, 0x2CF9, 0x198B, 0x269C, 0x1B43, 0x3B8D, 0x29DA, 0x2082, 0x2922, 0x2DE7, 
        0x3BDD, 0x0374, 0x0493, 0x0551, 0x1201, 0x1120, 0x0F26, 0x2077, 0x1CF6, 0x223F, 0x3C71, 0x340A, 0x1F34, 0x1DDD, 0x26D4, 0x2E57, 
        0x0CF7, 0x0F78, 0x3744, 0x264C, 0x07E2, 0x193B, 0x0D6B, 0x23B9, 0x2621, 0x0C55, 0x1666, 0x197A, 0x1CD8, 0x15F4, 0x3092, 0x20C7, 
        0x3D92, 0x2D4D, 0x1550, 0x31C3, 0x340D, 0x0DAC, 0x3064, 0x34BF, 0x202D, 0x37E8, 0x1000, 0x2D98, 0x1F13, 0x39A9, 0x1AB2, 0x2A6E, 
        0x218B, 0x1CF9, 0x2ACF, 0x267D, 0x259C, 0x23FD, 0x0AA1, 0x08F7, 0x3FFE, 0x06CB, 0x05B9, 0x1ACC, 0x2F97, 0x333A, 0x2B82, 0x1363, 
        0x3B11, 0x3FBF, 0x222F, 0x1C7E, 0x10C2, 0x3421, 0x114A, 0x30D4, 0x090A, 0x219D, 0x1AE2, 0x2F67, 0x05E1, 0x35ED, 0x20FE, 0x1545, 
        0x165F, 0x08FB, 0x24F9, 0x1202, 0x3A6A, 0x20FF, 0x1C42, 0x34E2, 0x166A, 0x0536, 0x1D68, 0x2150, 0x16E0, 0x2251, 0x3E21, 0x3EFF, 
        0x3881, 0x1538, 0x31C9, 0x128E, 0x2A16, 0x2DC5, 0x0B1C, 0x13AC, 0x16F0, 0x0C28, 0x086A, 0x3D1E, 0x097D, 0x2DA2, 0x0DD9, 0x01BF, 
        0x3D4F, 0x15A2, 0x116C, 0x200D, 0x04C5, 0x0C98, 0x2D90, 0x151B, 0x3378, 0x15AD, 0x126C, 0x21CA, 0x0D72, 0x01EF, 0x026F, 0x011B, 
        0x2FD2, 0x09B7, 0x0E59, 0x19CB, 0x39A8, 0x0D57, 0x39FA, 0x1677, 0x0F0F, 0x33E2, 0x06A0, 0x2211, 0x2B79, 0x0C0C, 0x1549, 0x0A38, 
        0x3D67, 0x347D, 0x22F3, 0x2D93, 0x22A1, 0x3D47, 0x289C, 0x0C0B, 0x35EF, 0x1F80, 0x3D28, 0x010B, 0x3439, 0x171D, 0x3C48, 0x1429, 
        0x2396, 0x3F8B, 0x093B, 0x073E, 0x3DCF, 0x1212, 0x3D6F, 0x0231, 0x274A, 0x1180, 0x085B, 0x122A, 0x3587, 0x3C36, 0x3EE9, 0x3785, 
        0x092E, 0x0FD1, 0x36DB, 0x05CF, 0x3DD3, 0x2C67, 0x16C9, 0x35B4, 0x1698, 0x2A53, 0x069B, 0x24EE, 0x236C, 0x0F7D, 0x30E7, 0x0E1B, 
        0x2A72, 0x2E94, 0x2A9E, 0x3195, 0x3901, 0x210C, 0x1348, 0x10D2, 0x0A2B, 0x3F74, 0x38D1, 0x35E1, 0x133C, 0x1B54, 0x0FE4, 0x3B42, 
        0x03D3, 0x3CED, 0x08D0, 0x3669, 0x2916, 0x3BEF, 0x3EAF, 0x1A96, 0x05DC, 0x2709, 0x0BD5, 0x37C5, 0x3970, 0x368A, 0x39FE, 0x338D, 
        0x03CF, 0x206D, 0x20E5, 0x0329, 0x2098, 0x3D17, 0x3E9C, 0x3056, 0x089E, 0x08B6, 0x37A9, 0x226D, 0x28C4, 0x2B9C, 0x0D0B, 0x30D8, 
        0x3E7D, 0x0CA3, 0x0193, 0x02AA, 0x2509, 0x22D4, 0x3327, 0x0230, 0x2637, 0x0A2D, 0x219B, 0x2EA4, 0x2FB5, 0x1CFF, 0x2AC9, 0x2503, 
        0x3A1F, 0x2E46, 0x07A6, 0x29BB, 0x0235, 0x392B, 0x27BF, 0x35F2, 0x07EE, 0x3D0F, 0x2394, 0x1EC7, 0x21F6, 0x0E6A, 0x0ADC, 0x3E3A, 
        0x0DDE, 0x0FA8, 0x105B, 0x19DB, 0x39EE, 0x0A18, 0x3236, 0x1C85, 0x0204, 0x3311, 0x0307, 0x0B0E, 0x3C2A, 0x08B5, 0x33B5, 0x22E0, 
        0x0B80, 0x1272, 0x16D4, 0x054A, 0x3FA6, 0x16DA, 0x3301, 0x3393, 0x0303, 0x0B45, 0x2348, 0x29D4, 0x2E9D, 0x1D22, 0x0EEF, 0x1AA8, 
        0x0189, 0x01C3, 0x1A79, 0x0674, 0x2662, 0x2814, 0x0676, 0x3954, 0x2ABA, 0x3DD0, 0x0FC5, 0x1CBD, 0x0FF2, 0x2839, 0x220F, 0x126E, 
        0x210A, 0x15D1, 0x2275, 0x1FAB, 0x1118, 0x2E4E, 0x0578, 0x0375, 0x37AA, 0x252B, 0x2355, 0x2162, 0x33AC, 0x1211, 0x35BE, 0x10C3, 
        0x2BD9, 0x12E0, 0x0A05, 0x37F5, 0x2864, 0x3F5F, 0x1CDB, 0x18DA, 0x050B, 0x1776, 0x19B7, 0x294C, 0x342A, 0x3279, 0x08CF, 0x3FA2, 
        0x27FD, 0x17FF, 0x112D, 0x1F4E, 0x27A9, 0x0B98, 0x0CEA, 0x025B, 0x226F, 0x0A8D, 0x2803, 0x191D, 0x0599, 0x115E, 0x2E6C, 0x25B6, 
        0x365B, 0x2353, 0x39A0, 0x3802, 0x2BD5, 0x3FAB, 0x262D, 0x2657, 0x2064, 0x12B7, 0x3CD4, 0x1C5D, 0x08D9, 0x3EC0, 0x0FE6, 0x2854, 
        0x03A2, 0x38E3, 0x0114, 0x22AD, 0x39BC, 0x18FC, 0x0BB8, 0x17AA, 0x035B, 0x21A3, 0x15C7, 0x3AF2, 0x262C, 0x07CB, 0x3939, 0x298A, 
        0x3E96, 0x39CE, 0x3884, 0x09B5, 0x16B5, 0x061E, 0x1EFD, 0x310F, 0x2BC1, 0x1FCB, 0x3CAF, 0x2201, 0x25B2, 0x0425, 0x08C2, 0x311D, 
        0x2404, 0x093C, 0x05A2, 0x2148, 0x12D2, 0x1715, 0x10F8, 0x34CF, 0x2B13, 0x3B03, 0x1A24, 0x3547, 0x0871, 0x15B8, 0x1AFA, 0x2237, 
        0x2086, 0x33AE, 0x1F05, 0x13B4, 0x386F, 0x02CD, 0x2C56, 0x169E, 0x1C18, 0x2F28, 0x2D43, 0x1703, 0x136C, 0x07BC, 0x2AF9, 0x3BA9, 
        0x2B40, 0x0855, 0x1235, 0x0306, 0x1A53, 0x3BDC, 0x0358, 0x05B5, 0x0810, 0x17DD, 0x346F, 0x0F5B, 0x334E, 0x09CF, 0x0BC1, 0x2A30, 
        0x1504, 0x3B2A, 0x2718, 0x0EFB, 0x1679, 0x0730, 0x3539, 0x0F1E, 0x3CA0, 0x0518, 0x1FF9, 0x1FED, 0x3A19, 0x2F9E, 0x2823, 0x254A, 
        0x11BC, 0x17F5, 0x34E0, 0x08B8, 0x28C0, 0x0AD7, 0x3AC3, 0x25D3, 0x3EAE, 0x32AE, 0x0D14, 0x29BD, 0x038B, 0x2627, 0x2A7C, 0x12B3, 
        0x06A8, 0x3AEB, 0x1C73, 0x33E3, 0x3008, 0x281E, 0x3ECA, 0x2780, 0x08EB, 0x2AA2, 0x33B8, 0x34A4, 0x2701, 0x1388, 0x033B, 0x1E76, 
        0x1160, 0x198F, 0x13DD, 0x051D, 0x366C, 0x36A4, 0x2885, 0x2C80, 0x06C9, 0x0821, 0x2BEC, 0x2D1D, 0x1742, 0x1EF1, 0x2D84, 0x2093, 
        0x1917, 0x2366, 0x1996, 0x256E, 0x2416, 0x03BF, 0x3C96, 0x2866, 0x18D1, 0x1950, 0x2499, 0x3BA2, 0x3D99, 0x3CE9, 0x0F8B, 0x0556, 
        0x177A, 0x1A44, 0x2774, 0x30A3, 0x1BEF, 0x2FF2, 0x0E48, 0x0DA6, 0x34FB, 0x3714, 0x1E81, 0x2850, 0x0D9F, 0x1DE6, 0x0F0D, 0x284C, 
        0x0C03, 0x2C28, 0x19A3, 0x0E07, 0x30F8, 0x2761, 0x1A57, 0x2760, 0x37EE, 0x37D3, 0x3087, 0x2A40, 0x2004, 0x13BD, 0x30BC, 0x12FF, 
        0x1CC3, 0x3F42, 0x3545, 0x36E7, 0x1965, 0x29FB, 0x143E, 0x0FF0, 0x06E5, 0x3128, 0x2985, 0x3471, 0x1031, 0x2D0D, 0x0917, 0x39D9, 
        0x14D7, 0x2E7B, 0x3BBE, 0x3BBB, 0x1AEE, 0x10EB, 0x2E1C, 0x1C74, 0x25FB, 0x08CE, 0x3CD2, 0x31F8, 0x0DEB, 0x33B1, 0x0D3D, 0x0779, 
        0x29C2, 0x38FF, 0x01AD, 0x09F7, 0x2A24, 0x325D, 0x10E7, 0x203D, 0x0B2A, 0x1F6F, 0x33B9, 0x2736, 0x1300, 0x334C, 0x135F, 0x1E3C, 
        0x15E2, 0x2E7E, 0x3C5D, 0x2991, 0x38F7, 0x154D, 0x37A8, 0x16A6, 0x2A23, 0x386B, 0x39D4, 0x02F8, 0x2E95, 0x0A1D, 0x13F6, 0x1AAA, 
        0x1B27, 0x02AB, 0x3E7E, 0x3B6B, 0x36B2, 0x02A0, 0x3352, 0x24F8, 0x0841, 0x381A, 0x216E, 0x16D3, 0x1112, 0x39FF, 0x1AC0, 0x1610, 
        0x05DE, 0x3024, 0x15DD, 0x05C4, 0x1601, 0x2360, 0x238F, 0x069A, 0x084C, 0x27F9, 0x191E, 0x19C1, 0x1049, 0x0BD7, 0x2FE8, 0x364D, 
        0x052B, 0x180B, 0x1135, 0x2451, 0x363A, 0x1A38, 0x3F25, 0x2F6A, 0x195C, 0x202A, 0x27DC, 0x2857, 0x2DDA, 0x1435, 0x19FC, 0x2ADB, 
        0x2F5E, 0x0B52, 0x164F, 0x120E, 0x3B9E, 0x2FC5, 0x2108, 0x291F, 0x2A7B, 0x0610, 0x24F6, 0x17E3, 0x2D3C, 0x0FA0, 0x371E, 0x3F9C, 
        0x32EF, 0x3E9A, 0x2678, 0x1362, 0x3D19, 0x370E, 0x296E, 0x2A8D, 0x0445, 0x2104, 0x336A, 0x2619, 0x131F, 0x0A9D, 0x2610, 0x03D2, 
        0x2E3E, 0x05CD, 0x19E0, 0x2A69, 0x24A7, 0x1D37, 0x1E3D, 0x01F7, 0x22E3, 0x269B, 0x2CAC, 0x331B, 0x29B3, 0x1481, 0x0769, 0x3A1C, 
        0x1AF8, 0x337E, 0x1871, 0x2BF2, 0x1596, 0x0527, 0x1730, 0x2A4A, 0x30AF, 0x2402, 0x13E8, 0x2101, 0x1C79, 0x3003, 0x2114, 0x1902, 
        0x10D5, 0x3D48, 0x0B3F, 0x208B, 0x0A25, 0x1943, 0x2452, 0x113B, 0x207D, 0x0E31, 0x2746, 0x1FBC, 0x2FEF, 0x0D4E, 0x0E93, 0x3BD6, 
        0x3A86, 0x35AB, 0x0C27, 0x0FBA, 0x0F73, 0x27DA, 0x2B32, 0x2E9A, 0x2953, 0x098F, 0x156F, 0x3F4B, 0x0E99, 0x0C80, 0x0682, 0x01C8, 
        0x29AB, 0x2F11, 0x3C13, 0x0C2F, 0x29B8, 0x1E61, 0x1064, 0x26BF, 0x2B86, 0x04DB, 0x0147, 0x036B, 0x3FE9, 0x1D72, 0x17A8, 0x092C, 
        0x287F, 0x14DF, 0x24E2, 0x3260, 0x13C5, 0x09F0, 0x012A, 0x2A61, 0x2F05, 0x072F, 0x3E57, 0x13EB, 0x0A12, 0x0842, 0x1912, 0x0B7C, 
        0x1D8B, 0x1AF0, 0x2D1E, 0x17CE, 0x13DA, 0x011C, 0x010C, 0x0720, 0x26E8, 0x174D, 0x0BE8, 0x35B0, 0x1ECE, 0x10BE, 0x071F, 0x1B36, 
        0x1D02, 0x27F3, 0x08C7, 0x0991, 0x0294, 0x0D6D, 0x2057, 0x3E77, 0x157F, 0x106D, 0x3AB4, 0x3B24, 0x188A, 0x0827, 0x0889, 0x028D, 
        0x0D17, 0x298D, 0x3A64, 0x17E5, 0x3AB3, 0x2026, 0x202E, 0x2F1D, 0x04C1, 0x19F9, 0x25BD, 0x3419, 0x2C10, 0x3A47, 0x362E, 0x3078, 
        0x3416, 0x30DC, 0x2DF7, 0x306A, 0x33D0, 0x335E, 0x0B3B, 0x0AFC, 0x2692, 0x2D18, 0x099A, 0x09C3, 0x0BAF, 0x3502, 0x05E3, 0x056F, 
        0x1A0E, 0x2239, 0x3116, 0x2BB1, 0x0411, 0x1B94, 0x3945, 0x1974, 0x183D, 0x25AC, 0x168C, 0x230E, 0x3675, 0x0D19, 0x33C4, 0x026A, 
        0x12EE, 0x2F3A, 0x0835, 0x06DC, 0x1790, 0x1D9A, 0x0E62, 0x1316, 0x28A8, 0x03CE, 0x3D84, 0x278F, 0x0D91, 0x30DE, 0x2318, 0x163F, 
        0x01B6, 0x2C5A, 0x3AF3, 0x3BA7, 0x05D6, 0x2DBD, 0x08C4, 0x25E2, 0x1EC8, 0x2386, 0x2C59, 0x399C, 0x1CBA, 0x1084, 0x1919, 0x2436, 
        0x0600, 0x1091, 0x0282, 0x0140, 0x367F, 0x0BFE, 0x2EC2, 0x3503, 0x09CB, 0x06B1, 0x0CFF, 0x167C, 0x3B61, 0x0F2E, 0x1961, 0x09C2, 
        0x3245, 0x31C5, 0x2142, 0x3F7A, 0x3C39, 0x27C2, 0x0E53, 0x249B, 0x386E, 0x3EEA, 0x18BB, 0x208C, 0x1680, 0x0534, 0x1E37, 0x383C, 
        0x1731, 0x3200, 0x2A3D, 0x3EB1, 0x1E58, 0x23A0, 0x37D4, 0x2595, 0x36E8, 0x0A78, 0x336B, 0x3BB4, 0x05EF, 0x3519, 0x1836, 0x364C, 
        0x127A, 0x3B4E, 0x2A5A, 0x2214, 0x326D, 0x1ECC, 0x1716, 0x0447, 0x277C, 0x222A, 0x2097, 0x0829, 0x34CB, 0x3EDD, 0x1FB0, 0x3BE1, 
        0x1FD7, 0x2D45, 0x2399, 0x2263, 0x10AE, 0x35A2, 0x3912, 0x036C, 0x3B7F, 0x2771, 0x1A89, 0x02C3, 0x1A88, 0x0113, 0x3969, 0x07D1, 
        0x3B77, 0x0794, 0x044A, 0x3BA1, 0x1329, 0x290B, 0x2013, 0x1566, 0x02F3, 0x1553, 0x1778, 0x0654, 0x23A3, 0x2759, 0x1D59, 0x3604, 
        0x0C52, 0x2EFD, 0x16A1, 0x29D8, 0x113C, 0x2066, 0x0434, 0x0236, 0x218C, 0x0746, 0x2CC8, 0x084D, 0x238E, 0x2B3C, 0x220A, 0x1FDF, 
        0x01D6, 0x1FBA, 0x2C8C, 0x2117, 0x1B88, 0x0514, 0x3C4F, 0x2E74, 0x1D1B, 0x3819, 0x11AA, 0x0D83, 0x13FC, 0x3D58, 0x17AB, 0x12CB, 
        0x1903, 0x0FBE, 0x2445, 0x1FBE, 0x3394, 0x25E0, 0x2267, 0x0F0A, 0x32D0, 0x1067, 0x2D88, 0x0269, 0x37AF, 0x326E, 0x34E8, 0x2AC5, 
        0x2ADE, 0x2F65, 0x1B8F, 0x2711, 0x1FC7, 0x3D34, 0x2D7B, 0x3578, 0x2E03, 0x07D9, 0x17B8, 0x1307, 0x31FA, 0x0B27, 0x3485, 0x1E63, 
        0x3065, 0x394F, 0x1C11, 0x19F6, 0x1C44, 0x09FF, 0x235E, 0x1375, 0x39B9, 0x06D9, 0x3CCD, 0x13DE, 0x23D5, 0x17CD, 0x0CC5, 0x26B3, 
        0x0939, 0x18C5, 0x3470, 0x1D8D, 0x094C, 0x0B7F, 0x1C5C, 0x29D9, 0x0E1F, 0x232F, 0x018F, 0x1702, 0x1E20, 0x1D81, 0x1AF6, 0x22C8, 
        0x0E67, 0x2D8E, 0x0A97, 0x1C07, 0x255E, 0x3EDB, 0x3955, 0x3897, 0x2A5D, 0x0A19, 0x0171, 0x2252, 0x0EBB, 0x0F0B, 0x10BA, 0x0B21, 
        0x0C79, 0x1F00, 0x283D, 0x2B18, 0x2DA7, 0x0339, 0x37CA, 0x0D94, 0x1993, 0x2BE5, 0x3361, 0x395E, 0x3E79, 0x305E, 0x379D, 0x1497, 
        0x2609, 0x3F8C, 0x1E99, 0x154E, 0x2F39, 0x16AC, 0x1A94, 0x03C6, 0x2A8F, 0x03B5, 0x2A11, 0x34AD, 0x27AF, 0x1253, 0x223C, 0x3D41, 
        0x3493, 0x3A36, 0x023B, 0x230B, 0x0F71, 0x30C4, 0x2C8D, 0x26E9, 0x1A28, 0x1FAF, 0x1F75, 0x2DC7, 0x0778, 0x1216, 0x36D9, 0x0239, 
        0x3B17, 0x3A68, 0x3A67, 0x3B8E, 0x21AB, 0x1B28, 0x3CAB, 0x27EA, 0x27D1, 0x2ED0, 0x0475, 0x369B, 0x1EA1, 0x30E0, 0x2BCE, 0x030C, 
        0x0738, 0x069F, 0x12B1, 0x02C5, 0x06B8, 0x0ABD, 0x17F0, 0x23C2, 0x1479, 0x3464, 0x0825, 0x3CB2, 0x3EFB, 0x3855, 0x3AE4, 0x048D, 
        0x047E, 0x1530, 0x3611, 0x1C33, 0x1A8A, 0x21A6, 0x38ED, 0x18E7, 0x3D07, 0x2042, 0x046D, 0x366B, 0x0550, 0x1E18, 0x0BB9, 0x3B97, 
        0x1F02, 0x2E13, 0x3AEA, 0x21CC, 0x14EC, 0x304A, 0x3170, 0x100B, 0x29BF, 0x359C, 0x2ED8, 0x0541, 0x216A, 0x33B3, 0x3703, 0x1FF1, 
        0x395D, 0x320D, 0x3995, 0x1B45, 0x2326, 0x312C, 0x1BCA, 0x1F30, 0x339F, 0x02A7, 0x280D, 0x2765, 0x0A40, 0x2BBA, 0x3899, 0x1A49, 
        0x1A21, 0x0C9E, 0x05DD, 0x02B9, 0x0731, 0x209A, 0x024D, 0x2160, 0x1404, 0x1C43, 0x2414, 0x2AE6, 0x2129, 0x2B1A, 0x3296, 0x3877, 
        0x0BD3, 0x18D9, 0x3685, 0x140B, 0x3777, 0x25C9, 0x01F2, 0x1D2D, 0x0657, 0x34F2, 0x3D1A, 0x229C, 0x3B55, 0x24E9, 0x066F, 0x1D10, 
        0x2805, 0x2905, 0x3FCF, 0x21FB, 0x05B3, 0x25EF, 0x3228, 0x1FCA, 0x3660, 0x13B5, 0x1700, 0x32B0, 0x22EA, 0x1723, 0x02DF, 0x3369, 
        0x17C4, 0x10AD, 0x335A, 0x0E13, 0x15F9, 0x0C08, 0x1FCF, 0x353C, 0x0DCE, 0x1FD4, 0x2354, 0x1E78, 0x3445, 0x05E9, 0x3643, 0x1523, 
        0x1F8F, 0x07D6, 0x02B4, 0x08C6, 0x0318, 0x2C8E, 0x39DA, 0x10E4, 0x36AC, 0x10DF, 0x1E79, 0x1AF3, 0x35F9, 0x10F3, 0x11B4, 0x308F, 
        0x2B39, 0x16E1, 0x2B26, 0x0A85, 0x3070, 0x071B, 0x2138, 0x1B0E, 0x3673, 0x3B72, 0x3883, 0x0E60, 0x29D3, 0x21B3, 0x2F02, 0x3CDC, 
        0x23B0, 0x11BB, 0x15E3, 0x1BC5, 0x2F7B, 0x0174, 0x3E4E, 0x354D, 0x2F26, 0x0FA2, 0x11CD, 0x0B3D, 0x0EAA, 0x1D23, 0x2579, 0x1B4B, 
        0x050E, 0x39D7, 0x0C58, 0x0FD0, 0x081A, 0x1AD4, 0x38DF, 0x298F, 0x178C, 0x0B8B, 0x168A, 0x3B9F, 0x28E4, 0x2FD4, 0x318A, 0x2BCB, 
        0x2971, 0x3CCC, 0x3F92, 0x2C1E, 0x0881, 0x36D0, 0x39FB, 0x1DB7, 0x325E, 0x0D46, 0x3E22, 0x353F, 0x1849, 0x18D0, 0x14F8, 0x30C7, 
        0x0586, 0x072C, 0x14BC, 0x0152, 0x07CD, 0x28D8, 0x173F, 0x080B, 0x18E8, 0x1CA8, 0x1D04, 0x32D8, 0x1D27, 0x2715, 0x2249, 0x1D4F, 
        0x2092, 0x2E01, 0x0F80, 0x0E2A, 0x2B4C, 0x3A34, 0x33AD, 0x353D, 0x2819, 0x032D, 0x012F, 0x0ECD, 0x1FEA, 0x1C1D, 0x04A5, 0x1C0B, 
        0x3F62, 0x3F0B, 0x1C52, 0x3850, 0x3FBD, 0x04A7, 0x2BB9, 0x205D, 0x391E, 0x06F8, 0x09A3, 0x3CFA, 0x2521, 0x2A33, 0x2911, 0x3E2D, 
        0x086C, 0x21C5, 0x1D46, 0x396F, 0x0D7E, 0x1CD2, 0x2248, 0x1B98, 0x3512, 0x3CEE, 0x1A34, 0x02AD, 0x264A, 0x0748, 0x2CAA, 0x24FC, 
        0x2444, 0x1947, 0x11AE, 0x139A, 0x1237, 0x0FB1, 0x0262, 0x189A, 0x1E67, 0x0B63, 0x0B67, 0x3094, 0x3D27, 0x1D74, 0x39AB, 0x0DF3, 
        0x2F5C, 0x3E4A, 0x343E, 0x33EA, 0x1EA8, 0x27A6, 0x30B5, 0x1DF2, 0x2D8D, 0x1020, 0x1F77, 0x2F1A, 0x0321, 0x36F9, 0x2D9D, 0x053F, 
        0x1BAA, 0x0CA4, 0x0B4F, 0x3B9D, 0x390D, 0x0CA8, 0x07DF, 0x0165, 0x14AB, 0x31F7, 0x387F, 0x3DA1, 0x1A3C, 0x04A9, 0x06D6, 0x375D, 
        0x0A4C, 0x1734, 0x22BB, 0x01A8, 0x0665, 0x266C, 0x380C, 0x3A93, 0x2822, 0x36FD, 0x29DF, 0x2C06, 0x223B, 0x09BA, 0x1146, 0x0540, 
        0x0791, 0x28AE, 0x052F, 0x12FA, 0x2623, 0x07F7, 0x3C9E, 0x1695, 0x3C30, 0x14FF, 0x1586, 0x1263, 0x118B, 0x2028, 0x0F85, 0x12D4, 
        0x2D3A, 0x1D3A, 0x1DD5, 0x115C, 0x1475, 0x315D, 0x1CFA, 0x3F4D, 0x1ABF, 0x2996, 0x216C, 0x2A5F, 0x2FA3, 0x3AED, 0x324A, 0x3355, 
        0x25E9, 0x138E, 0x2E23, 0x128C, 0x26F0, 0x21F7, 0x1673, 0x2BCA, 0x3F81, 0x29A9, 0x1225, 0x1CC8, 0x152A, 0x05D5, 0x2F29, 0x2B67, 
        0x1685, 0x24F2, 0x1A98, 0x16D1, 0x0EA3, 0x3F34, 0x0D51, 0x0F62, 0x19B6, 0x29AF, 0x17AD, 0x0BF2, 0x1448, 0x019C, 0x3197, 0x291E, 
        0x0408, 0x0BF7, 0x26CA, 0x37BC, 0x1B81, 0x1F5C, 0x0EB6, 0x299F, 0x0BD2, 0x11B6, 0x2468, 0x329C, 0x063C, 0x17EB, 0x36F2, 0x0115, 
        0x2AF6, 0x09E4, 0x02E7, 0x04D4, 0x0B88, 0x21C9, 0x0F1F, 0x37B4, 0x07B1, 0x1B4A, 0x1254, 0x050C, 0x2FEB, 0x239D, 0x13F9, 0x1994, 
        0x3600, 0x3A12, 0x1995, 0x24F1, 0x164C, 0x25C5, 0x2A12, 0x23DD, 0x05CC, 0x0EEC, 0x3D64, 0x1862, 0x2D19, 0x3D94, 0x2561, 0x37A6, 
        0x387C, 0x0C43, 0x32C1, 0x3187, 0x217C, 0x1153, 0x1637, 0x3F44, 0x1DA1, 0x01C9, 0x3FB0, 0x1423, 0x3135, 0x2FAD, 0x2E48, 0x3A3A, 
        0x1178, 0x2F72, 0x2D55, 0x2FCB, 0x1055, 0x330C, 0x1026, 0x1C3B, 0x2DAC, 0x3E6E, 0x2F94, 0x1708, 0x0202, 0x3501, 0x2D40, 0x2BCF, 
        0x216B, 0x26CF, 0x2EA0, 0x3A6E, 0x240F, 0x1359, 0x1044, 0x269A, 0x0508, 0x110B, 0x26E4, 0x2FF1, 0x38FE, 0x3ACF, 0x0861, 0x3906, 
        0x2E51, 0x36E6, 0x2705, 0x3FF3, 0x0593, 0x1A75, 0x20D2, 0x11CC, 0x0806, 0x2B50, 0x0903, 0x3ABB, 0x3AB0, 0x3453, 0x17CA, 0x39B3, 
        0x0FEB, 0x0413, 0x2A19, 0x0468, 0x1800, 0x34C3, 0x3B76, 0x0925, 0x387E, 0x29EA, 0x3091, 0x297F, 0x0ED6, 0x15E0, 0x2FF6, 0x3631, 
        0x156A, 0x170D, 0x122D, 0x21CB, 0x257E, 0x278A, 0x033D, 0x1909, 0x0BF8, 0x1630, 0x0C1A, 0x3F33, 0x068D, 0x1398, 0x20C8, 0x1056, 
        0x0C11, 0x1364, 0x1F19, 0x093E, 0x01E9, 0x2BD8, 0x3786, 0x3F2E, 0x11F4, 0x0569, 0x18B4, 0x1BEA, 0x301A, 0x2EAC, 0x293A, 0x11A6, 
        0x11D1, 0x0E18, 0x1518, 0x0A28, 0x0149, 0x21A4, 0x24CE, 0x2E79, 0x33B7, 0x16EB, 0x206B, 0x03CA, 0x2BC5, 0x1DFC, 0x04B2, 0x34EC, 
        0x2FD1, 0x0ED0, 0x1D2C, 0x34C0, 0x3C14, 0x306B, 0x01CF, 0x27CB, 0x1EDE, 0x1358, 0x11E6, 0x1BB7, 0x1808, 0x10A7, 0x17EE, 0x0314, 
        0x2841, 0x1CBC, 0x2FD8, 0x3576, 0x3DBA, 0x0F6D, 0x3715, 0x0B5E, 0x297C, 0x12BA, 0x178A, 0x0852, 0x28B1, 0x3C12, 0x1FA6, 0x2306, 
        0x255B, 0x3915, 0x0D7D, 0x1CE5, 0x309B, 0x1C8F, 0x250C, 0x3609, 0x3C0E, 0x3FC7, 0x34CE, 0x358B, 0x142F, 0x3A94, 0x0B48, 0x1357, 
        0x0396, 0x056C, 0x043A, 0x19E7, 0x14B3, 0x0CBB, 0x3E2C, 0x17C3, 0x3707, 0x354C, 0x2502, 0x0C1C, 0x083D, 0x2F13, 0x34AE, 0x0E1D, 
        0x29EF, 0x2ACB, 0x1ACF, 0x0A56, 0x0C70, 0x0611, 0x3F29, 0x3B2B, 0x11FF, 0x1B3B, 0x1668, 0x070D, 0x3F84, 0x1CE0, 0x2909, 0x1935, 
        0x2B78, 0x31CD, 0x0AB6, 0x1421, 0x03C8, 0x24B9, 0x273B, 0x1BE8, 0x0DAF, 0x1B64, 0x32A1, 0x0362, 0x1BB0, 0x0D53, 0x16F7, 0x11A9, 
        0x1C2F, 0x2F23, 0x1C7F, 0x183C, 0x2E6B, 0x2C3F, 0x39AD, 0x1E34, 0x33E6, 0x01B8, 0x3EC4, 0x031D, 0x3E53, 0x32ED, 0x1E6A, 0x39AC, 
        0x3D43, 0x39F4, 0x2B7D, 0x30FD, 0x0EE8, 0x1393, 0x1E27, 0x31F9, 0x3182, 0x3E16, 0x0EB7, 0x1604, 0x1278, 0x028C, 0x1973, 0x380E, 
        0x2D3D, 0x3462, 0x3BCC, 0x2F89, 0x13AA, 0x2F9A, 0x1EBD, 0x13C4, 0x376C, 0x1E2C, 0x392F, 0x36D4, 0x38E0, 0x0ADB, 0x3AB5, 0x183F, 
        0x293E, 0x06A3, 0x2495, 0x388A, 0x2FB7, 0x0770, 0x29D6, 0x3C17, 0x0AD9, 0x185B, 0x0252, 0x2D11, 0x149E, 0x1EDB, 0x3755, 0x2CC3, 
        0x0BEF, 0x29DB, 0x0BB2, 0x347E, 0x1215, 0x3A65, 0x199F, 0x2D38, 0x05C8, 0x30F5, 0x08EA, 0x100D, 0x2726, 0x2C3B, 0x3B1E, 0x2717, 
        0x3FFB, 0x0A08, 0x284A, 0x3780, 0x08D2, 0x3543, 0x0687, 0x13FE, 0x291C, 0x38D5, 0x1EDC, 0x3FAD, 0x0187, 0x0957, 0x3CF3, 0x045D, 
        0x257A, 0x2D09, 0x3B6F, 0x264E, 0x0B6C, 0x1FB6, 0x16E5, 0x2161, 0x0F39, 0x3823, 0x1F71, 0x0847, 0x094A, 0x3132, 0x1E6E, 0x1562, 
        0x2545, 0x1DD4, 0x17CB, 0x24AC, 0x10A2, 0x06AE, 0x3AF8, 0x1521, 0x1796, 0x2829, 0x390A, 0x0A79, 0x14FD, 0x209D, 0x0AF8, 0x2C46, 
        0x0F55, 0x0AE6, 0x01C4, 0x3271, 0x027C, 0x234E, 0x3510, 0x0E82, 0x1A97, 0x2F25, 0x370F, 0x1465, 0x105C, 0x3C00, 0x1085, 0x0D37, 
        0x340B, 0x3A99, 0x070A, 0x3ACA, 0x3BED, 0x06AD, 0x01E8, 0x262F, 0x2B56, 0x0403, 0x368D, 0x26F9, 0x17F1, 0x0C3C, 0x1B6A, 0x2CAF, 
        0x3AAB, 0x2AE5, 0x3036, 0x1810, 0x08FD, 0x1409, 0x13DF, 0x3354, 0x3E8E, 0x015B, 0x161A, 0x1BFB, 0x02C7, 0x2EE7, 0x3370, 0x2C23, 
        0x3F27, 0x18D6, 0x2F92, 0x10B1, 0x359B, 0x1A0B, 0x1399, 0x1DCB, 0x13B6, 0x2D95, 0x1571, 0x0D3E, 0x04C7, 0x35EC, 0x04F2, 0x1F29, 
        0x1D29, 0x1809, 0x3BF9, 0x21F0, 0x2446, 0x3625, 0x1892, 0x3C40, 0x3E95, 0x279D, 0x0A66, 0x0383, 0x2C04, 0x2FF7, 0x2E0F, 0x2DCD, 
        0x13A3, 0x3006, 0x0DEE, 0x1F97, 0x359A, 0x3F0E, 0x03D4, 0x0D20, 0x1577, 0x0203, 0x38AC, 0x152B, 0x03E9, 0x01DE, 0x1690, 0x3CC0, 
        0x1418, 0x3900, 0x099D, 0x35F5, 0x11E8, 0x37A5, 0x328B, 0x0181, 0x1B0F, 0x024C, 0x2351, 0x268C, 0x2AFD, 0x27FC, 0x2428, 0x3917, 
        0x0FF4, 0x09B0, 0x07C3, 0x2AB3, 0x3E02, 0x0BB3, 0x065C, 0x1A2F, 0x3617, 0x2BB6, 0x2F31, 0x0EB0, 0x3E87, 0x37FF, 0x0D9D, 0x1147, 
        0x3232, 0x2840, 0x288B, 0x3FB6, 0x272F, 0x2C81, 0x0877, 0x088E, 0x1FE0, 0x05AE, 0x1110, 0x365F, 0x1039, 0x361B, 0x28CE, 0x3E72, 
        0x1018, 0x10DC, 0x18F6, 0x094D, 0x3A3D, 0x3694, 0x3DB2, 0x36B6, 0x10D4, 0x0CA1, 0x0B06, 0x3220, 0x0507, 0x0467, 0x3E93, 0x370D, 
        0x0773, 0x18EA, 0x3784, 0x192F, 0x0D1B, 0x186D, 0x146B, 0x0372, 0x0BC0, 0x0D84, 0x0904, 0x2834, 0x0A8C, 0x0454, 0x0462, 0x1BCB, 
        0x2EE4, 0x30E8, 0x0246, 0x0C36, 0x0575, 0x3B13, 0x1C6C, 0x2D01, 0x2F7E, 0x3EE2, 0x3D73, 0x0F23, 0x0C2B, 0x165D, 0x2BAC, 0x080D, 
        0x2515, 0x1879, 0x24C3, 0x1368, 0x2A18, 0x029E, 0x3ADE, 0x2C7A, 0x2FC0, 0x06E4, 0x0574, 0x29FE, 0x20CA, 0x1C5A, 0x39BD, 0x1606, 
        0x0793, 0x1648, 0x3C4E, 0x1675, 0x1DBE, 0x1414, 0x213D, 0x2596, 0x1DD1, 0x3ACE, 0x31EF, 0x2CD7, 0x3CD3, 0x0195, 0x0C61, 0x1D45, 
        0x2983, 0x3679, 0x0CF6, 0x3717, 0x16D7, 0x3E48, 0x3D69, 0x07A5, 0x3514, 0x2FB6, 0x0159, 0x3AF0, 0x221E, 0x0AA6, 0x2C5C, 0x37E1, 
        0x0914, 0x26FA, 0x3140, 0x2053, 0x3873, 0x21FC, 0x0FBD, 0x1ACB, 0x2040, 0x1117, 0x238D, 0x3762, 0x0233, 0x049D, 0x3D95, 0x1DE8, 
        0x1759, 0x1E84, 0x1AE5, 0x3C77, 0x083F, 0x3BFA, 0x0616, 0x03A5, 0x1797, 0x140F, 0x0EF6, 0x1727, 0x3810, 0x38C6, 0x2E11, 0x2C45, 
        0x3368, 0x2AA0, 0x1285, 0x153A, 0x2222, 0x236A, 0x35CF, 0x10AC, 0x2BE2, 0x09F9, 0x39A2, 0x1E7A, 0x27AB, 0x0BA5, 0x012D, 0x224F, 
        0x3CC1, 0x2762, 0x08E3, 0x22BC, 0x2307, 0x0C4E, 0x3934, 0x1F63, 0x1CF1, 0x0907, 0x0C7A, 0x1E62, 0x06C8, 0x26DC, 0x2BF5, 0x3344, 
        0x1E51, 0x0299, 0x133A, 0x10CB, 0x2D9C, 0x24E4, 0x0784, 0x3261, 0x2F47, 0x3865, 0x2B35, 0x3764, 0x1213, 0x17EA, 0x1097, 0x330E, 
        0x0266, 0x0726, 0x2C26, 0x1C24, 0x38C5, 0x1D58, 0x2324, 0x2F7C, 0x1843, 0x2430, 0x1A7D, 0x08AF, 0x1857, 0x3C01, 0x3381, 0x1672, 
        0x3109, 0x3BC1, 0x3A18, 0x1290, 0x1512, 0x201D, 0x218D, 0x2B24, 0x23B3, 0x32AB, 0x3EA7, 0x1688, 0x212F, 0x3265, 0x1D53, 0x379B, 
        0x2845, 0x3082, 0x2654, 0x184B, 0x1887, 0x1812, 0x26B9, 0x0D68, 0x3252, 0x1CDE, 0x043D, 0x061B, 0x1413, 0x21A0, 0x2B03, 0x2769, 
        0x12CC, 0x071C, 0x0634, 0x2D23, 0x21B5, 0x0977, 0x3264, 0x0985, 0x0148, 0x0C0A, 0x3257, 0x0543, 0x378B, 0x379E, 0x3C49, 0x3B37, 
        0x1854, 0x0921, 0x3ECD, 0x2B97, 0x2852, 0x374A, 0x3429, 0x39BE, 0x3AF5, 0x18FE, 0x11F6, 0x33AA, 0x16EE, 0x3599, 0x2B0C, 0x0111, 
        0x0F25, 0x3B59, 0x16FF, 0x1E24, 0x2FFE, 0x2F6B, 0x3B39, 0x271E, 0x2C33, 0x2E26, 0x2259, 0x0D66, 0x3619, 0x2938, 0x3C5C, 0x2B84, 
        0x2861, 0x0124, 0x2BF0, 0x2779, 0x29E5, 0x121D, 0x34C9, 0x3A3C, 0x39D3, 0x365E, 0x07B3, 0x1818, 0x05D9, 0x2D68, 0x2904, 0x1CFE, 
        0x14B9, 0x38E2, 0x1AD5, 0x3484, 0x3F5A, 0x334A, 0x1182, 0x0F8F, 0x029A, 0x2C70, 0x2931, 0x0327, 0x15B9, 0x2227, 0x1A69, 0x0ADA, 
        0x0473, 0x02BD, 0x3EB2, 0x2B08, 0x0446, 0x35C5, 0x075D, 0x3DEB, 0x0DB7, 0x2AFC, 0x3D61, 0x12DF, 0x2E62, 0x179B, 0x0D90, 0x1EFB, 
        0x11F5, 0x252E, 0x0AD5, 0x1F24, 0x18A1, 0x3C31, 0x1377, 0x0F83, 0x0B3E, 0x3751, 0x3B2F, 0x12E4, 0x0B44, 0x07F3, 0x3310, 0x07C5, 
        0x3B8A, 0x088D, 0x2F1C, 0x3EF3, 0x1884, 0x3B7C, 0x3AD5, 0x0A07, 0x3B3B, 0x08AB, 0x10B4, 0x28A2, 0x0F91, 0x225B, 0x06FE, 0x2BF9, 
        0x1BB4, 0x31CC, 0x1C72, 0x33BA, 0x034A, 0x1746, 0x2C57, 0x04FA, 0x3DAF, 0x3EBA, 0x3403, 0x1BD1, 0x332E, 0x3739, 0x0BC8, 0x30AA, 
        0x06EB, 0x22CC, 0x3C88, 0x3BAC, 0x34A5, 0x1D2E, 0x0F5A, 0x1C6F, 0x20DE, 0x0210, 0x2439, 0x317F, 0x2591, 0x16A3, 0x17D1, 0x3358, 
        0x36E2, 0x0260, 0x155E, 0x2102, 0x17A6, 0x03AB, 0x0D08, 0x0BA1, 0x1419, 0x0976, 0x1F8A, 0x0D54, 0x1D62, 0x1430, 0x0DDF, 0x210B, 
        0x35D5, 0x2FF5, 0x2EC7, 0x3A50, 0x3309, 0x1612, 0x2339, 0x1139, 0x0A43, 0x23EA, 0x1736, 0x38DB, 0x37DE, 0x2F0A, 0x1036, 0x240C, 
        0x224D, 0x30B8, 0x3EF8, 0x0C8D, 0x3C89, 0x0709, 0x1852, 0x057E, 0x0138, 0x0D04, 0x2D76, 0x14DC, 0x1B2A, 0x28CD, 0x3101, 0x1E64, 
        0x3D45, 0x236B, 0x0512, 0x34B2, 0x3564, 0x3CF0, 0x17D8, 0x3C56, 0x340C, 0x38B4, 0x32D3, 0x2752, 0x3D57, 0x24CF, 0x3146, 0x0ED7, 
        0x2E59, 0x02BB, 0x30D9, 0x0D1D, 0x014B, 0x17E8, 0x32CE, 0x358F, 0x0D2C, 0x25F6, 0x02D1, 0x02A8, 0x273F, 0x0D9B, 0x2518, 0x3386, 
        0x16B9, 0x04F1, 0x11E0, 0x37C1, 0x2A2C, 0x2029, 0x3B2E, 0x0D86, 0x1F65, 0x30B1, 0x37D9, 0x3768, 0x3EC3, 0x3269, 0x3979, 0x1076, 
        0x0C0E, 0x383E, 0x1472, 0x04D7, 0x3A05, 0x26D0, 0x3BEB, 0x38E6, 0x0DC5, 0x2065, 0x18DD, 0x30B2, 0x21D7, 0x2A21, 0x2011, 0x33C2, 
        0x2928, 0x296B, 0x352C, 0x1682, 0x059A, 0x354E, 0x3BC0, 0x394A, 0x1C8A, 0x375E, 0x35E7, 0x017A, 0x23C6, 0x1EB1, 0x226B, 0x197C, 
        0x3079, 0x36A5, 0x3C8A, 0x2827, 0x0367, 0x341C, 0x1890, 0x321B, 0x0F77, 0x2A5B, 0x2984, 0x1503, 0x3E09, 0x07E7, 0x0A82, 0x106B, 
        0x0579, 0x08AE, 0x01A3, 0x2F20, 0x16AE, 0x374F, 0x2EB9, 0x370A, 0x1224, 0x218A, 0x0BF9, 0x1D85, 0x2231, 0x27AA, 0x1440, 0x2C55, 
        0x237E, 0x0E02, 0x3A2E, 0x2030, 0x0948, 0x3DD7, 0x136A, 0x12AF, 0x2775, 0x2250, 0x3E12, 0x0EEB, 0x1AC6, 0x0AED, 0x0741, 0x18F5, 
        0x129F, 0x2C4F, 0x32FF, 0x0A3F, 0x288C, 0x17E9, 0x2552, 0x02E5, 0x0D0A, 0x2BFD, 0x0DFC, 0x2777, 0x09DF, 0x1FE9, 0x2312, 0x0625, 
        0x1844, 0x18C6, 0x2643, 0x324B, 0x2946, 0x27A1, 0x1632, 0x3BFC, 0x3647, 0x12F6, 0x01EC, 0x2CE4, 0x34B3, 0x2405, 0x1D3E, 0x3013, 
        0x1219, 0x2234, 0x259B, 0x26EB, 0x34CD, 0x2337, 0x102B, 0x1A50, 0x0B70, 0x2EFF, 0x1655, 0x1791, 0x0850, 0x124F, 0x0FBF, 0x0EC5, 
        0x3948, 0x24FF, 0x03A6, 0x30EE, 0x12FE, 0x043E, 0x216D, 0x3CA8, 0x05C9, 0x0A0D, 0x0678, 0x28D3, 0x1F6D, 0x2F90, 0x184A, 0x1DED, 
        0x2D49, 0x1B2B, 0x2804, 0x27FA, 0x22F6, 0x0D2B, 0x3044, 0x0964, 0x02A4, 0x3FD8, 0x1B53, 0x16B2, 0x284F, 0x1066, 0x25F4, 0x0C87, 
        0x085A, 0x3DAC, 0x1CDF, 0x089A, 0x303A, 0x3F64, 0x0B93, 0x2BD3, 0x0605, 0x37E5, 0x3430, 0x0B39, 0x0869, 0x2084, 0x26E6, 0x33F7, 
        0x2393, 0x0373, 0x3B25, 0x3FE2, 0x03BB, 0x2147, 0x325A, 0x19F1, 0x1652, 0x0689, 0x22BD, 0x1792, 0x1C36, 0x0EDF, 0x350C, 0x0EBD, 
        0x07EB, 0x3E70, 0x195B, 0x2010, 0x1486, 0x3C46, 0x1B21, 0x2559, 0x2A8C, 0x1D71, 0x2597, 0x2C90, 0x0B43, 0x0AF7, 0x1CF3, 0x28F5, 
        0x1F5E, 0x0BF3, 0x2CA0, 0x2656, 0x02B0, 0x33FF, 0x2B89, 0x0C66, 0x1CED, 0x1A47, 0x1BF3, 0x3E3B, 0x0FD3, 0x0377, 0x0101, 0x3598, 
        0x3E31, 0x0FE8, 0x0DA8, 0x28A7, 0x153F, 0x2F33, 0x3150, 0x189E, 0x2111, 0x144C, 0x1CAF, 0x1A4B, 0x1634, 0x28D5, 0x1A7B, 0x2B5D, 
        0x3E8F, 0x3F53, 0x3653, 0x32CF, 0x2A50, 0x1494, 0x046F, 0x074A, 0x2679, 0x1A1F, 0x12A8, 0x1340, 0x1442, 0x2A63, 0x307F, 0x2284, 
        0x2D58, 0x31FE, 0x186F, 0x277B, 0x0E7E, 0x1467, 0x14EA, 0x3892, 0x0859, 0x3CDA, 0x388B, 0x212B, 0x3174, 0x07B8, 0x1845, 0x0B41, 
        0x3708, 0x26B6, 0x0F8E, 0x07A0, 0x2902, 0x16F4, 0x3D5E, 0x2A94, 0x3849, 0x2F0C, 0x3F82, 0x2CE6, 0x35F0, 0x28FA, 0x2D9F, 0x0B26, 
        0x3401, 0x2242, 0x328F, 0x0838, 0x27A2, 0x094B, 0x24DB, 0x16B6, 0x119E, 0x34ED, 0x3FCD, 0x368E, 0x3992, 0x1BA7, 0x1DA0, 0x3103, 
        0x14D9, 0x1E0D, 0x37F2, 0x10F5, 0x08D6, 0x2D3F, 0x1692, 0x01A4, 0x17DE, 0x02B5, 0x16AD, 0x1F40, 0x12AC, 0x3584, 0x16BD, 0x0BE9, 
        0x0A34, 0x1355, 0x134C, 0x3A31, 0x083C, 0x0CBE, 0x372C, 0x17E4, 0x33AF, 0x049C, 0x3F1E, 0x3732, 0x3C23, 0x371D, 0x03A7, 0x11DD, 
        0x3EBC, 0x1848, 0x1DCF, 0x169F, 0x3B94, 0x1335, 0x3E13, 0x0D32, 0x2EB3, 0x2043, 0x258E, 0x3702, 0x054C, 0x3E19, 0x35B3, 0x3A28, 
        0x2531, 0x3B15, 0x078A, 0x2121, 0x2640, 0x2DEA, 0x1629, 0x292C, 0x36B4, 0x28C9, 0x18ED, 0x2C02, 0x3F09, 0x2870, 0x275F, 0x1711, 
        0x214B, 0x1D2F, 0x35DC, 0x3A00, 0x2385, 0x0C76, 0x342F, 0x023C, 0x075B, 0x182F, 0x2486, 0x20F6, 0x300F, 0x3307, 0x3A76, 0x1DBC, 
        0x0D6A, 0x2D66, 0x09AE, 0x11D4, 0x3032, 0x1057, 0x173B, 0x22F7, 0x0206, 0x21D6, 0x12ED, 0x0A9F, 0x278E, 0x077B, 0x2D99, 0x0742, 
        0x1291, 0x112E, 0x0D99, 0x3010, 0x110E, 0x0275, 0x3A37, 0x1CF5, 0x1A0D, 0x059C, 0x2650, 0x2DC3, 0x3E6D, 0x2881, 0x35E0, 0x18F3, 
        0x33E9, 0x0BB6, 0x2AC8, 0x37C9, 0x03A3, 0x12F4, 0x275D, 0x2BF4, 0x3213, 0x0C5A, 0x2972, 0x1944, 0x1A30, 0x2E06, 0x0173, 0x1A26, 
        0x33FB, 0x075A, 0x109F, 0x190F, 0x0256, 0x3FB3, 0x1754, 0x2A66, 0x2FC7, 0x365D, 0x2F1F, 0x0DE8, 0x15CB, 0x167A, 0x06B4, 0x15A3, 
        0x26F5, 0x04C2, 0x1AF4, 0x0F11, 0x1A23, 0x0EA9, 0x2115, 0x29D5, 0x3B6C, 0x0F75, 0x0D7F, 0x2062, 0x0938, 0x1462, 0x0238, 0x12D7, 
        0x3CDD, 0x0B1D, 0x200F, 0x3D3B, 0x3BC3, 0x0FD8, 0x311B, 0x2465, 0x0D63, 0x1894, 0x1218, 0x1DAF, 0x1B5F, 0x159A, 0x1860, 0x09ED, 
        0x0F63, 0x0839, 0x1ED8, 0x1BD8, 0x2825, 0x3D79, 0x1ACE, 0x1468, 0x1526, 0x0264, 0x268B, 0x099F, 0x1336, 0x3B52, 0x05BF, 0x137B, 
        0x0971, 0x053E, 0x20D0, 0x19BC, 0x3825, 0x0937, 0x25F2, 0x0F1A, 0x1493, 0x358C, 0x1D94, 0x397E, 0x191B, 0x0C6B, 0x2EA8, 0x0170, 
        0x20CD, 0x3EB5, 0x15BB, 0x1CE8, 0x318B, 0x27C1, 0x2728, 0x1643, 0x37B6, 0x0BCF, 0x3467, 0x2A27, 0x0145, 0x0E36, 0x21B4, 0x3319, 
        0x24F0, 0x17F8, 0x26E3, 0x1384, 0x0B1E, 0x03ED, 0x25D8, 0x0F89, 0x14C9, 0x28EC, 0x1DD6, 0x0930, 0x3DCB, 0x0524, 0x2723, 0x0CFD, 
        0x32E7, 0x34E6, 0x34DE, 0x0192, 0x052D, 0x2376, 0x1F07, 0x204A, 0x0D64, 0x317D, 0x3F6A, 0x3925, 0x2751, 0x0945, 0x329D, 0x09D2, 
        0x01C0, 0x1E38, 0x02E2, 0x2563, 0x27A8, 0x29C7, 0x16F5, 0x29C8, 0x37B3, 0x0CCC, 0x2E04, 0x0343, 0x2155, 0x0C8E, 0x0C82, 0x332C, 
        0x18BE, 0x1539, 0x123B, 0x2373, 0x383D, 0x15CD, 0x1B23, 0x1725, 0x087D, 0x22E7, 0x0E0D, 0x21CD, 0x3DDC, 0x1CF8, 0x0D02, 0x04EC, 
        0x3F04, 0x0506, 0x3974, 0x2CC2, 0x0669, 0x21DC, 0x0E49, 0x2F19, 0x3C21, 0x32EC, 0x2140, 0x3C9B, 0x3E6F, 0x2167, 0x1C2D, 0x1CB9, 
        0x284D, 0x0FF6, 0x0199, 0x0BAE, 0x2350, 0x170B, 0x0918, 0x329E, 0x0E09, 0x0940, 0x2504, 0x1478, 0x2FD6, 0x2449, 0x2329, 0x2D10, 
        0x1223, 0x03A9, 0x2365, 0x361E, 0x22C6, 0x1EE2, 0x1043, 0x1306, 0x087C, 0x3F59, 0x2FED, 0x14CC, 0x222D, 0x3CBC, 0x3000, 0x0270, 
        0x33F5, 0x0D9A, 0x1AAC, 0x205F, 0x1D8F, 0x01F9, 0x3BCE, 0x1227, 0x0B1F, 0x0FBC, 0x0288, 0x32E6, 0x0B72, 0x28C8, 0x1221, 0x0214, 
        0x3F87, 0x1A9E, 0x38FB, 0x3472, 0x127F, 0x024F, 0x0870, 0x0ADF, 0x1088, 0x2F3F, 0x1B8B, 0x2942, 0x1DEC, 0x1505, 0x180F, 0x2907, 
        0x1A93, 0x28BB, 0x1EBF, 0x15F3, 0x196B, 0x1B95, 0x290A, 0x3C4C, 0x2981, 0x2528, 0x067D, 0x091D, 0x34D7, 0x1E5D, 0x01C1, 0x0523, 
        0x2BEB, 0x0943, 0x33C5, 0x11F2, 0x247D, 0x12B2, 0x25A6, 0x23FA, 0x3F61, 0x1100, 0x3D56, 0x181E, 0x16A8, 0x19DC, 0x3588, 0x07C4, 
        0x0785, 0x2FF4, 0x01AA, 0x3B8C, 0x3700, 0x2600, 0x06D4, 0x12C1, 0x0D48, 0x03FD, 0x3C1D, 0x0487, 0x24BC, 0x265D, 0x0BC6, 0x15A6, 
        0x2315, 0x234B, 0x3EAB, 0x2A89, 0x3A7A, 0x12A5, 0x2720, 0x211E, 0x0DA1, 0x2128, 0x3350, 0x0432, 0x3F6E, 0x3C6B, 0x3B32, 0x0615, 
        0x3050, 0x17B9, 0x3E9D, 0x382E, 0x386A, 0x1BCC, 0x3DCA, 0x156C, 0x0732, 0x2E64, 0x25DC, 0x1847, 0x3AA0, 0x0390, 0x31AA, 0x012E, 
        0x022F, 0x2ECA, 0x16B0, 0x2837, 0x0F70, 0x3D75, 0x292A, 0x38A6, 0x3ED8, 0x34B7, 0x2A25, 0x0628, 0x2E69, 0x2B2D, 0x23B4, 0x3014, 
        0x3DE7, 0x0FF3, 0x3D93, 0x1310, 0x2A32, 0x13CC, 0x1129, 0x213C, 0x3461, 0x3AD6, 0x20F8, 0x1FE6, 0x1A6A, 0x2FE3, 0x20FA, 0x0648, 
        0x2B1B, 0x1BF6, 0x1D05, 0x0864, 0x22DF, 0x2778, 0x2C27, 0x1132, 0x2DB1, 0x08D4, 0x1E66, 0x0EF0, 0x361D, 0x108A, 0x202B, 0x382A, 
        0x33F4, 0x16AA, 0x1B96, 0x329F, 0x3645, 0x0662, 0x04E6, 0x30AD, 0x28CB, 0x36AE, 0x1022, 0x1896, 0x2FF9, 0x064A, 0x1F28, 0x3C6D, 
        0x37A4, 0x28E8, 0x2DC4, 0x16F3, 0x10C5, 0x05E2, 0x3B0B, 0x2B4E, 0x0D92, 0x32DD, 0x270C, 0x3FE8, 0x17CC, 0x0C97, 0x1F0D, 0x3F95, 
        0x1D65, 0x09B4, 0x137D, 0x21A9, 0x2C12, 0x09AC, 0x158D, 0x2217, 0x2BE1, 0x04F6, 0x3A1D, 0x25DD, 0x0923, 0x2ACD, 0x2C94, 0x276F, 
        0x1735, 0x215C, 0x1F9E, 0x2E40, 0x1779, 0x25A2, 0x3A1B, 0x3A5C, 0x3DAD, 0x2A5E, 0x248A, 0x1BE0, 0x26B1, 0x1DFD, 0x174C, 0x28DB, 
        0x2D65, 0x2E9C, 0x170A, 0x0B86, 0x0698, 0x218E, 0x10DB, 0x3E78, 0x3F36, 0x29B2, 0x3052, 0x1C87, 0x0EAF, 0x38E8, 0x36BB, 0x1BD2, 
        0x3F18, 0x150D, 0x1DF8, 0x32F6, 0x3030, 0x2DF4, 0x3960, 0x187D, 0x1083, 0x1607, 0x29D1, 0x3C3A, 0x0B3C, 0x2AB2, 0x1FA3, 0x1A06, 
        0x3704, 0x0766, 0x0783, 0x285C, 0x197D, 0x0F14, 0x1017, 0x3E0F, 0x3D81, 0x1D2B, 0x0437, 0x142A, 0x0D8D, 0x2F44, 0x1684, 0x2659, 
        0x089F, 0x289A, 0x023A, 0x3090, 0x3D85, 0x042F, 0x2677, 0x090C, 0x2DE8, 0x32BF, 0x0815, 0x0CA6, 0x0489, 0x39D8, 0x3691, 0x19D9, 
        0x15B4, 0x2C97, 0x139D, 0x0E56, 0x068E, 0x07C1, 0x1D0F, 0x3AF6, 0x3C04, 0x05F5, 0x33BC, 0x1E87, 0x2BCC, 0x33CD, 0x39A5, 0x28A0, 
        0x1A3A, 0x1F90, 0x19C3, 0x26BD, 0x34AF, 0x2987, 0x08DE, 0x0241, 0x3E98, 0x3375, 0x2046, 0x1FBF, 0x2921, 0x30D3, 0x3C93, 0x3AC1, 
        0x0814, 0x3569, 0x2025, 0x3F45, 0x2CEC, 0x3B29, 0x0483, 0x1DA8, 0x2584, 0x1D26, 0x3B90, 0x3C7D, 0x336F, 0x2E45, 0x0110, 0x0A67, 
        0x274F, 0x0A63, 0x1B73, 0x3FA7, 0x3C78, 0x11D6, 0x2A9A, 0x32C5, 0x1999, 0x0626, 0x3DAB, 0x3322, 0x06CF, 0x0A42, 0x3627, 0x2CF0, 
        0x1726, 0x2A2B, 0x0872, 0x15E7, 0x04B1, 0x382D, 0x29BC, 0x1D84, 0x2C4D, 0x1A09, 0x2D59, 0x2571, 0x1FBD, 0x3E5C, 0x1986, 0x2DA6, 
        0x2AEC, 0x1485, 0x3C9D, 0x2B4A, 0x161B, 0x1BBB, 0x1D5E, 0x22F4, 0x366E, 0x35E9, 0x3A88, 0x1B5D, 0x0A3E, 0x18C3, 0x2033, 0x09A7, 
        0x0B81, 0x377B, 0x0B54, 0x3D68, 0x15E9, 0x0C29, 0x127B, 0x22CA, 0x23A7, 0x377C, 0x194C, 0x040B, 0x3CA6, 0x2D5E, 0x1C63, 0x0DDC, 
        0x16A2, 0x328A, 0x0EE9, 0x35B9, 0x21EC, 0x1EC9, 0x05D3, 0x30A9, 0x1AE4, 0x36FC, 0x0274, 0x0EE5, 0x27FB, 0x2C2E, 0x068A, 0x053B, 
        0x1985, 0x09B1, 0x18A2, 0x0974, 0x1354, 0x1233, 0x04D6, 0x2C6E, 0x09CA, 0x26AD, 0x3565, 0x2B3B, 0x2868, 0x3FEA, 0x1D1F, 0x05C7, 
        0x3526, 0x28BD, 0x39DC, 0x282B, 0x3605, 0x1C41, 0x1AA0, 0x21A2, 0x3033, 0x01FE, 0x2B19, 0x3556, 0x320A, 0x3772, 0x1B37, 0x0EFD, 
        0x0A16, 0x3BC8, 0x0AFD, 0x3AB7, 0x07E0, 0x29CB, 0x235A, 0x1140, 0x2E78, 0x18B6, 0x1502, 0x233A, 0x2E2F, 0x0B40, 0x3952, 0x1087, 
        0x1279, 0x1B5E, 0x1A91, 0x117F, 0x07D4, 0x2789, 0x2C61, 0x2B0E, 0x265A, 0x239F, 0x27EF, 0x29B0, 0x0F43, 0x25F0, 0x089C, 0x3452, 
        0x3F1D, 0x22B7, 0x1124, 0x2755, 0x050D, 0x1C2E, 0x2BDF, 0x0819, 0x0463, 0x1F03, 0x05F9, 0x265C, 0x04DE, 0x2459, 0x2784, 0x03B0, 
        0x30A6, 0x03F7, 0x3051, 0x0EEE, 0x2EB1, 0x21AD, 0x074C, 0x37D6, 0x0BA2, 0x3D8B, 0x116D, 0x0C8C, 0x37CF, 0x3FF2, 0x1D03, 0x29CC, 
        0x2FA7, 0x1B15, 0x32B9, 0x384E, 0x1E6D, 0x3047, 0x212D, 0x231D, 0x2575, 0x355F, 0x2235, 0x06C5, 0x1C56, 0x319A, 0x2F4B, 0x1271, 
        0x165C, 0x119F, 0x161D, 0x3F2B, 0x0D35, 0x0B7E, 0x3935, 0x146A, 0x3687, 0x1A60, 0x0892, 0x1D5B, 0x0E3A, 0x1659, 0x1CEB, 0x21F8, 
        0x2E07, 0x231F, 0x1C82, 0x1E83, 0x0813, 0x3281, 0x05C1, 0x307E, 0x2E83, 0x3C3D, 0x2D15, 0x17E2, 0x1F83, 0x1D4A, 0x08FE, 0x2682, 
        0x1623, 0x2440, 0x1104, 0x3017, 0x026C, 0x261F, 0x3885, 0x2CDF, 0x2F96, 0x37C6, 0x1A08, 0x19D5, 0x2F27, 0x016C, 0x0DB1, 0x113F, 
        0x1811, 0x0D75, 0x0FBB, 0x2AB0, 0x23A9, 0x0710, 0x2B91, 0x2346, 0x131B, 0x0ADD, 0x2690, 0x0D38, 0x3E80, 0x2E3A, 0x19A8, 0x3988, 
        0x0182, 0x3BBC, 0x26F3, 0x20B9, 0x14AF, 0x283C, 0x1620, 0x11D0, 0x05A6, 0x3FFC, 0x1127, 0x1DF5, 0x24A8, 0x02EA, 0x17D3, 0x0558, 
        0x0DAD, 0x3009, 0x3D09, 0x3B5E, 0x2FFB, 0x1A59, 0x1598, 0x01EB, 0x0FE1, 0x2ED9, 0x26BA, 0x19FF, 0x3B2C, 0x380F, 0x2431, 0x0EF4, 
        0x36FB, 0x377E, 0x0B28, 0x24B5, 0x3F89, 0x320F, 0x1D34, 0x2456, 0x2116, 0x1163, 0x2EE0, 0x1B72, 0x14F3, 0x241F, 0x0FDD, 0x135A, 
        0x2AD3, 0x0FAF, 0x1B8E, 0x25D4, 0x12D9, 0x1DE3, 0x3A6B, 0x3566, 0x368F, 0x21DB, 0x2767, 0x0DBC, 0x1298, 0x123C, 0x0959, 0x3559, 
        0x22FE, 0x2CD2, 0x0B23, 0x21B6, 0x2D5D, 0x2CC0, 0x34DB, 0x2E16, 0x0C0F, 0x359F, 0x1671, 0x0AAC, 0x33CA, 0x0D95, 0x2671, 0x3FD5, 
        0x3918, 0x09D7, 0x0470, 0x0C5D, 0x3346, 0x2CF5, 0x2F2B, 0x2F5A, 0x2F52, 0x027E, 0x17DB, 0x1FC6, 0x3F8F, 0x1A37, 0x3136, 0x1658, 
        0x0427, 0x2BC4, 0x314B, 0x1938, 0x3BE7, 0x1851, 0x3888, 0x2CA3, 0x2109, 0x2380, 0x26D3, 0x3B96, 0x0477, 0x2A15, 0x08C8, 0x2C4E, 
        0x1050, 0x39B5, 0x31AF, 0x361F, 0x23A6, 0x2475, 0x0CE8, 0x13A1, 0x3E42, 0x1DAC, 0x3A8F, 0x22DD, 0x3613, 0x14C8, 0x1AE0, 0x26BB, 
        0x2230, 0x1C12, 0x3CEB, 0x1DF0, 0x185F, 0x35F1, 0x0713, 0x240B, 0x22D6, 0x13B3, 0x243C, 0x382B, 0x01B2, 0x03E1, 0x1077, 0x0ECB, 
        0x16C0, 0x13EE, 0x2288, 0x2BB8, 0x129E, 0x05AF, 0x0FA6, 0x105A, 0x19E3, 0x167D, 0x35BC, 0x3432, 0x19C4, 0x2050, 0x28A4, 0x0AFF, 
        0x0331, 0x029B, 0x3D7D, 0x2604, 0x2409, 0x0154, 0x3CCF, 0x1B08, 0x1BBA, 0x0DCF, 0x1BA0, 0x2791, 0x22B9, 0x1510, 0x3D5B, 0x3097, 
        0x0765, 0x0D26, 0x2BFA, 0x0E5A, 0x3FB2, 0x3727, 0x3F68, 0x0DEC, 0x2653, 0x1925, 0x1C59, 0x3E99, 0x3AF9, 0x043B, 0x3293, 0x2D9E, 
        0x0E84, 0x144D, 0x37DD, 0x3175, 0x3ACD, 0x3054, 0x3D59, 0x3966, 0x2B00, 0x3741, 0x187A, 0x14B4, 0x26ED, 0x3117, 0x264D, 0x3E07, 
        0x0C42, 0x3343, 0x199A, 0x11C6, 0x3433, 0x2DD2, 0x36D2, 0x1AA9, 0x3967, 0x1E97, 0x1407, 0x1B56, 0x2A77, 0x1A6E, 0x191C, 0x02D7, 
        0x1179, 0x116B, 0x3A7D, 0x39B2, 0x2670, 0x3F70, 0x26AF, 0x05E0, 0x1FD0, 0x2473, 0x2D70, 0x2516, 0x0480, 0x0500, 0x193E, 0x2D5C, 
        0x23A5, 0x35EA, 0x03AD, 0x06AB, 0x24D8, 0x3323, 0x244E, 0x2961, 0x0CD5, 0x30F3, 0x0450, 0x22C0, 0x19D6, 0x3F4C, 0x15DA, 0x27D0, 
        0x25A0, 0x2FAC, 0x086B, 0x0F3C, 0x0D36, 0x045B, 0x1E17, 0x2210, 0x24BA, 0x3A45, 0x138F, 0x22C1, 0x0363, 0x3CC6, 0x3B9A, 0x21B7, 
        0x1C99, 0x2B43, 0x260E, 0x07C2, 0x3297, 0x368B, 0x03DD, 0x3039, 0x281C, 0x28B4, 0x39C3, 0x21DE, 0x20E0, 0x34CA, 0x33D3, 0x0E11, 
        0x2358, 0x3A54, 0x08AA, 0x35D8, 0x2E21, 0x16B1, 0x28FD, 0x2B5B, 0x3F65, 0x2E1F, 0x1BFE, 0x3BC9, 0x18B7, 0x11C1, 0x3A70, 0x12B9, 
        0x0BE2, 0x0439, 0x0CC6, 0x19A7, 0x2BA3, 0x0638, 0x3367, 0x2494, 0x3621, 0x04D3, 0x30D5, 0x109E, 0x304C, 0x1A04, 0x0B4A, 0x39C2, 
        0x3D15, 0x05FF, 0x167F, 0x1932, 0x3720, 0x31ED, 0x2DDD, 0x0160, 0x2E58, 0x0D58, 0x22DC, 0x3E0C, 0x120A, 0x328C, 0x2323, 0x14E9, 
        0x385E, 0x1360, 0x1BDA, 0x3AD3, 0x3A29, 0x1437, 0x0799, 0x019E, 0x3C91, 0x23AF, 0x32A2, 0x1900, 0x0FA9, 0x1BC3, 0x04E0, 0x1B2D, 
        0x1F53, 0x3C99, 0x12E1, 0x2BA0, 0x27C3, 0x2AD4, 0x1FDB, 0x2DDF, 0x207E, 0x0747, 0x1A33, 0x3800, 0x03EC, 0x27A0, 0x1BB3, 0x1C7D, 
        0x34F3, 0x011F, 0x1FDD, 0x0C8A, 0x0E65, 0x2C53, 0x119D, 0x1217, 0x28E1, 0x19F3, 0x1A40, 0x3A78, 0x1DC8, 0x3A41, 0x2F95, 0x3DFA, 
        0x2F2C, 0x03FF, 0x256F, 0x23D3, 0x3EF7, 0x1707, 0x3487, 0x1EC6, 0x231E, 0x3E37, 0x2695, 0x3AA8, 0x363C, 0x0419, 0x07BB, 0x3517, 
        0x243B, 0x2C79, 0x3EE5, 0x1F82, 0x0A71, 0x31E6, 0x1BA9, 0x17C2, 0x3FD7, 0x210F, 0x3374, 0x3EC2, 0x3CD0, 0x19DD, 0x2429, 0x1B02, 
        0x1C6B, 0x2959, 0x3ADC, 0x2B65, 0x316F, 0x1BA2, 0x3FD0, 0x2124, 0x016D, 0x3801, 0x0DF0, 0x3746, 0x1C02, 0x2003, 0x01A7, 0x3E29, 
        0x2917, 0x378D, 0x29E1, 0x1D20, 0x0AEB, 0x376B, 0x229A, 0x0B12, 0x2DAE, 0x2D71, 0x1A8C, 0x12F7, 0x1CCF, 0x0D00, 0x1874, 0x28D9, 
        0x36A9, 0x1352, 0x2374, 0x384B, 0x38A2, 0x29CD, 0x270E, 0x222B, 0x1DE0, 0x11EB, 0x0381, 0x32AC, 0x1F0E, 0x0D10, 0x2EC5, 0x11EE, 
        0x0E9E, 0x0BEE, 0x1B31, 0x2811, 0x2A7E, 0x3D9C, 0x30EA, 0x261D, 0x285F, 0x1B07, 0x2058, 0x0BCE, 0x1AD7, 0x20F0, 0x395B, 0x3B56, 
        0x0D2E, 0x1838, 0x19BE, 0x11AC, 0x026E, 0x0A60, 0x255D, 0x1E1E, 0x2E98, 0x0C35, 0x1E2A, 0x08CD, 0x272E, 0x32B7, 0x071E, 0x32F1, 
        0x38AE, 0x0BE0, 0x0498, 0x3BAA, 0x2060, 0x2D79, 0x228D, 0x3A21, 0x3059, 0x0EE7, 0x15B6, 0x2ABE, 0x3486, 0x16C6, 0x29E0, 0x2220, 
        0x307A, 0x03F3, 0x0F3A, 0x2B62, 0x27B7, 0x11B8, 0x358E, 0x056E, 0x0B60, 0x0219, 0x282A, 0x32BE, 0x3302, 0x06D8, 0x08C9, 0x28D4, 
        0x0673, 0x0104, 0x27BD, 0x2A26, 0x3EF9, 0x2B87, 0x0983, 0x1619, 0x259D, 0x36A8, 0x3BAE, 0x07B6, 0x2C54, 0x14E7, 0x1753, 0x1F4F, 
        0x3FA5, 0x2407, 0x37CB, 0x38FA, 0x20E8, 0x0DD2, 0x396B, 0x2AB6, 0x0163, 0x32DB, 0x0491, 0x2A87, 0x05F1, 0x2EA6, 0x2295, 0x2C9B, 
        0x3D62, 0x34A0, 0x18AD, 0x15F8, 0x023E, 0x35A1, 0x3F63, 0x3422, 0x0137, 0x07B5, 0x1C45, 0x1DCE, 0x3414, 0x0E37, 0x2398, 0x0EB1, 
        0x1058, 0x283B, 0x36D8, 0x1BFC, 0x17D7, 0x1027, 0x1D73, 0x1DA2, 0x1003, 0x3144, 0x16A4, 0x0F99, 0x17A0, 0x2AF0, 0x344C, 0x3E47, 
        0x0B9D, 0x18B8, 0x11C9, 0x159B, 0x15E6, 0x021A, 0x1644, 0x2006, 0x304D, 0x3379, 0x0F5E, 0x064B, 0x1102, 0x3630, 0x3CB6, 0x3FBA, 
        0x1DBA, 0x2873, 0x04AA, 0x145B, 0x22A7, 0x3594, 0x18D2, 0x3B86, 0x3438, 0x3083, 0x1141, 0x23A4, 0x23DB, 0x363E, 0x175A, 0x2D6E, 
        0x2EBA, 0x22A9, 0x3076, 0x1781, 0x1565, 0x21C3, 0x0B17, 0x399B, 0x18AB, 0x228F, 0x05D7, 0x1BF0, 0x2523, 0x34DC, 0x322E, 0x19CC, 
        0x1E01, 0x19DE, 0x27C0, 0x1841, 0x3FDD, 0x10BC, 0x0371, 0x03BA, 0x3F15, 0x356C, 0x0DD4, 0x37FA, 0x2546, 0x1E5A, 0x2CB4, 0x3165, 
        0x0DA0, 0x3CA4, 0x1320, 0x398F, 0x0DB9, 0x23DF, 0x2448, 0x14BD, 0x1F3A, 0x1A5B, 0x0A4E, 0x1C92, 0x03E0, 0x0774, 0x08EF, 0x0E0A, 
        0x1F12, 0x29E7, 0x15EF, 0x2A51, 0x125D, 0x153E, 0x3D20, 0x23A2, 0x370C, 0x07CE, 0x1CE2, 0x2361, 0x0632, 0x1A7A, 0x2586, 0x094E, 
        0x383A, 0x35CC, 0x06AF, 0x20C5, 0x0FA1, 0x3875, 0x0376, 0x027F, 0x143D, 0x1DDF, 0x3F85, 0x266A, 0x1581, 0x2DF5, 0x0F53, 0x2FD7, 
        0x2BAB, 0x0F60, 0x0DC6, 0x0D88, 0x2776, 0x3110, 0x2258, 0x1CA3, 0x2289, 0x039E, 0x32BA, 0x09DA, 0x3CF9, 0x02F1, 0x03D5, 0x2833, 
        0x1F21, 0x23CA, 0x3227, 0x3E11, 0x23AB, 0x2A31, 0x1BDB, 0x2286, 0x01FD, 0x0F7C, 0x1638, 0x1BB8, 0x3B6D, 0x319E, 0x1108, 0x1382, 
        0x0AAB, 0x3460, 0x2F0D, 0x3567, 0x0F68, 0x2A22, 0x1C38, 0x2851, 0x02BA, 0x09E7, 0x2914, 0x35AA, 0x14FB, 0x3428, 0x0E91, 0x0E19, 
        0x2C63, 0x084A, 0x0695, 0x2A1F, 0x1BAD, 0x2683, 0x29BA, 0x1F37, 0x0C4A, 0x1F72, 0x1611, 0x1C88, 0x1801, 0x3A55, 0x2632, 0x09DC, 
        0x13B8, 0x26C8, 0x27D5, 0x19FD, 0x2EE2, 0x1F69, 0x0D55, 0x324F, 0x1203, 0x33DC, 0x3C08, 0x1713, 0x3648, 0x34A1, 0x1174, 0x30BA, 
        0x2152, 0x0E3C, 0x2336, 0x3CE8, 0x025E, 0x1443, 0x2412, 0x136B, 0x1401, 0x03A1, 0x26DE, 0x2FE1, 0x2301, 0x0AFE, 0x355E, 0x1C61, 
        0x0809, 0x1A27, 0x2A34, 0x35C4, 0x0F94, 0x05EC, 0x08EC, 0x1E0B, 0x26C6, 0x3F3F, 0x253F, 0x318D, 0x15D5, 0x2457, 0x315F, 0x1FA2, 
        0x10B2, 0x3C16, 0x0DA3, 0x3C51, 0x22AC, 0x3AA5, 0x1916, 0x071A, 0x2B0B, 0x3A17, 0x3C66, 0x2F17, 0x38A3, 0x2332, 0x3A4B, 0x1E3B, 
        0x0DB0, 0x1B97, 0x05F4, 0x1C9A, 0x2418, 0x0F3E, 0x3B5B, 0x0781, 0x1D01, 0x2078, 0x23E4, 0x15E4, 0x1B06, 0x1DFF, 0x27E6, 0x0956, 
        0x3A2D, 0x0C83, 0x3127, 0x15DC, 0x0B16, 0x1445, 0x0FEC, 0x079D, 0x172F, 0x0F9E, 0x0987, 0x164E, 0x1654, 0x1150, 0x2B5F, 0x32BD, 
        0x3641, 0x3037, 0x30A8, 0x3A92, 0x195E, 0x0C26, 0x2B96, 0x35CA, 0x18E1, 0x3479, 0x17A1, 0x1613, 0x11CE, 0x1AC1, 0x3831, 0x183E, 
        0x36CD, 0x0984, 0x16B4, 0x1BD5, 0x10CE, 0x3923, 0x1953, 0x075F, 0x3F4A, 0x3671, 0x0C7F, 0x0422, 0x3F90, 0x087E, 0x321A, 0x1CD6, 
        0x19AA, 0x3B58, 0x03AF, 0x3475, 0x392C, 0x0C84, 0x2EB8, 0x3754, 0x147F, 0x2C3A, 0x1C01, 0x3D0B, 0x2602, 0x16E9, 0x39BB, 0x33BD, 
        0x35D0, 0x109A, 0x0DE0, 0x284E, 0x2D78, 0x31FB, 0x3878, 0x3E10, 0x0337, 0x219E, 0x1C93, 0x15A8, 0x1D21, 0x10B5, 0x2DCC, 0x1C78, 
        0x0BBA, 0x398E, 0x281B, 0x2073, 0x3211, 0x3733, 0x1BAB, 0x3DB4, 0x3431, 0x36CF, 0x0771, 0x1F3F, 0x36BD, 0x297B, 0x3CD8, 0x3DD2, 
        0x3663, 0x3A4C, 0x2875, 0x014E, 0x3542, 0x244D, 0x10F7, 0x3928, 0x0E14, 0x25BB, 0x0A46, 0x3726, 0x1587, 0x11A7, 0x10E6, 0x3F58, 
        0x16C7, 0x01D9, 0x174E, 0x03BD, 0x162F, 0x2E86, 0x2562, 0x24A1, 0x0255, 0x33FA, 0x3D60, 0x18C4, 0x0F9B, 0x350A, 0x3606, 0x017C, 
        0x3ADD, 0x0247, 0x3811, 0x254B, 0x3F71, 0x262E, 0x335B, 0x1568, 0x385B, 0x1B6C, 0x1580, 0x14D6, 0x2D81, 0x36F6, 0x388E, 0x1D6D, 
        0x10CF, 0x09E5, 0x01DD, 0x3C5A, 0x2A71, 0x2C24, 0x327E, 0x22A4, 0x185E, 0x1827, 0x0F01, 0x07F1, 0x1C0E, 0x162B, 0x3A0E, 0x1D07, 
        0x3C03, 0x1A03, 0x20B0, 0x0886, 0x0562, 0x2556, 0x237F, 0x152D, 0x2F57, 0x24A6, 0x291B, 0x2103, 0x2956, 0x3A58, 0x38A8, 0x1762, 
        0x2BC2, 0x3ED1, 0x2FF0, 0x02BC, 0x172A, 0x0D81, 0x2181, 0x2FC6, 0x1E1D, 0x3957, 0x263C, 0x1520, 0x172E, 0x3C44, 0x01FC, 0x3147, 
        0x3275, 0x2BE7, 0x2498, 0x056D, 0x10C9, 0x052C, 0x22E6, 0x1F2C, 0x139C, 0x2ED4, 0x06BA, 0x11E4, 0x22FB, 0x0196, 0x2800, 0x16CD, 
        0x39BF, 0x32B5, 0x297E, 0x19CA, 0x2273, 0x3F7F, 0x18C1, 0x10A8, 0x18BD, 0x0FCF, 0x134A, 0x0A83, 0x2C50, 0x1A15, 0x2322, 0x08CA, 
        0x0736, 0x0942, 0x0913, 0x2052, 0x0C39, 0x2F37, 0x15A4, 0x0DD6, 0x1075, 0x291D, 0x0721, 0x0FF7, 0x02EE, 0x2A86, 0x3CEA, 0x1964, 
        0x18E3, 0x1AE7, 0x0740, 0x05DF, 0x2AA6, 0x3721, 0x39AE, 0x2A68, 0x24E5, 0x0175, 0x3AC9, 0x0AD4, 0x3042, 0x1683, 0x2C60, 0x27CF, 
        0x28BC, 0x1CC7, 0x1305, 0x038E, 0x02A1, 0x260B, 0x2CD0, 0x29F2, 0x2FD0, 0x1FC8, 0x2A2A, 0x3CB5, 0x2667, 0x0382, 0x1035, 0x3770, 
        0x31BC, 0x11D7, 0x1946, 0x0953, 0x0F18, 0x08A2, 0x1350, 0x0968, 0x2240, 0x3535, 0x0749, 0x2AB5, 0x07F9, 0x356F, 0x03E8, 0x1188, 
        0x0DAB, 0x3BFF, 0x13E0, 0x0194, 0x182C, 0x10A5, 0x0DCD, 0x2F87, 0x125C, 0x3B20, 0x0C01, 0x31C0, 0x3DF0, 0x36AB, 0x2F9C, 0x0460, 
        0x1D9E, 0x2CCA, 0x22B8, 0x3CC8, 0x3C1A, 0x16EC, 0x38A4, 0x1DB4, 0x1199, 0x2CED, 0x2400, 0x37D5, 0x37FD, 0x19D3, 0x20C9, 0x0F17, 
        0x2510, 0x2D6F, 0x37FB, 0x1B9B, 0x3624, 0x1A51, 0x1089, 0x132B, 0x2C0C, 0x1E65, 0x2673, 0x2F22, 0x0C1F, 0x3C55, 0x23EE, 0x0185, 
        0x3B0E, 0x03CD, 0x0D03, 0x3738, 0x3FF4, 0x3521, 0x36C8, 0x2447, 0x0E57, 0x2B3A, 0x0503, 0x1F20, 0x3893, 0x3B99, 0x26A9, 0x1600, 
        0x0647, 0x1740, 0x01BB, 0x0E4F, 0x17F3, 0x1E47, 0x2A07, 0x2513, 0x0812, 0x292D, 0x2820, 0x33B4, 0x37EA, 0x19EE, 0x0A01, 0x0228, 
        0x2450, 0x0C75, 0x07F4, 0x378E, 0x3142, 0x05F8, 0x0982, 0x34D6, 0x1A73, 0x082D, 0x0D27, 0x3E20, 0x0843, 0x099B, 0x18BA, 0x13F5, 
        0x3E17, 0x0191, 0x320E, 0x3826, 0x27D7, 0x0116, 0x0A52, 0x20B3, 0x2FFC, 0x30D2, 0x33FD, 0x0350, 0x213E, 0x3C62, 0x0105, 0x2B64, 
        0x1242, 0x2860, 0x08D1, 0x0F5C, 0x1145, 0x225D, 0x1111, 0x0E5D, 0x3015, 0x1F11, 0x2669, 0x1D76, 0x270B, 0x3BB3, 0x333F, 0x1D13, 
        0x0F51, 0x24C9, 0x0E01, 0x37C3, 0x1DA6, 0x1FCD, 0x34E1, 0x2170, 0x2BFC, 0x3586, 0x1A84, 0x3C59, 0x3AE9, 0x269F, 0x3BD5, 0x1861, 
        0x2A4D, 0x1309, 0x15F1, 0x3929, 0x3709, 0x1A02, 0x03BE, 0x09FE, 0x1AA5, 0x200A, 0x0C81, 0x3701, 0x303F, 0x0AB7, 0x1645, 0x33C8, 
        0x1376, 0x0B53, 0x07FB, 0x0E92, 0x3FB9, 0x3697, 0x037C, 0x2ED5, 0x01B5, 0x1327, 0x2CA6, 0x0153, 0x1551, 0x03D6, 0x2782, 0x0DD3, 
        0x0688, 0x2172, 0x2372, 0x014F, 0x0FB7, 0x1706, 0x1C4F, 0x1B90, 0x2F55, 0x0215, 0x1F01, 0x24C0, 0x01E1, 0x086F, 0x1886, 0x15D3, 
        0x1D66, 0x37FC, 0x2E9F, 0x14A0, 0x09CC, 0x1709, 0x0697, 0x2A80, 0x25B1, 0x3B21, 0x2B71, 0x3011, 0x36B0, 0x289F, 0x16DC, 0x203C, 
        0x3DDE, 0x3D33, 0x1608, 0x1FB1, 0x15F5, 0x07D5, 0x1E9B, 0x3E4D, 0x1F94, 0x0D93, 0x3D0D, 0x0CAA, 0x3F5D, 0x04D1, 0x2019, 0x0D56, 
        0x3F5B, 0x1C09, 0x36F1, 0x2DFE, 0x01BE, 0x3D6B, 0x2543, 0x3A26, 0x10D8, 0x2072, 0x201A, 0x38D4, 0x36FE, 0x20A7, 0x302D, 0x1816, 
        0x18DC, 0x06B9, 0x24D2, 0x3A40, 0x381E, 0x0967, 0x0B42, 0x287D, 0x08CC, 0x2749, 0x2A70, 0x0F37, 0x2ED3, 0x33FE, 0x076F, 0x3A35, 
        0x1738, 0x388D, 0x0BFC, 0x07C7, 0x0A02, 0x04CC, 0x0E6C, 0x285A, 0x2A3E, 0x1881, 0x26A5, 0x2C20, 0x352D, 0x3B48, 0x04F8, 0x22E4, 
        0x3D25, 0x3FDA, 0x3D49, 0x33DA, 0x1E21, 0x2205, 0x10FC, 0x3C0F, 0x1EE5, 0x109D, 0x2748, 0x07EF, 0x25E6, 0x32A6, 0x3A32, 0x2212, 
        0x09A9, 0x0A59, 0x239B, 0x3536, 0x0CED, 0x2271, 0x1BC8, 0x2A1D, 0x0601, 0x1282, 0x32F8, 0x085C, 0x1774, 0x1F46, 0x341D, 0x24ED, 
        0x3167, 0x0F03, 0x0644, 0x37A3, 0x0278, 0x222C, 0x2C42, 0x0EA1, 0x0C3E, 0x1F58, 0x3CF1, 0x1878, 0x02BE, 0x3E6A, 0x28E6, 0x04C8, 
        0x341B, 0x2DA0, 0x100C, 0x090B, 0x206A, 0x1CC4, 0x0A0C, 0x0893, 0x1696, 0x18B3, 0x0FFC, 0x3553, 0x1321, 0x11C7, 0x14A5, 0x10FB, 
        0x20A1, 0x1ADE, 0x2F32, 0x2D51, 0x0D06, 0x3CDE, 0x271C, 0x0FC4, 0x0186, 0x3767, 0x0801, 0x03C2, 0x2F74, 0x025A, 0x13EC, 0x2A3A, 
        0x22B3, 0x2CBE, 0x11CA, 0x2A88, 0x145A, 0x0AC9, 0x14F5, 0x3A69, 0x098D, 0x1B30, 0x3D7C, 0x0136, 0x23BF, 0x3591, 0x2C92, 0x248C, 
        0x14CA, 0x06C1, 0x096B, 0x3BB9, 0x093A, 0x296C, 0x2EE1, 0x06B5, 0x1E59, 0x3ACB, 0x3FA8, 0x20D5, 0x3D4E, 0x1E52, 0x0C44, 0x01EE, 
        0x22DA, 0x3626, 0x0DBE, 0x0DD1, 0x2E63, 0x1830, 0x1B46, 0x1FEF, 0x12A4, 0x1B62, 0x1060, 0x3043, 0x3863, 0x2130, 0x15D6, 0x05CA, 
        0x09C9, 0x29A8, 0x2560, 0x3EE6, 0x3C82, 0x27E1, 0x1769, 0x1F0A, 0x21BE, 0x1D95, 0x1BEB, 0x27DE, 0x2D28, 0x20AD, 0x1AA6, 0x3A95, 
        0x1FE8, 0x138B, 0x0880, 0x013D, 0x39C0, 0x29C6, 0x3C79, 0x364F, 0x030B, 0x12C4, 0x1096, 0x1FFE, 0x29DE, 0x30F6, 0x1A18, 0x0F44, 
        0x1BD7, 0x2C64, 0x1DC7, 0x3F9B, 0x2DA3, 0x3ABE, 0x0CF2, 0x2A90, 0x14F6, 0x2DA5, 0x2D04, 0x1425, 0x1978, 0x2629, 0x13F4, 0x2413, 
        0x2570, 0x23B2, 0x171B, 0x032A, 0x1491, 0x0609, 0x1168, 0x1346, 0x1F89, 0x3F72, 0x2913, 0x134D, 0x11CB, 0x0BB7, 0x3736, 0x2247, 
        0x0387, 0x0134, 0x039D, 0x3968, 0x18FF, 0x0A93, 0x2D16, 0x1286, 0x28DD, 0x044B, 0x0F3B, 0x35B8, 0x373C, 0x3FDB, 0x338F, 0x0743, 
        0x0D18, 0x0E0F, 0x142E, 0x13CE, 0x398D, 0x095F, 0x32FA, 0x339A, 0x19B9, 0x0E83, 0x0830, 0x2B17, 0x1C37, 0x0F4E, 0x39EF, 0x0596, 
        0x04CA, 0x267C, 0x2ADA, 0x2E0A, 0x2B10, 0x3EF1, 0x3EB6, 0x1B55, 0x18FA, 0x3217, 0x3996, 0x147D, 0x12D5, 0x34A6, 0x3181, 0x1806, 
        0x26DB, 0x3C29, 0x07E6, 0x06ED, 0x312D, 0x2357, 0x0867, 0x23D6, 0x293C, 0x3C84, 0x06CA, 0x0F92, 0x393A, 0x2796, 0x02A5, 0x2BD6, 
        0x0E28, 0x31BB, 0x2110, 0x238A, 0x2B7B, 0x18A6, 0x1EED, 0x0C67, 0x2AF8, 0x3AC7, 0x3DA4, 0x187C, 0x30A4, 0x033A, 0x1C58, 0x173E, 
        0x108B, 0x2190, 0x0EBF, 0x394E, 0x2D2A, 0x251B, 0x203A, 0x1D77, 0x2B61, 0x15C6, 0x32E4, 0x3DFD, 0x2C05, 0x18C2, 0x1AD1, 0x1209, 
        0x0AD2, 0x2BEE, 0x0C33, 0x3282, 0x3B51, 0x3E58, 0x305A, 0x029F, 0x24E6, 0x0EF3, 0x1E0E, 0x0466, 0x1181, 0x1951, 0x1AF2, 0x1B32, 
        0x026D, 0x1E5C, 0x220B, 0x32D5, 0x08ED, 0x0772, 0x053D, 0x0693, 0x34EB, 0x039C, 0x33C0, 0x08F5, 0x1CE3, 0x1A10, 0x1C13, 0x34A8, 
        0x067A, 0x1D38, 0x3BEA, 0x0B87, 0x2E05, 0x3816, 0x21AF, 0x194A, 0x3073, 0x11B9, 0x323D, 0x2D8F, 0x0161, 0x0802, 0x3219, 0x347F, 
        0x0ABE, 0x08A9, 0x2CF7, 0x1873, 0x08E5, 0x2986, 0x17B4, 0x3571, 0x3E5E, 0x24F3, 0x06F7, 0x3061, 0x181C, 0x0F27, 0x0B56, 0x0D97, 
        0x17D9, 0x3DF1, 0x3832, 0x3B05, 0x1AED, 0x3EA2, 0x31D7, 0x333C, 0x1098, 0x2270, 0x38BC, 0x0986, 0x37A1, 0x2555, 0x1941, 0x2A79, 
        0x06D1, 0x1D5C, 0x2C2A, 0x122E, 0x0DBD, 0x14CF, 0x30C5, 0x0FC1, 0x1065, 0x3C6A, 0x3EDC, 0x318C, 0x3B10, 0x2B01, 0x3040, 0x02C2, 
        0x333B, 0x26AE, 0x379C, 0x0A4F, 0x22AF, 0x18A4, 0x31FC, 0x3615, 0x1BD4, 0x086E, 0x2DE4, 0x3DB0, 0x15B7, 0x1125, 0x3874, 0x0800, 
        0x28BA, 0x2AB9, 0x057A, 0x3A02, 0x29CF, 0x13C8, 0x242C, 0x0C6E, 0x03F9, 0x29B9, 0x0F08, 0x1C14, 0x248F, 0x0F6E, 0x26EE, 0x0C21, 
        0x04EB, 0x1745, 0x1D11, 0x2CD3, 0x2E90, 0x3A4E, 0x32F9, 0x1733, 0x0234, 0x0C30, 0x0D78, 0x1BC2, 0x0547, 0x3EF6, 0x093D, 0x32EE, 
        0x0BD6, 0x36EF, 0x1FEB, 0x0F41, 0x3AFD, 0x0CB7, 0x1F84, 0x2134, 0x3B71, 0x0A22, 0x1483, 0x079F, 0x19B4, 0x2174, 0x2487, 0x3756, 
        0x081E, 0x3CD7, 0x1A83, 0x34BD, 0x38D9, 0x125E, 0x1061, 0x049E, 0x215D, 0x303B, 0x066A, 0x1156, 0x1A54, 0x2951, 0x0912, 0x159D, 
        0x08E7, 0x1628, 0x1650, 0x2684, 0x237D, 0x34A7, 0x0F0E, 0x03EF, 0x168B, 0x1798, 0x37E0, 0x07C6, 0x28AC, 0x1019, 0x26DD, 0x1FFF, 
        0x14F1, 0x374B, 0x3866, 0x155D, 0x2737, 0x0808, 0x32CB, 0x1D0D, 0x35C1, 0x3338, 0x0A98, 0x2179, 0x1CD3, 0x1A66, 0x1C51, 0x2C82, 
        0x05A1, 0x216F, 0x14AE, 0x14D4, 0x16CB, 0x3C42, 0x2BBE, 0x37AC, 0x112B, 0x0EB3, 0x322A, 0x2E28, 0x193C, 0x1E4A, 0x18A5, 0x3527, 
        0x0C10, 0x2B9E, 0x1261, 0x036A, 0x3961, 0x2478, 0x13C2, 0x18CC, 0x0125, 0x062F, 0x0179, 0x35AF, 0x1E43, 0x04F0, 0x3A1A, 0x10B3, 
        0x2AAC, 0x20AA, 0x11FB, 0x2540, 0x1823, 0x23CB, 0x2FB4, 0x02F6, 0x1C03, 0x26AB, 0x153D, 0x0699, 0x2488, 0x2438, 0x168E, 0x041D, 
        0x3DBC, 0x25E5, 0x25FF, 0x3AEE, 0x054E, 0x3233, 0x012B, 0x2E89, 0x224E, 0x34CC, 0x3B93, 0x2C2D, 0x1990, 0x36F4, 0x346E, 0x1EFE, 
        0x286F, 0x3654, 0x2E2E, 0x2DED, 0x399D, 0x2CB2, 0x302B, 0x3A2B, 0x37B1, 0x227C, 0x3D52, 0x3023, 0x12AE, 0x2460, 0x0890, 0x32E3, 
        0x397B, 0x16D0, 0x3D5F, 0x1FA9, 0x1B00, 0x3D22, 0x2764, 0x2D56, 0x1831, 0x3FC9, 0x2C98, 0x0A48, 0x3585, 0x0D73, 0x096F, 0x1C9B, 
        0x1514, 0x3B1F, 0x35CD, 0x01D7, 0x0A20, 0x2514, 0x2927, 0x2FEE, 0x1BDD, 0x0B0C, 0x07A3, 0x1433, 0x3551, 0x331C, 0x18B0, 0x0DE9, 
        0x2AE0, 0x0D62, 0x1C8C, 0x2AA5, 0x14A3, 0x20A8, 0x1115, 0x04A6, 0x3CE7, 0x2565, 0x1669, 0x0A1B, 0x0A03, 0x148F, 0x3B67, 0x062B, 
        0x267F, 0x33FC, 0x014C, 0x240D, 0x28FC, 0x2F77, 0x2E81, 0x1E50, 0x3857, 0x3612, 0x18F0, 0x1771, 0x2EA1, 0x141A, 0x236E, 0x348C, 
        0x2568, 0x1889, 0x2CE2, 0x37EC, 0x38B9, 0x19AC, 0x0756, 0x1570, 0x1F55, 0x04D9, 0x0F1D, 0x07DC, 0x3984, 0x1365, 0x1FE7, 0x318F, 
        0x05AB, 0x1B9A, 0x2001, 0x2BBF, 0x1BA4, 0x3A46, 0x01F8, 0x0AA2, 0x3F7D, 0x3563, 0x1BF8, 0x1383, 0x2DDC, 0x171A, 0x251A, 0x38E1, 
        0x3193, 0x35C3, 0x060D, 0x23C8, 0x0EDE, 0x0CCE, 0x2DEB, 0x23C9, 0x25B8, 0x0A26, 0x08F3, 0x3834, 0x23ED, 0x2D03, 0x0ACB, 0x1A43, 
        0x34BC, 0x35A3, 0x0505, 0x2BAA, 0x1D67, 0x06DD, 0x3D42, 0x0E9D, 0x144F, 0x2A75, 0x0542, 0x058C, 0x0B47, 0x2CAE, 0x0EBC, 0x17BE, 
        0x0DCB, 0x1DC0, 0x2D00, 0x2CEA, 0x2279, 0x3F6C, 0x08DA, 0x08FA, 0x3791, 0x334D, 0x1231, 0x1194, 0x3CDB, 0x0966, 0x2A60, 0x3554, 
        0x0CB5, 0x10C8, 0x3105, 0x0F07, 0x1D70, 0x2FC2, 0x3B88, 0x361A, 0x3353, 0x26EA, 0x13C6, 0x1086, 0x1B7F, 0x3EBB, 0x098C, 0x2AD2, 
        0x0B9F, 0x0554, 0x017D, 0x2CE5, 0x1107, 0x0B29, 0x0AD1, 0x04A4, 0x346A, 0x1303, 0x0B6E, 0x3D2D, 0x33E8, 0x2AE7, 0x2005, 0x1BDE, 
        0x1F99, 0x0566, 0x2638, 0x36DA, 0x3376, 0x31C6, 0x1C4C, 0x1B3F, 0x2C91, 0x3947, 0x0D25, 0x1F4A, 0x38B7, 0x1C96, 0x0C07, 0x29FA, 
        0x127E, 0x27B4, 0x25D9, 0x26B4, 0x1A8D, 0x1133, 0x0BB0, 0x22FC, 0x3C19, 0x060B, 0x3719, 0x3B80, 0x1C89, 0x18CE, 0x1450, 0x254D, 
        0x3B04, 0x1691, 0x3EDF, 0x04E7, 0x24E3, 0x274E, 0x0EC3, 0x366D, 0x2139, 0x3F8E, 0x1016, 0x0862, 0x2966, 0x1069, 0x2DC0, 0x2608, 
        0x0139, 0x2FCD, 0x250E, 0x1240, 0x2D9A, 0x0A73, 0x3B98, 0x0BD1, 0x040C, 0x3818, 0x1821, 0x0B36, 0x0A37, 0x3B62, 0x3B89, 0x161C, 
        0x2AFA, 0x2151, 0x1E91, 0x231B, 0x26D2, 0x2FEA, 0x13B1, 0x0CEB, 0x360B, 0x1A72, 0x28B8, 0x12C8, 0x36CA, 0x3D21, 0x269E, 0x3562, 
        0x3303, 0x374E, 0x0642, 0x3DC4, 0x1957, 0x0908, 0x3981, 0x3EEF, 0x16D9, 0x0919, 0x3BC4, 0x3B7D, 0x2882, 0x08D5, 0x3069, 0x3F7B, 
        0x1ACD, 0x2807, 0x08E8, 0x3E88, 0x14B7, 0x0F09, 0x0C06, 0x2DC1, 0x09A4, 0x2CBF, 0x0844, 0x2A4F, 0x3A22, 0x3347, 0x23E7, 0x2933, 
        0x1AFF, 0x0380, 0x36BF, 0x17B3, 0x3BFD, 0x1699, 0x30DF, 0x0D98, 0x12A0, 0x3046, 0x3A14, 0x013A, 0x0261, 0x2C1D, 0x0A6D, 0x2EF2, 
        0x190A, 0x295B, 0x1045, 0x2FAA, 0x1640, 0x26BC, 0x01D4, 0x148E, 0x2A91, 0x09D3, 0x0553, 0x05C6, 0x1F64, 0x3418, 0x10D0, 0x3822, 
        0x3937, 0x146D, 0x3757, 0x152F, 0x346D, 0x3F10, 0x1D48, 0x2E4B, 0x1DC3, 0x1667, 0x2FA2, 0x28DA, 0x196D, 0x2647, 0x2618, 0x01C2, 
        0x2D4F, 0x15D4, 0x26F7, 0x3CA3, 0x236D, 0x2797, 0x2A3B, 0x2B2B, 0x2743, 0x1E53, 0x2941, 0x0490, 0x2582, 0x373A, 0x3E66, 0x09E1, 
        0x2219, 0x2D47, 0x0304, 0x3BB7, 0x252D, 0x015F, 0x1908, 0x1D69, 0x3DDB, 0x3D05, 0x3522, 0x29CE, 0x2DD6, 0x30FF, 0x36AA, 0x0C77, 
        0x23B1, 0x0B7B, 0x2888, 0x1B14, 0x1386, 0x0FA7, 0x0980, 0x220D, 0x17C8, 0x2978, 0x357B, 0x0684, 0x2824, 0x2AEF, 0x281A, 0x1590, 
        0x3690, 0x210E, 0x1686, 0x2200, 0x3C7B, 0x1F35, 0x3A71, 0x12B6, 0x1C7A, 0x3B91, 0x0C56, 0x11FC, 0x2B8B, 0x11C0, 0x01A1, 0x083B, 
        0x22F0, 0x0A88, 0x2023, 0x21D8, 0x0832, 0x2744, 0x3D29, 0x1B38, 0x27B1, 0x33CF, 0x0622, 0x3B02, 0x2522, 0x03F1, 0x061C, 0x1720, 
        0x0650, 0x2FB1, 0x1B26, 0x26E2, 0x0242, 0x297D, 0x3AE8, 0x093F, 0x134E, 0x0C4B, 0x29B6, 0x2A6D, 0x2539, 0x2D46, 0x3728, 0x3FC5, 
        0x2686, 0x2197, 0x0A7F, 0x2287, 0x1E6C, 0x0EED, 0x1528, 0x33BF, 0x0C7E, 0x2349, 0x3C27, 0x0DA2, 0x0728, 0x2B9F, 0x1BD9, 0x3775, 
        0x25C4, 0x0455, 0x3706, 0x213B, 0x3C3B, 0x0E8C, 0x3BDF, 0x0399, 0x0156, 0x2836, 0x3434, 0x2CCF, 0x27C4, 0x1A2D, 0x069C, 0x1588, 
        0x154A, 0x3792, 0x2668, 0x15CA, 0x17F7, 0x1DB5, 0x3735, 0x2477, 0x06DB, 0x0CF0, 0x33C9, 0x1B9E, 0x2031, 0x3AE6, 0x3EAC, 0x2DD0, 
        0x0A24, 0x34DD, 0x1FEE, 0x38B2, 0x04AC, 0x3C2D, 0x0EF1, 0x2990, 0x0EDB, 0x07DE, 0x28B2, 0x1876, 0x1660, 0x04EA, 0x18C8, 0x3833, 
        0x32F2, 0x09E6, 0x2D62, 0x214D, 0x1948, 0x30A0, 0x1E9F, 0x22ED, 0x2356, 0x2EBD, 0x351B, 0x23C5, 0x19C9, 0x2CF8, 0x1B7A, 0x3A5D, 
        0x3629, 0x3A7E, 0x2945, 0x3007, 0x3580, 0x29B4, 0x04E2, 0x1E16, 0x1522, 0x1DAA, 0x1B78, 0x1CA7, 0x2A96, 0x20E6, 0x11DB, 0x2FE4, 
        0x0DC1, 0x15FE, 0x0F31, 0x17C0, 0x3A87, 0x2DD4, 0x35B6, 0x256C, 0x1410, 0x3F6D, 0x39F8, 0x080C, 0x253E, 0x2E54, 0x1029, 0x0B09, 
        0x344E, 0x19AE, 0x1EBE, 0x20D9, 0x3F21, 0x0E9A, 0x3CB9, 0x17FE, 0x1A6C, 0x3998, 0x2A98, 0x1EC4, 0x0EBA, 0x0218, 0x3D66, 0x3C95, 
        0x0ED3, 0x0E87, 0x3FBC, 0x3084, 0x21CE, 0x1EA3, 0x2587, 0x2F53, 0x3E3F, 0x3BD8, 0x1275, 0x299E, 0x37F6, 0x3034, 0x27C9, 0x2A29, 
        0x0DB3, 0x2801, 0x2681, 0x0E0B, 0x2DE9, 0x1E8C, 0x030D, 0x2A1A, 0x1AF5, 0x2EDA, 0x3390, 0x36DE, 0x1161, 0x384C, 0x3172, 0x341A, 
        0x19A2, 0x3450, 0x2607, 0x155C, 0x3EEB, 0x0797, 0x126F, 0x3E34, 0x26CB, 0x3373, 0x3C52, 0x39C6, 0x1689, 0x332D, 0x05A3, 0x0A53, 
        0x0927, 0x1F68, 0x1C8D, 0x04A1, 0x1FF0, 0x30DB, 0x1CCD, 0x192A, 0x0837, 0x072E, 0x307D, 0x2615, 0x0AFA, 0x0932, 0x3B5C, 0x03B7, 
        0x1040, 0x15EB, 0x2750, 0x1D9C, 0x3476, 0x188B, 0x2A7A, 0x150E, 0x0597, 0x3BA5, 0x0D42, 0x23C0, 0x0598, 0x37D1, 0x2B15, 0x0178, 
        0x3FEF, 0x0386, 0x2CE0, 0x052E, 0x37F1, 0x063D, 0x09BC, 0x22CF, 0x1C22, 0x0668, 0x251E, 0x0C96, 0x23E2, 0x375A, 0x1CCE, 0x03F0, 
        0x3199, 0x0B4B, 0x2B14, 0x279F, 0x3402, 0x0635, 0x1CA6, 0x0C48, 0x376A, 0x2912, 0x332A, 0x25F1, 0x19A5, 0x0A35, 0x3E26, 0x3B23, 
        0x104B, 0x174F, 0x1DFA, 0x2B2C, 0x0E47, 0x0E55, 0x0B94, 0x37C0, 0x10CA, 0x0E94, 0x178D, 0x07DD, 0x3EE8, 0x20A0, 0x3074, 0x175D, 
        0x2B25, 0x3C69, 0x0AA0, 0x2620, 0x162E, 0x16BF, 0x3760, 0x0268, 0x3891, 0x28E2, 0x3B92, 0x359E, 0x2D0B, 0x1252, 0x06A6, 0x3DC9, 
        0x0FC3, 0x040A, 0x3474, 0x16CF, 0x0AAA, 0x0FAB, 0x0645, 0x253B, 0x12B8, 0x3F0F, 0x3A27, 0x1F86, 0x2144, 0x360F, 0x245C, 0x12CF, 
        0x20D7, 0x2C6D, 0x24A3, 0x0A0F, 0x1B87, 0x224A, 0x2D53, 0x0994, 0x2410, 0x0CD2, 0x2786, 0x0544, 0x1E15, 0x3D32, 0x1113, 0x0E20, 
        0x13D6, 0x0FDA, 0x1387, 0x1F9F, 0x1C23, 0x158A, 0x1544, 0x2853, 0x0A44, 0x097C, 0x344F, 0x22AA, 0x3FC3, 0x1DB9, 0x1992, 0x2434, 
        0x14F4, 0x1AEA, 0x10F0, 0x0792, 0x12F5, 0x3B47, 0x0CE4, 0x3B0D, 0x3DD4, 0x1A81, 0x1942, 0x088A, 0x255F, 0x05A0, 0x2485, 0x0BBD, 
        0x220C, 0x2034, 0x16E6, 0x3292, 0x3D2C, 0x35F4, 0x26C3, 0x32C8, 0x1177, 0x2113, 0x201F, 0x0680, 0x24EA, 0x1E30, 0x32FE, 0x3CB7, 
        0x2E53, 0x128A, 0x2B9A, 0x35FC, 0x0A75, 0x36A1, 0x2AD0, 0x02C6, 0x1D9F, 0x0D85, 0x1B58, 0x266D, 0x3C1E, 0x0D4C, 0x1739, 0x2397, 
        0x3546, 0x206E, 0x24CB, 0x085F, 0x3730, 0x3E45, 0x11F1, 0x2048, 0x0B8C, 0x31DF, 0x1C4B, 0x3AB6, 0x3026, 0x2A9D, 0x2422, 0x1B6B, 
        0x185D, 0x1972, 0x0584, 0x034F, 0x1173, 0x2D85, 0x0D0D, 0x33D7, 0x3F73, 0x169C, 0x07FC, 0x3F86, 0x1822, 0x1537, 0x2998, 0x0516, 
        0x05F7, 0x1E45, 0x0CCD, 0x3EDA, 0x1DE1, 0x3BE5, 0x36D3, 0x014A, 0x13CB, 0x0CB1, 0x33D1, 0x1977, 0x07BF, 0x0494, 0x2F01, 0x3455, 
        0x1054, 0x2A8A, 0x3862, 0x3240, 0x105D, 0x1374, 0x39A1, 0x15F6, 0x1B69, 0x3977, 0x3DB3, 0x08B0, 0x063E, 0x36B1, 0x384D, 0x1817, 
        0x33B2, 0x2C17, 0x0775, 0x38A0, 0x03DC, 0x1E1A, 0x0C19, 0x24CC, 0x2FBB, 0x0560, 0x3B34, 0x3B75, 0x1C67, 0x11D9, 0x0F02, 0x3646, 
        0x1451, 0x2A37, 0x180A, 0x3328, 0x11FE, 0x2A0F, 0x35B2, 0x09CE, 0x2FD3, 0x2D74, 0x14A2, 0x0F84, 0x338E, 0x0B68, 0x2B07, 0x3D18, 
        0x322C, 0x24D6, 0x1226, 0x133B, 0x378F, 0x0DD7, 0x0360, 0x3652, 0x0AEA, 0x0D22, 0x1DCC, 0x2BE3, 0x1B42, 0x258D, 0x30A5, 0x3CCB, 
        0x2264, 0x26CD, 0x355C, 0x2E55, 0x1BFF, 0x31E2, 0x0A2C, 0x3223, 0x1169, 0x1E3E, 0x3EC8, 0x28A5, 0x2F5B, 0x0C16, 0x02E0, 0x1283, 
        0x0298, 0x2C8F, 0x0E3D, 0x3C6E, 0x3224, 0x3596, 0x3911, 0x0604, 0x244A, 0x2BD7, 0x2B37, 0x32CC, 0x0C9A, 0x0667, 0x1A9A, 0x1052, 
        0x10DD, 0x0828, 0x20D4, 0x3C02, 0x06B6, 0x20F1, 0x398B, 0x0DC3, 0x0568, 0x3E83, 0x13BC, 0x09FA, 0x2958, 0x2ECE, 0x18A0, 0x0284, 
        0x2377, 0x17D0, 0x33AB, 0x2636, 0x1326, 0x04A0, 0x1A14, 0x2CFF, 0x2527, 0x37DB, 0x31BD, 0x22C4, 0x07EC, 0x3A52, 0x0702, 0x0E26, 
        0x3DC2, 0x3C20, 0x2D0E, 0x395A, 0x2CC9, 0x248D, 0x233C, 0x35A4, 0x3DF6, 0x2898, 0x28C6, 0x314D, 0x2F12, 0x2806, 0x1980, 0x34B5, 
        0x142C, 0x1C3A, 0x280C, 0x1D3B, 0x38B5, 0x3A75, 0x2877, 0x37DA, 0x094F, 0x300D, 0x076C, 0x24A2, 0x01D3, 0x1FB4, 0x355B, 0x387A, 
        0x07CF, 0x1458, 0x3CF8, 0x2EBC, 0x3158, 0x0CD4, 0x13C0, 0x212A, 0x046A, 0x2489, 0x075C, 0x1190, 0x3AA4, 0x1BC7, 0x25AE, 0x3AEF, 
        0x16A7, 0x20E3, 0x120F, 0x35D3, 0x32CA, 0x2014, 0x04C4, 0x173C, 0x07FD, 0x0AC1, 0x253D, 0x3176, 0x158E, 0x1756, 0x39F6, 0x1907, 
        0x34BB, 0x20AF, 0x1E3F, 0x055C, 0x3603, 0x3C80, 0x2CB7, 0x34C6, 0x056A, 0x0D50, 0x1EB3, 0x13CF, 0x27B9, 0x1B9C, 0x0883, 0x19A4, 
        0x3DBD, 0x0FDB, 0x20A6, 0x1AA7, 0x345D, 0x21AA, 0x3F05, 0x2F07, 0x271F, 0x2C22, 0x3FE3, 0x07D3, 0x079C, 0x2B1F, 0x3372, 0x0DF5, 
        0x263A, 0x0B78, 0x0CF4, 0x31F1, 0x08F8, 0x14F7, 0x124C, 0x1191, 0x1529, 0x2C5F, 0x3670, 0x2537, 0x3B5D, 0x25F5, 0x0D41, 0x0146, 
        0x2E4A, 0x3FED, 0x1856, 0x311C, 0x0FB4, 0x2872, 0x1C20, 0x1DD2, 0x0352, 0x3A13, 0x0213, 0x0744, 0x3B41, 0x14EF, 0x0504, 0x07AC, 
        0x2C08, 0x3644, 0x3A59, 0x2B94, 0x0D76, 0x2808, 0x17A2, 0x2202, 0x1DA9, 0x1106, 0x2E8C, 0x09D5, 0x277A, 0x199E, 0x3189, 0x037F, 
        0x2C73, 0x2132, 0x2320, 0x227B, 0x308A, 0x1532, 0x21A5, 0x2C0F, 0x0458, 0x2A7D, 0x0B22, 0x0404, 0x179C, 0x1EB7, 0x09B2, 0x2849, 
        0x3BDB, 0x24EB, 0x2002, 0x2B69, 0x1552, 0x0526, 0x085D, 0x389D, 0x19B3, 0x25DF, 0x299D, 0x1474, 0x3CBE, 0x1F25, 0x12B5, 0x0788, 
        0x3975, 0x07ED, 0x0D71, 0x2810, 0x25A4, 0x0324, 0x05B1, 0x132E, 0x08F6, 0x0DFB, 0x1438, 0x0EA8, 0x1CBB, 0x1525, 0x1D6C, 0x3985, 
        0x08E9, 0x3125, 0x3724, 0x24EF, 0x2305, 0x127C, 0x042E, 0x3F66, 0x31CB, 0x1D18, 0x385C, 0x0190, 0x1F52, 0x1E7D, 0x1E8A, 0x060A, 
        0x2773, 0x207A, 0x1204, 0x1787, 0x2FFF, 0x1CB3, 0x2B20, 0x1FDE, 0x113D, 0x1431, 0x2D52, 0x3763, 0x3983, 0x078D, 0x0576, 0x0760, 
        0x3930, 0x157A, 0x1A86, 0x18F8, 0x0A54, 0x03C0, 0x33E7, 0x203E, 0x015D, 0x38F1, 0x3114, 0x1317, 0x1D99, 0x129C, 0x2D4E, 0x3636, 
        0x021D, 0x15B1, 0x2F76, 0x0120, 0x373F, 0x1004, 0x214F, 0x1542, 0x2E72, 0x124A, 0x0209, 0x303C, 0x231A, 0x15D8, 0x2261, 0x27D2, 
        0x3E3E, 0x12BC, 0x0A45, 0x225C, 0x13A2, 0x14C2, 0x1079, 0x30F7, 0x18A9, 0x293D, 0x242E, 0x141D, 0x3572, 0x3664, 0x3001, 0x1159, 
        0x0690, 0x275E, 0x05FB, 0x2569, 0x3BF8, 0x1288, 0x1093, 0x35C7, 0x0B1A, 0x116E, 0x1408, 0x2B8F, 0x3CDF, 0x1882, 0x2B58, 0x2B81, 
        0x1193, 0x0FE0, 0x327D, 0x3D0E, 0x2CE7, 0x39E2, 0x13F7, 0x09C0, 0x1AF1, 0x3D8C, 0x1D1A, 0x0D77, 0x3D90, 0x2D4A, 0x2BB5, 0x26A1, 
        0x2DA8, 0x3298, 0x07A7, 0x10ED, 0x3C94, 0x2B1D, 0x38EE, 0x2A97, 0x2C07, 0x25F7, 0x2CBB, 0x06B2, 0x2EEF, 0x0707, 0x3742, 0x1F27, 
        0x2075, 0x313D, 0x136D, 0x1FAC, 0x1E41, 0x2076, 0x0CA5, 0x3D31, 0x2F6F, 0x05E4, 0x3134, 0x22FA, 0x3077, 0x1DEA, 0x3E94, 0x3E41, 
        0x148B, 0x02D0, 0x1D55, 0x0320, 0x17ED, 0x3F7E, 0x08B9, 0x2D44, 0x23E9, 0x06B7, 0x140C, 0x2957, 0x3F11, 0x0C86, 0x1C1A, 0x0BFF, 
        0x0BDE, 0x136E, 0x308E, 0x1195, 0x0F8C, 0x1DEE, 0x0E7B, 0x3BD9, 0x237A, 0x35BF, 0x0A1E, 0x117C, 0x0723, 0x16FD, 0x2177, 0x2892, 
        0x20C1, 0x0DF8, 0x2566, 0x27E2, 0x19F4, 0x2D75, 0x2017, 0x297A, 0x147E, 0x175B, 0x2474, 0x1A3B, 0x36C7, 0x0CA2, 0x351D, 0x098B, 
        0x0D07, 0x08A7, 0x2698, 0x03B6, 0x3E2A, 0x3EDE, 0x26FB, 0x2272, 0x2CC4, 0x3AC5, 0x110D, 0x1FE1, 0x0AC0, 0x13E4, 0x3809, 0x2E52, 
        0x12F1, 0x0CC4, 0x36A0, 0x2EA5, 0x288A, 0x3FC4, 0x07A1, 0x1C81, 0x0703, 0x07AE, 0x3119, 0x2847, 0x1198, 0x3F12, 0x1477, 0x0F3D, 
        0x1B5C, 0x2AF5, 0x125F, 0x3A9E, 0x17AF, 0x05D1, 0x3CAE, 0x2176, 0x1B50, 0x30B9, 0x15AE, 0x2D1C, 0x2024, 0x22F2, 0x380B, 0x15B3, 
        0x3D78, 0x0891, 0x2D61, 0x251D, 0x1238, 0x31D1, 0x3CCE, 0x39CF, 0x18D5, 0x0E90, 0x2039, 0x337D, 0x38AA, 0x3A30, 0x0DC8, 0x3BDE, 
        0x21C8, 0x04F4, 0x2E39, 0x26B8, 0x3DA7, 0x143F, 0x070F, 0x28F8, 0x28E9, 0x3B5F, 0x2281, 0x3778, 0x3308, 0x21F5, 0x1301, 0x1766, 
        0x106A, 0x0D5A, 0x0AEF, 0x0C8B, 0x2CA1, 0x1A25, 0x3388, 0x02D2, 0x3B0F, 0x0D60, 0x3F0D, 0x0CF5, 0x0EDD, 0x2738, 0x0A5C, 0x0672, 
        0x13FD, 0x017E, 0x2213, 0x064D, 0x1D8C, 0x1DA4, 0x243D, 0x260C, 0x1D7A, 0x1829, 0x2A62, 0x1D6E, 0x222E, 0x29FC, 0x0D5C, 0x1A32, 
        0x26C7, 0x180D, 0x065A, 0x1C76, 0x20BD, 0x3425, 0x348B, 0x27C7, 0x3C37, 0x3828, 0x04F5, 0x337B, 0x2542, 0x2469, 0x02BF, 0x07B0, 
        0x2A55, 0x2424, 0x0947, 0x12D1, 0x3058, 0x031F, 0x1062, 0x0BBC, 0x2123, 0x0555, 0x3F46, 0x247C, 0x149C, 0x28A6, 0x17F9, 0x07F0, 
        0x372F, 0x013B, 0x396D, 0x0E8F, 0x3A74, 0x301F, 0x0C3D, 0x3BF4, 0x3A53, 0x2ECB, 0x2F70, 0x3FC0, 0x1FD2, 0x3237, 0x0EE0, 0x03E7, 
        0x1A68, 0x011D, 0x1719, 0x35F3, 0x044D, 0x3196, 0x2E97, 0x0A23, 0x3DAE, 0x2A10, 0x2455, 0x326C, 0x30ED, 0x31C4, 0x2AF1, 0x1987, 
        0x35A0, 0x2483, 0x189C, 0x1F08, 0x0949, 0x1D6A, 0x11BA, 0x3306, 0x2887, 0x102A, 0x233B, 0x182A, 0x1F98, 0x28D6, 0x373E, 0x0341, 
        0x3A9D, 0x1208, 0x10AA, 0x0896, 0x345F, 0x125A, 0x29F7, 0x36CC, 0x3F06, 0x02EF, 0x179D, 0x096A, 0x31AE, 0x182E, 0x3AD4, 0x2F4C, 
        0x29AC, 0x3C3E, 0x062D, 0x3D91, 0x0F5D, 0x0340, 0x36BC, 0x1E44, 0x21D9, 0x2D39, 0x36C1, 0x21D5, 0x3ED3, 0x2E20, 0x26FC, 0x288F, 
        0x3A57, 0x019A, 0x12DB, 0x24DA, 0x31EA, 0x22B6, 0x2655, 0x2A08, 0x399E, 0x1E69, 0x2E1B, 0x2CC1, 0x16CC, 0x1FFD, 0x3A08, 0x0992, 
        0x2727, 0x1DE5, 0x3FAA, 0x09F2, 0x2A9C, 0x01A6, 0x176D, 0x31FF, 0x343D, 0x11B1, 0x3BFB, 0x2E4C, 0x3004, 0x17E7, 0x2700, 0x0B0F, 
        0x369D, 0x1FB7, 0x0285, 0x3F98, 0x1A3D, 0x0C50, 0x2846, 0x0F67, 0x367C, 0x31F3, 0x35FF, 0x2BFF, 0x0C72, 0x0333, 0x0941, 0x325B, 
        0x2A4E, 0x0FC9, 0x046B, 0x3941, 0x3D1F, 0x0887, 0x3A49, 0x2FB8, 0x3473, 0x1F79, 0x0A7A, 0x268E, 0x33A2, 0x21BD, 0x1008, 0x0347, 
        0x35AC, 0x1710, 0x218F, 0x2B9D, 0x1441, 0x1D80, 0x0F28, 0x1025, 0x2A4B, 0x35EE, 0x14DA, 0x3F37, 0x1B86, 0x314C, 0x110C, 0x2137, 
        0x39AA, 0x0924, 0x0E0C, 0x1751, 0x0978, 0x13AE, 0x2BC9, 0x3325, 0x1294, 0x163C, 0x39CC, 0x0CA0, 0x0757, 0x29FD, 0x18B9, 0x0A9C, 
        0x2CF1, 0x37CE, 0x0745, 0x3AA2, 0x3CA5, 0x3B06, 0x3EB4, 0x3A3E, 0x2E00, 0x2724, 0x2054, 0x3D5C, 0x2D91, 0x169A, 0x25A7, 0x3CB1, 
        0x2664, 0x29AD, 0x10A1, 0x31A2, 0x1EA2, 0x3504, 0x139B, 0x0A86, 0x1C9F, 0x3D9A, 0x2702, 0x1750, 0x1B41, 0x3371, 0x3FF0, 0x2E2B, 
        0x3B35, 0x194E, 0x1F8E, 0x038C, 0x0AC4, 0x2A81, 0x3AAE, 0x3020, 0x354F, 0x0661, 0x096D, 0x36F0, 0x3377, 0x3F9A, 0x2B68, 0x2E5B, 
        0x2FFD, 0x1EEC, 0x1F8D, 0x016E, 0x04E5, 0x132D, 0x0571, 0x13FA, 0x0DDB, 0x328E ]
